#!/usr/bin/perl
#
# $Header: //sapdb/V76/c_00/b_32/sys/src/install/perl/SAPDB/Install/App/Reg.pm#1 $
# $DateTime: 2006/07/06 16:50:32 $
# $Change: 130190 $
#
# Desc: application to call register method of each installed MaxDB package
#		this is useful if registrations has been deleted




package SAPDB::Install::App::Reg;

use SDB::Common::Getopt::Long;
use SAPDB::Install::System;


sub print_usage{
	require SAPDB::Install::Version;
	print "\nMaxDB register / unregister tool\nVersion: ".$SAPDB::Install::Version::Buildstring."\n";
	print $SAPDB::Install::Version::Copyright."\n\n";
	
	print "usage: sdbreg [-h] [-logfile <logfile>] [-d <db name>]\n".
		  "       [-u [-sw_all_kernel | -sw_all | -sw_kernel]]\n".
		  "       [-r [-R <dependent path>] [-sw_client | -sw_all | -sw_kernel]\n".
		  "       [-start_xserver] [-start_db]]".($^O =~ /mswin/i ? ' [-substitute_drive]' : '')."\n\n";
	print "  -u                  unregister database and/or database software\n\n";
	print "  -r                  register database and/or database software\n\n";
	print "  -d <db name>        register / unregister database <db name>\n\n";
	print "  -R <dependent path> register database <db name> with kernel \n".
		  "                      software in <dependent path>\n\n";
	print "  -logfile <logfile>  write logfile <logfile>, default is no logfile\n\n";
	print "  -sw_kernel          register / unregister kernel software of\n".
	      "                      database <db name>\n\n";
	print "  -sw_all_kernel      unregister all existing database kernel software\n\n";
	print "  -sw_all             register / unregister all existing MaxDB software\n\n";
	print "  -sw_client          register all existing MaxDB client software\n\n";
	print "  -start_xserver      start MaxDB communication service (x_server) after\n".
	      "                      successful software registration\n\n";
	print "  -start_db           start database <db name> after successful registration\n\n";
	print "  -substitute_drive   installation drive letter differs from original system\n\n"
		if($^O =~ /mswin/i);
	return 1;
}

sub main{
	local @ARGV = @_;
	my $rc = 0;
	my $opt_unregister,$opt_help;
	
	unless(IsAdmin()){
		print STDERR "please restart as ".($^O =~ /mswin/i ? 'local administrator' : 'root')."!\n";
		return -1;
	}
	my %opt_ctrl = (
		'u' => \$opt_unregister,
		'r' => \$opt_register,
		'h' => \$opt_help,
		'help' => \$opt_help
	);	
	
	SDB::Common::Getopt::Long::Configure ('pass_through','no_ignore_case');	
	
	GetOptions(%opt_ctrl);
		
	if ($opt_help){
		print_usage();
		return 0;
	}	
		
	SDB::Common::Getopt::Long::Configure ('no_pass_through','no_ignore_case');
	
	if($opt_unregister && $opt_register){
		print "wrong usage: cannot use options -r and -u at the same time\n";
		print_usage();
		return -1;
	}
	
	if ($opt_unregister){
		require SAPDB::Install::App::UnregisterPackages;
		$rc = SAPDB::Install::App::UnregisterPackages::main(@ARGV);
		if ($rc == 42){
			print_usage();
			return -1;
		} 	
	}
	elsif ($opt_register){
		require SAPDB::Install::App::RegisterPackages;
		$rc = SAPDB::Install::App::RegisterPackages::main(@ARGV);
		if ($rc == 42){
			print_usage();
			return -1;
		} 
	}
	else{
		print "wrong usage: need -r or -u option \n";
		print_usage();
		return -1;
	}
	return $rc;
}

1;
