#!/usr/bin/perl
#
# $Header: //sapdb/V76/c_00/b_32/sys/src/install/perl/SAPDB/Install/BuildPackage.pm#1 $
# $DateTime: 2006/07/06 16:50:32 $
# $Change: 130190 $
#
# Desc: create MaxDB / SAP DB installation packages


package SAPDB::Install::BuildPackage;

require SDB::Common::Exporter;
our @EXPORT = qw (main);

use SDB::Common::Getopt::Long;
use SAPDB::Install::Values;
use SAPDB::Install::Misc;
use SAPDB::Install::System;
use SAPDB::Install::StdIO;
use SAPDB::Install::Version;
use SAPDB::Install::MD5Sum;
use SAPDB::Install::SetDebug;
use SAPDB::Install::Tools;
use SAPDB::Install::SigHandler;
use SAPDB::Install::GnuTar;
use SAPDB::Install::Term::ReadKey;

our $DEBUG = 0;
	
sub checkOpts{
	my $rc = 1;
	if($opt_backup){
		unless(-f $opt_backup){
			print2stderr("wrong usage: -backup: $!\n");
			$rc = 0;
		}  
	}
	else{
		print2stderr("wrong usage: \"-backup\" switch required\n");
		$rc = 0;
	}
	if($opt_package){
		unless($opt_package eq 'DB Instance'){
			print2stderr("wrong usage: -package: package \"$opt_package\" unknown\n");
			$rc = 0;
		
		}			
	}
	else{
		print2stderr("wrong usage: \"-package\" switch required\n");
		$rc = 0;
	}

	if($opt_dir){
		unless(-d $opt_dir){
			print2stderr("wrong usage: -dir: $!\n");
			$rc = 0;
		} 
	}
	if( $opt_archivename=~ /\S/ ){
		unless($opt_archivename =~ /^IA[A-Z0-9]{0,6}\.TGZ$/){
			print2stderr("wrong usage: -archivename: archive name need conventions IA<your string>.TGZ,\n".
										"\t\twith <your string> max length of 6 characters and\n".
										"\t\tcharacter set A-Z 0-9\n");
			$rc=0;
		}
	}

			
	return $rc;
}


sub getPackageFiles{
	my ($package) = @_;

	my %packdefs = (
						'DB Instance' => {	
											'subdir' => 'instance'
						}
	);

	my %returnvalue;
	
	eval{
		require "SAPDB::Install::BuildPackage::packages::".$packdefs{$opt_package}->{'subdir'};
	};
	
	if($@){
		print2stderr("cannot load $package package definition: $@\n");
		return undef;
	}

	$returnvalue{'packagedata'} = ${'SAPDB::Install::BuildPackage::packages::'.$packdefs{$opt_package}->{'subdir'}.'::packagedata'};
	$returnvalue{'script'} = ${'SAPDB::Install::BuildPackage::packages::'.$packdefs{$opt_package}->{'subdir'}.'::script'};
	$returnvalue{'scriptname'} = ${'SAPDB::Install::BuildPackage::packages::'.$packdefs{$opt_package}->{'subdir'}.'::script_name'};
	return \%returnvalue;
}



sub main{
	local (@ARGV) = @_;

	SetDebug(\@ARGV);
	if (printVersion(\@ARGV)){
		return 0;
	}

	%opt_ctrl=(
		'h' => \$opt_help,
		'help' => \$opt_help,
		'package=s' => \$opt_package,
		'dir=s' => \$opt_dir,
		'backup=s' => \$opt_backup,
		'archivename=s' => \$opt_archivename
	);	

	$opt_package = 'DB Instance'; # default
	
	my $usage= " [-h|--help] | [-v|--version] | --backup <backup file>\n".
				"\t[-package <package name>] [-dir <package directory>] [-archivename IA<your string>.TGZ]\n"; 
	my $usage_desc =	"\t\t-h --help\t\t\tshow this\n".
				"\t\t-v --version\t\t\tshow version\n".
				"\t\t-package <package name>\n".
				"\t\t-dir <package directory>\tcreate package in <package directory>\n".
				"\t\t-backup <backup file>\n".
				"\t\t-archivename IA<your string>.TGZ\t create archive with this name\n";	
							
	chdir($SAPDB::Install::Config{'CallerDir'});
	SDB::Common::Getopt::Long::Configure ('no_pass_through');					
	if(!GetOptions(%opt_ctrl) || $opt_help || !checkOpts()){
		print ("\n\nusage: ".$SAPDB::Install::Config{'ProgramName'}."$usage \n\n $usage_desc\n\n\n");
		if($SAPDB::Install::Config{'HasOwnConsole'}){
			print2stdout("\npress any key...");
			SAPDB::Install::Term::ReadKey::ReadMode('raw');
			SAPDB::Install::Term::ReadKey::ReadKey();
		}
		die("\n\n\n");
	}
 	
	$SAPDB::Install::Values::diemsg = 'MaxDB buildpackage tool exited abnormally';
	setSigHandler();
	my $archive_name = 'IASDB.TGZ';
	if($opt_archivename =~ /\S/){
		$archive_name = $opt_archivename;
	}
	
	
	my $archive = $opt_dir ? "$opt_dir/$archive_name" : $archive_name;
	if(-f $archive){
		print2stderr("achive $archive already exists\n");
		diesoft($diemsg);		
	}
	
	my $packagefiles = getPackageFiles($opt_package);

	unless (exists $packagefiles->{'packagedata'}){
		print2stderr("cannot get PACKAGEDATA of package $package\n");
		diesoft($diemsg);
	}
	
	unless (exists $packagefiles->{'script'}){
		print2stderr("cannot get script of package $package\n");
		diesoft($diemsg);
	}
		
	my $gtar = SAPDB::Install::GnuTar->new();
	$gtar->OpenArchive($archive,'w9');
	print2stdout("adding PACKAGEDATA...\n");
	$gtar->AddScalar("PACKAGEDATA",$packagefiles->{'packagedata'});
	print2stdout("adding script...\n");
	$gtar->AddScalar($packagefiles->{'scriptname'},$packagefiles->{'script'});
	print2stdout("adding backup...\n");
	$gtar->Add($opt_backup,'BACKUP');
	$gtar->CloseArchive($archive);
	print2stdout("instance archive \"$archive\" packed successfully!\n");
	return 0;
}

1;