#!/usr/bin/perl
#
# $Header: //sapdb/V76/c_00/b_32/sys/src/install/perl/SAPDB/Install/Frame/Explore.pm#1 $
# $DateTime: 2006/07/06 16:50:32 $
# $Change: 130190 $
#
# Desc: start SAP DB installation/update/patch/instanceupgrade, called by SDBINST/SDBUPD

package SAPDB::Install::Frame::Explore;

use Wx::Event qw (
	EVT_MENU EVT_SIZE EVT_SPLITTER_SASH_POS_CHANGED
	EVT_TREE_SEL_CHANGED);

use Wx qw (
		 wxTE_READONLY wxTE_MULTILINE
		 wxVERTICAL wxEXPAND);

use SAPDB::Install::Dialog::About;
use SAPDB::Install::InstallRegistry;


use SAPDB::Install::Uninstall::RemoveFilesInUse;

use base SAPDB::Install::Frame::Base;

sub RefHasMaxLayer{
	my ($ref,$layer) = @_;
	my $tlayer = $layer - 1;
	if($layer == 0){
		return 0;
	}
	#unless(defined $ref){
	#	return 0;
	#}
	
	unless(ref($ref)){
		return 1;
	}


	if(ref($ref) eq 'SCALAR'){
		return RefHasMaxLayer($$ref,$tlayer);
	}
	elsif(ref($ref) eq 'ARRAY'){
		return 0 if $#{@$ref} == -1;
		foreach my $mem (@$ref){
			my $rc = RefHasMaxLayer($mem,$tlayer);
			unless($rc){
				return 0;
			}
		}
		return 1;
	}
	elsif(ref($ref) eq 'HASH'){
		foreach my $key (keys(%$ref)){
			my $rc = RefHasMaxLayer($ref->{$key},$tlayer);
			unless($rc){
				return 0;
			}
		}
		return 1;
	}
	else{
		return 1;
	}
}


sub new {
 	my( $class ) = shift;
 	my( $ID_ABOUT, $ID_EXIT ) = ( 1, 2 );
	my( $this ) = $class->SUPER::new($_[0], $_[1], $_[2] );
	$this->{'splitter'} = Wx::SplitterWindow->new($this,-1,[0,0],$_[2]);
	my $menubar = Wx::MenuBar->new();
	my $file_menu = Wx::Menu->new();
	$file_menu->Append($ID_EXIT,'&Exit');
	$menubar->Append($file_menu,'&Registry');
	my $help_menu = Wx::Menu->new();
	$help_menu->Append($ID_ABOUT,"&About");
	$menubar->Append($help_menu,'&Help');
	$this->SetMenuBar($menubar);
	my $keywin = Wx::ScrolledWindow->new($this->{'splitter'},-1,[0,0],[10,10]);
	my $key_sizer = Wx::BoxSizer->new(wxVERTICAL);
	my $value_sizer = Wx::BoxSizer->new(wxVERTICAL);
	my $valuewin = Wx::ScrolledWindow->new($this->{'splitter'},-1,[0,0],[10,10]);
	$this->{'keywin'} = $keywin;
	$this->{'valuewin'} = $valuewin;
	$this->{'treectrl'} = Wx::TreeCtrl->new($keywin,-1,[0,0],[10,10]);
	$key_sizer->Add($this->{'treectrl'},1,wxEXPAND);
	$keywin->SetSizer($key_sizer);
	$keywin->SetAutoLayout(1);
	$this->{'textctrl'} = 	Wx::TextCtrl->new($valuewin,-1,'',[0,0],$valuewin->GetClientSize,wxTE_READONLY | wxTE_MULTILINE);
	$value_sizer->Add($this->{'textctrl'},1,wxEXPAND);
	$valuewin->SetSizer($value_sizer);
	$valuewin->SetAutoLayout(1);
	$this->{'splitter'}->SplitVertically($keywin,$valuewin,$_[2]->GetWidth - abs($_[2]->GetWidth * 0.70));
	EVT_SIZE($this,\&OnSize);
	EVT_SPLITTER_SASH_POS_CHANGED($this,$this->{'splitter'},\&OnSize);
	EVT_TREE_SEL_CHANGED($this,$this->{'treectrl'},\&OnItemSelect);
	EVT_MENU($this,$ID_EXIT,\&OnExit);
	EVT_MENU($this,$ID_ABOUT,\&OnAbout);
	$this->OnSize;
	$this->{'maxrefdepth'} = 1;
	$this->Show(1);
    $this;
}

sub OnSize{
	my ($self,$event) = @_;
	$self->{'splitter'}->SetSize($self->GetClientSize);
	$self->{'keywin'}->SetSize([$self->{splitter}->GetSashPosition,
					    $self->{'splitter'}->GetClientSize->GetHeight]);	
	$self->{'valuewin'}->SetSize([$self->{'splitter'}->GetClientSize->GetWidth - 
						$self->{splitter}->GetSashPosition - 5,
						$self->{'splitter'}->GetClientSize->GetHeight]);	
}

sub OnItemSelect{
	my ($self,$event) = @_;
	my $item = $event->GetItem;
	my $data = $self->{'treectrl'}->GetPlData($item);
	if(ref($data) eq 'SCALAR'){
		$self->{'textctrl'}->SetValue($$data);
	}
	elsif(ref($data) eq 'ARRAY'){
		$self->{'textctrl'}->SetValue(join("\n",@$data));
	}
	elsif(ref($data) eq 'HASH'){
		my $txt;
		foreach my $key (sort keys(%$data)){
			$txt .= $key.' = '.$data->{$key}."\n"; 
		}
		$self->{'textctrl'}->SetValue($txt);
	}
	else{
		$self->{'textctrl'}->SetValue($data);
	}
}


sub AddData{
	my ($self,$data,$root_name) = @_;
	unless(ref($data)){
		return undef;
	}
	if(ref($data) eq 'HASH'){
		my @keys = keys(%$data);
		if(defined $self->{'cur_node'}){
			if($#keys > 0){
				my $node = $self->{'cur_node'};
				my %flat,$flat_found = 0;
				foreach $key (sort @keys){
					if( not RefHasMaxLayer($data->{$key},$self->{'maxrefdepth'})){
						my $key_name = $key;
						$key_name = 'Paths' if ($key eq 'Pathes');
						$self->{'cur_node'} = $self->{'treectrl'}->AppendItem($self->{'cur_node'},$key_name,-1,-1);
						$self->AddData($data->{$key});
						$self->{'cur_node'} = $node;	
					}
					else{
						$flat{$key} = $data->{$key};
						$flat_found = 1;
						#$self->{'treectrl'}->SetPlData($self->{'cur_node'},$data->{$key});
					}
				}
				if($flat_found){
					$self->{'treectrl'}->SetPlData($self->{'cur_node'},\%flat);
				}
				
			}
			else{
				if(not RefHasMaxLayer($data->{$keys[0]},1)){
					$self->{'cur_node'} = $self->{'treectrl'}->AppendItem($self->{'cur_node'},$keys[0],-1,-1);
					$self->AddData($data->{$keys[0]});
				}
				else{
					$self->{'treectrl'}->SetPlData($self->{'cur_node'},$data);
				}

				
			}
		}
		else{
			if($#keys > 0){
				$self->{'cur_node'} = $self->{'treectrl'}->AddRoot($root_name =~ /\S/ ? $root_name : 'root',-1,-1);
				my %flat,$flat_found = 0;
				foreach $key (sort @keys){
					my $node = $self->{'cur_node'};
					my $key_name = $key;
					$key_name = 'Software Packages' if ($key eq 'HashRef_Packages');
					$key_name = 'Databases in Update' if ($key eq 'InstanceData');
					if(not RefHasMaxLayer($data->{$key},$self->{'maxrefdepth'})){
						$self->{'cur_node'} = $self->{'treectrl'}->AppendItem($self->{'cur_node'},$key_name,-1,-1);
						$self->AddData($data->{$key});
						$self->{'cur_node'} = $node;
					}
					else{
						$flat{$key} = $data->{$key};
						$flat_found = 1;

						#$self->{'treectrl'}->SetPlData($self->{'cur_node'},$data->{$key});
					}
				}
				if($flat_found){
					$self->{'treectrl'}->SetPlData($self->{'cur_node'},\%flat);
				}
			}
			else{
				$self->{'cur_node'} = $self->{'treectrl'}->AddRoot($keys[0],-1,-1);
			}
			$self->{'treectrl'}->SelectItem($self->{'cur_node'});
			$self->{'treectrl'}->Expand($self->{'cur_node'});
		}
	}
	elsif(ref($data) eq 'ARRAY'){
		my $node = $self->{'cur_node'};
		my $counter = 0;
		my @flat,$flat_found = 0;
		foreach my $mem (@$data){
			if(not RefHasMaxLayer($mem,$self->{'maxrefdepth'})){
				$self->{'cur_node'} = $self->{'treectrl'}->AppendItem($self->{'cur_node'},$counter++,-1,-1);
				$self->AddData($mem);		
				$self->{'cur_node'} = $node;
			}
			else{
				push @flat,$mem;
				$flat_found = 1;
			}
			if($flat_found){
				$self->{'treectrl'}->SetPlData($self->{'cur_node'},\@flat);
			}
		}
	}
	elsif(ref($data) eq 'SCALAR'){
		$self->{'treectrl'}->SetPlData($self->{'cur_node'},$data);
	}
	else{
		return undef;
	}	
	
	return 1;
}

sub OnExit{
	my ($self,$event) = @_;
	$self->Destroy;
}

sub OnAbout{
	my ($self,$event) = @_;
	my $about = SAPDB::Install::Dialog::About->new($self,-1);

}

sub Destroy{
	my ($self) = @_;
	while($self->{'keywin'}->GetSizer->Remove(0)) {}
	while($self->{'valuewin'}->GetSizer->Remove(0)) {}
	$self->{'treectrl'}->Destroy;
	delete $self->{'treectrl'};
	$self->{'textctrl'}->Destroy;
	delete $self->{'textctrl'};
	$self->{'keywin'}->Destroy;
	delete $self->{'keywin'};
	$self->{'valuewin'}->Destroy;
	delete $self->{'valuewin'};
	$self->{'splitter'}->Destroy;
	delete $self->{'splitter'};
	$self->SUPER::Destroy;
}

sub DESTROY{
	my ($self) =  @_;
	print ">>>> DESTROY $self\n"
		if $SAPDB::Install::DEBUG::DESTRUCT;
}

1;
