#!/usr/bin/perl
#
# $Header: //sapdb/V76/c_00/b_32/sys/src/install/perl/SAPDB/Install/Grid/Base.pm#1 $
# $DateTime: 2006/07/06 16:50:32 $
# $Change: 130190 $
#
# Desc: Base Grid Class 


package SAPDB::Install::Grid::Base;


use Wx::Event qw(EVT_GRID_CELL_CHANGE EVT_KEY_DOWN
				EVT_GRID_SELECT_CELL EVT_KILL_FOCUS
				EVT_SET_FOCUS EVT_GRID_CELL_LEFT_CLICK);
use base Wx::Grid;
use Wx qw(wxSUNKEN_BORDER WXK_DELETE wxWHITE wxSYS_COLOUR_WINDOWTEXT
		  WXK_TAB wxSYS_COLOUR_GRAYTEXT wxBLACK WXK_SHIFT wxEVT_CHAR) ;

our $disable_colour = Wx::SystemSettings::GetColour(wxSYS_COLOUR_GRAYTEXT); 
our	$enable_colour = Wx::SystemSettings::GetColour(wxSYS_COLOUR_WINDOWTEXT);

our $labelfont_size;


sub new{
	my $class = shift;
	my $self = $class->SUPER::new(@_);
	$self->SetWindowStyleFlag(wxSUNKEN_BORDER);
	bless($self,$class);
	$self->SetRowLabelSize(0);
	#$self->SetDefaultCellBackgroundColour($self->GetParent->GetBackgroundColour);
	#$self->SetBackgroundColour(wxWHITE);
	unless(defined $label_fontsize){
		my $memdc = Wx::MemoryDC->new();
		$memdc->SelectObject(Wx::Bitmap->new(1,1))
			if ($^O !~ /mswin/i);
		my ($x,$y) = $memdc->GetTextExtent('test',$self->GetLabelFont);
		$label_fontsize = $y;
	}
	$self->SetColLabelSize($label_fontsize + 6);
	
	EVT_GRID_CELL_LEFT_CLICK($self,\&OnClick);
	EVT_KEY_DOWN($self,\&OnKeyDown);
	#EVT_SET_FOCUS($self->GetGridWindow,\&OnSetFocus);
	return $self;
}


sub FindNextEditable{
	my ($self,$backward) = @_;
	my $col = $self->GetGridCursorCol;
	my $maxcol = $self->GetNumberCols - 1;
	my $row = $self->GetGridCursorRow;
	my $maxrow = $self->GetNumberRows - 1;
	if($row == 0 and $col == ($maxcol + 1)){
			unless($self->IsReadOnly(0,0)){
				$self->SetGridCursor(0,0);
				$self->EnableCellEditControl(1);
				return 1;	
			}
			$self->SetGridCursor(0,0);	
			$row = 0;
			$col = 0;
	}
	
	my $new_col,$new_row;
	if($backward){
		if($col == 0 and $row == 0){
			$self->SetGridCursor(0,$maxcol + 1 );
			return undef;
		}
		if($col > 0){
			 $new_row = $row;
			 $new_col = $col - 1;
		}
		elsif($row > 0){
			 $new_row = $row - 1;
			 $new_col = $maxcol;
		}
		$self->SetGridCursor($new_row,$new_col);
		unless($self->IsReadOnly($new_row,$new_col)){
			$self->SetGridCursor(0,$maxcol + 1 );
			return 1;
		}
	}
	else{
		if($col == $maxcol and $row == $maxrow){
			$self->SetGridCursor(0,$maxcol + 1 );
			my $self_found = 0;
			my $focus_set = 0;
			foreach my $child ($self->GetParent->GetChildren){
				if($child->GetId == $self->GetId){
					$self_found = 1;
					next;
				}
				next unless $self_found;
				next if $child =~ /static/i;
					$focus_set = 1;
					$child->SetFocus;
					last;
			}
				
			my $id_found = 0;

			unless($focus_set){
				foreach my $child ($self->GetParent->GetParent->GetChildren){
					next if ($child == $self->GetParent);
					next unless $child->IsShown;
					next unless $child->IsEnabled;
					next if $child =~ /static/i;
					$focus_set = 1;
					$child->SetFocus;
					last;
				}
			}
			return undef;
		}
		if($col < $maxcol){
			 $new_row = $row;
			 $new_col = $col + 1;
		}
		elsif($row < $maxrow){
			 $new_row = $row + 1;
			 $new_col = 0;
		}
		$self->SetGridCursor($new_row,$new_col);
		unless($self->IsReadOnly($new_row,$new_col)){
			return 1;
		}
	}
	return $self->FindNextEditable($backward);
}

sub OnKeyDown{
	my ($self,$evt) = @_;
	if($self->IsEditable and $evt->GetKeyCode == WXK_TAB){
		$self->DisableAllEditors;
		my $parent = $self->GetGridWindow;
		if($evt->ShiftDown){
			if( defined $self->FindNextEditable(1)){
				$self->EnableCellEditControl(1);
			}
			else{	
				$parent->GetParent->SetFocus();
			}
		}
		else{
			if( defined $self->FindNextEditable()){
				$self->EnableCellEditControl(1);
			}
		}		
	}
	elsif($evt->GetKeyCode == WXK_SHIFT){
	
	}
	else{
		$evt->Skip(1);
	}
}

sub OnClick{
	my ($self,$evt) = @_;
	my $row = $evt->GetRow();
	my $col = $evt->GetCol();
	$self->DisableAllEditors;
	if($self->IsReadOnly($row,$col)){
		$evt->Skip(1);	
	}
	else{
		$self->SetGridCursor($row,$col);
		$self->EnableCellEditControl(1);	
	}

}

sub OnKillFocus{
	my ($self,$evt) = @_;
	#$evt->Skip(1);
}

sub DisableAllEditors{
	my ($self) = @_;
	my $save_row = $self->GetGridCursorRow;
	my $save_col = $self->GetGridCursorCol;
	foreach my $col (0 .. ($self->GetNumberCols - 1)){
		foreach my $row (0 .. ($self->GetNumberRows - 1)){
			$self->SetGridCursor($row,$col);
			if($self->IsCellEditControlEnabled){
				$self->SaveEditControlValue;
				$self->EnableCellEditControl(0);
			}
		}
	}
	$self->SetGridCursor($save_row,$save_col);
	return 1;
}


sub OnSetFocus{
	my ($grid_window,$evt) = @_;
	my $self = $grid_window->GetParent;
	bless $self,'SAPDB::Install::Grid::Base'; 
	$self->FindNextEditable;
	#$self->DisableAllEditors;
	#$self->EnableCellEditControl(1);
	#$evt->Skip(1);
}


sub Fit{
	my ($self,$columns_def) = @_;
	$self->SUPER::Fit;
	if(defined $columns_def){
		my $col = 0;
		foreach my $column (@$columns_def){
			if($column->{'invisible'}){
				$self->SetColSize($col,0);		
			}
			$col++;
		}
	}
	return 1;
}


sub init{
	my ($self,$columns_def,$ref) = @_;
	my @columns = @{$columns_def};
	return undef unless(ref($ref) eq 'ARRAY');	
	my $rows = $#{@$ref} + 1;
	
	$self->CreateGrid($rows,$#columns + 1);
	my $col = 0;
	foreach my $column (@columns){ 
		$self->SetColLabelValue($col++,$column->{'label'});
	}
	my $row = 0;
	foreach my $element (@$ref){
		$col = 0;
		foreach my $column (@columns){ 
			if($column->{'readonly'}){
				$self->SetReadOnly( $row, $col );	
				$self->SetCellTextColour($row,$col, $disable_colour);
			}
			else{
				#$self->SetCellBackgroundColour($row,$col, wxWHITE);
			}
			if($column->{'invisible'}){
				$self->SetColSize($col,0);		
			}
			$self->SetCellValue( $row, $col,$element->{$column->{'struct_key'}} );	
			if($column->{'type'} eq 'voltype'){
				require SAPDB::Install::Grid::CellVolTypeEditor;
				$self->SetCellEditor($row,$col,
					SAPDB::Install::Grid::CellVolTypeEditor->new());
			}
			elsif($column->{'type'} eq 'size_pages'){
				my $value = $element->{$column->{'struct_key'}};
				$value = int($value * 8 / 1024);
				$self->SetCellValue( $row, $col,$value);
				require SAPDB::Install::Grid::CellSizeEditor;
				$self->SetCellEditor($row,$col,
					SAPDB::Install::Grid::CellSizeEditor->new());	
			}
			elsif($column->{'type'} eq 'password'){
				require SAPDB::Install::Grid::CellPasswdEditor; 
				$self->SetCellEditor($row,$col,
					SAPDB::Install::Grid::CellPasswdEditor->new());
				require SAPDB::Install::Grid::CellPasswdRenderer; 
				$self->SetCellRenderer($row,$col,
					SAPDB::Install::Grid::CellPasswdRenderer->new());
			}
			elsif($column->{'type'} eq 'drive'){
				require SAPDB::Install::Grid::CellDriveEditor; 
				$self->SetCellEditor($row,$col,
					SAPDB::Install::Grid::CellDriveEditor->new());
			}
			elsif($column->{'type'} eq 'path'){
				require SAPDB::Install::Grid::CellPathEditor; 
				$self->SetCellEditor($row,$col,
					SAPDB::Install::Grid::CellPathEditor->new());
			}
			$col++;
		}
		$row++;
	}
	$self->SetGridCursor(0,$col);
	$self->Fit;
	return 1;
}


sub GetValue{
	my ($self,$columns_def) = @_;
	my @columns = @$columns_def; 
	my @ref;
	$self->DisableAllEditors;
	my $primary_col = 0;
	my $col = 0;

	foreach my $column (@columns){ 
		if($column->{'is_primary'}){
			$primary_col = $col++;
		}
	}

	foreach my $row (0 .. ($self->GetNumberRows - 1)){
		$col = 0;
		if($row == ($self->GetNumberRows - 1) and 
			$self->GetCellValue( $row, $primary_col) !~ /\S/){
			last;
		}
		my $element = {};
		foreach my $column (@columns){ 
			if($column->{'type'} eq 'size_pages'){
				my $value = $self->GetCellValue( $row, $col);
				$value = int ($value * 1024 / 8);
				$element->{$column->{'struct_key'}} = $value;
				
			}
			else{
				$element->{$column->{'struct_key'}} = $self->GetCellValue( $row, $col);	
			}
			$col++;
		}
		push @ref,$element;
	}
	return \@ref; 
}


sub InitRow{
	my ($self,$row,$columns_def) = @_;
	
	my @columns = @$columns_def;
	
	if($row >= $self->GetNumberRows){
		return undef;
	}

	my $primary_col = 0;
	my $col = 0;
	foreach my $column (@columns){ 
		if($column->{'is_primary'}){
			$primary_col = $col++;
		}
	}

	$self->DisableAllEditors;

	$col = 0;
	foreach my $column (@columns){
		
		if($column->{'readonly'}){
			$self->SetCellTextColour($row,$col, $disable_colour);
			$self->SetReadOnly( $row, $col );	
		}
		else{
			$self->SetCellTextColour($row,$col, $enable_colour);
			#$self->SetCellBackgroundColour($row,$col, wxWHITE);
			$self->SetReadOnly( $row, $col,0);	
		}	
		
		if($column->{'type'} eq 'voltype'){
			require SAPDB::Install::Grid::CellVolTypeEditor;
			$self->SetCellEditor($row,$col,
				SAPDB::Install::Grid::CellVolTypeEditor->new());
		}
		elsif($column->{'type'} eq 'size_pages'){
			my $value = $element->{$column->{'struct_key'}};
			$value = int($value * 8 / 1024);
			$self->SetCellValue( $row, $col,$value);
			require SAPDB::Install::Grid::CellSizeEditor;
			$self->SetCellEditor($row,$col,
				SAPDB::Install::Grid::CellSizeEditor->new());	
		}
		elsif($column->{'type'} eq 'password'){
			require SAPDB::Install::Grid::CellPasswdEditor; 
			$self->SetCellEditor($row,$col,
				SAPDB::Install::Grid::CellPasswdEditor->new());
			require SAPDB::Install::Grid::CellPasswdRenderer; 
			$self->SetCellRenderer($row,$col,
				SAPDB::Install::Grid::CellPasswdRenderer->new());
		}
		elsif($column->{'type'} eq 'drive'){
			require SAPDB::Install::Grid::CellDriveEditor; 
			$self->SetCellEditor($row,$col,
				SAPDB::Install::Grid::CellDriveEditor->new());
		}
		elsif($column->{'type'} eq 'path'){
			require SAPDB::Install::Grid::CellPathEditor; 
			$self->SetCellEditor($row,$col,
			SAPDB::Install::Grid::CellPathEditor->new());
		}
		
		if($col == $primary_col or not $column->{'init_new'}){
			$col++;
			next;
		}
		if($row > 0){
			$self->SetCellValue( $row, $col,$self->GetCellValue($row - 1,$col));	
		}	
		$col++;
	}
	
	$self->AppendEmptyRow($columns_def);
	return 1;
}


sub AppendEmptyRow{
	my ($self,$columns_def) = @_;
	$self->SetRowLabelSize(20);
	$self->AppendRows(1);
	my $col = 0;
	my $last_row = $self->GetNumberRows - 1; 
	foreach my $column (@$columns_def){
		if($col == $primary_col){
			#$self->SetCellBackgroundColour($last_row,$col, wxWHITE);
			$self->SetReadOnly($last_row,$col++,0);
			next;				
		}
		$self->SetCellTextColour($last_row,$col, $disable_colour);
		$self->SetReadOnly($last_row,$col++ );			
	}
	return 1;
}

sub EnableAddRemoveRows{
	my ($self,$columns_def) = @_;
	
	$self->AppendEmptyRow($columns_def);
	
	my @columns = @$columns_def;
	my $primary_col = 0;
	my $col = 0;
	
	foreach my $column (@columns){ 
		if($column->{'is_primary'}){
			$primary_col = $col++;
		}
	}

	EVT_GRID_CELL_CHANGE($self,
		sub {
			my ($self,$evt) = @_;
			if($evt->GetRow == ($self->GetNumberRows - 1) && $evt->GetCol == $primary_col){
				$self->InitRow($evt->GetRow,$columns_def);
			}			
			$evt->Skip(1);
		}
	);
	EVT_KEY_DOWN($self,
		sub {
			my ($self,$evt) = @_;
			if($evt->GetKeyCode == WXK_DELETE){
				my @selected_rows = ($self->GetSelectedRows);
				foreach my $row (sort {$b <=> $a} @selected_rows){
					next if($row == ($self->GetNumberRows - 1));
					$self->DeleteRows($row,1);
				}
			}
			else{
				$self->OnKeyDown($evt);
			}
		}
	);
	return 1;
}


sub Destroy{
	my ($self) = @_;
	foreach my $row (0 .. ($self->GetNumberRows() -1 )){
		foreach my $col (0 .. ($self->GetNumberCols() -1 )){
			my $editor = $self->GetCellEditor($row,$col);
			$editor->Destroy() if defined $editor;
		}
	}
	$self->SUPER::Destroy;
}

1;
