#!/usr/bin/perl
#
# $Header: //sapdb/V76/c_00/b_32/sys/src/install/perl/SAPDB/Install/Instance/Check.pm#1 $
# $DateTime: 2006/07/06 16:50:32 $
# $Change: 130190 $
#

package SAPDB::Install::Instance::Check;

use SAPDB::Install::Instance::Base;
use SAPDB::Install::Instance::Check::Conditions;
use SAPDB::Install::Instance::Check::DataCons;
use SAPDB::Install::Instance::Check::DomainUser;
use SAPDB::Install::Instance::Check::Instances;
use SAPDB::Install::Instance::Check::Finalize;
use SAPDB::Install::Instance::Check::Params;
use SAPDB::Install::Instance::Check::Volumes;

our @ISA = qw (
	SAPDB::Install::Instance::Base 
	SAPDB::Install::Instance::Check::Conditions
	SAPDB::Install::Instance::Check::DataCons
	SAPDB::Install::Instance::Check::DomainUser
	SAPDB::Install::Instance::Check::Instances
	SAPDB::Install::Instance::Check::Finalize
	SAPDB::Install::Instance::Check::Params
	SAPDB::Install::Instance::Check::Volumes);

#
# do_anycheck ()
# do any possible check method
#
sub do_anycheck {
	my ($self) = @_; 
	my $rc;

	$rc = $self->conditions ();
	return (-1) unless (defined $rc && $rc == 0);

	$rc = $self->instances ();
	return (-1) unless (defined $rc && $rc == 0);

	$rc = $self->params ();
	return (-1) unless (defined $rc && $rc == 0);

	$rc = $self->volumes ();
	return (-1) unless (defined $rc && $rc == 0);

	$rc = $self->domainuser ();
	return (-1) unless (defined $rc && $rc == 0);

	$rc = $self->datacons ();
	return (-1) unless (defined $rc && $rc == 0);

	return 0;
}

#
# sub do_finalizecheck ()
# check for recovered instance when using
# migration strategy 'EXTERNAL_CONSISTENT_BACKUP'
#
sub do_finalizecheck {
	my ($self) = @_;

	return ($self->finalizecheck ());
}

#
# main
#
sub main {
	return (SAPDB::Install::Instance::Check::Test::main (@_));
}

package SAPDB::Install::Instance::Check::Test;

our $DEBUG = 0;

use SDB::Common::Getopt::Long;

sub main {
	local @ARGV = @_;

	my $dbname, $userpwd, $target, $verbose, $help, $finalize, $apotarget;

	# allow interpretation of '-v' as '--verbose'
	SDB::Common::Getopt::Long::Configure ('auto_abbrev');
	GetOptions (
		'dbname=s', \$dbname, 
		'user=s', \$userpwd,
		'target:s', \$target,
		'apotarget:s', \$apotarget,
		'help', \$help,
		'finalize', \$finalize,
		'verbose', \$verbose);

	if (defined $help) {
		print "SAPDB::Install::Instance::Check::main\n";
		print "check, if instance will be ready\n";
		print "for software upgrade or patch installation\n";
		print "usage:\n";
		print "  -d (--dbname) <dbname>\n";
		print "  -u (--user)   <dbmusr,dbmpasswd>\n";
		print "  -t (--target) <buildstring>\n";
		print "  -a (--apotarget) <buildstr>\n";
		print "  -f (--finalize)\n";
		print "  -v (--verbose)\n";
		print "\n";
		return 0;
	}

	$DEBUG = 1 if ($verbose);

	my $ic = SAPDB::Install::Instance::Check->new ();

	$ic->{msg0} = \&msg0;
	$ic->{msg1} = \&msg1 if ($DEBUG);

	unless (defined $dbname) {
		printmsg ("missing instance name\n");		
		return 0;
	}

	unless (defined $userpwd) {
		printmsg ("missing user,password of dbm user\n");
		return 0;
	}

	my $rc;
	if (defined $finalize) {
		$rc = $ic->session ('dbname' => $dbname,
		      'userpwd' => $userpwd, 'target_release' => $target,
		      'apo_target_release' => $apotarget);
	} else {
		$rc = $ic->session ('dbname' => $dbname,
		      'apo_target_release' => $apotarget, 
		      'userpwd' => $userpwd, 'target_release' => $target);	
	}

	unless (defined $rc)  {
		printmsg ($ic->{msgtxt});
		return 0;
	}

	if (defined $finalize) {
		$rc = $ic->do_finalizecheck ();
		printmsg ("\n");
		unless (defined $rc) {
			printmsg ("finalize check failed during:\n");
		} elsif ($rc == 0) {
			printmsg ("finalization allowed\n");
		} else {
			printmsg ("finalization not allowed\n");
		}
	} else {
		$rc = $ic->do_anycheck ();
		printmsg ("\n");
		if ($rc == 0) {
			printmsg ("instance check ok\n");
		} else {
			printmsg ("instance check failed during:\n");
		}
	}

	unless ($ic->get_errorstate () eq 'OK') {
		printmsg ("\n");
		printmsg ($ic->{msgtxt});
	}
	printmsg ("errorstate = ".$ic->get_errorstate ()."\n");
}

sub msg1 {
	print "INFO>>> ";
	print @_;
}

sub msg0 {
	print "MSG >>> ";
	print @_;
}

sub printmsg {
	my $txt = shift;

	msg0 ($txt) if ($txt) eq "\n";

	foreach (split '\n', $txt) {
		msg0 ($_."\n");
	}
}

1;
