#!/usr/bin/perl
#
# $Header: //sapdb/V76/c_00/b_32/sys/src/install/perl/SAPDB/Install/Log.pm#1 $
# $DateTime: 2006/07/06 16:50:32 $
# $Change: 130190 $
#
# Desc: 

package SAPDB::Install::Log;

use SAPDB::Install::Base;
use SAPDB::Install::System;
use SAPDB::Install::StdIO;
use SAPDB::Install::Values;
use SAPDB::Install::Registry;
use SAPDB::Install::Trace;

our @ISA=qw(SAPDB::Install::Base);

my %members=(
				LogPath => undef,
				LogOff  => undef,
				LogName => undef,
				Messages => undef, 
				Filename => 'SDBINST'
			);

sub new{
	my ($type,$path) = @_;
	my $self= SAPDB::Install::Base->new;
	foreach my $key (keys(%members)){
		$self->{$key}=$members{$key};
	}
	$self = bless $self,$type;
	
	$self->Messages([]);

	if($path){
		$self->LogPath($path);
	}
	return $self;
}


sub SetMsg{
    my ($self,$msg)=@_;
    TraceMsg($msg,2,\$DEBUG,2) unless($msg =~ /^STD/);
	*prot=$self->Messages;
	push @prot,$msg;
	return 1;
}

sub writeLog{
    my ($self)=@_;
	$self->LogOff and return;
	unless($self->LogPath){
		my ($indep_data,$indep_prog)=readIndepPath();
		if($indep_data){
			$self->LogPath("$indep_data/wrk");
		}
		else{
			$self->LogPath($SAPDB::Install::Values::curDir);
		}
	}
	defined &SAPDB::Install::History::writeHistory && SAPDB::Install::History::writeHistory();
	unless(-d $self->LogPath){
		makedir($self->LogPath, 0777,$SAPDB::Install::uid,$SAPDB::Install::gid);
	}
	my $name="MaxDB Software";
	$self->LogName =~ /\S/ and $name = $self->LogName; 
	$name=~s/\s//g; # delete whitespaces
	my $hour=(localtime)[2];
	my $minute = (localtime)[1];
	my $second= (localtime)[0];
	my $day= (localtime)[3];
	my $month=((localtime)[4]+1);
	$hour = "0$hour" if(length($hour)==1);
	$minute = "0$minute" if(length($minute)==1); 
    $second = "0$second" if(length($second)==1);
	$day = "0$day" if(length($day)==1);
	$month = "0$month" if(length($month)==1);
	my $fullname=$self->LogPath.'/'.$name."_install-$day.$month.".((localtime)[5]+1900)."-$hour.$minute.log";
	
	open(LOG,">$fullname") or print2stderr("cannot create logfile \"$fullname\"\n");
    *prot=$self->Messages;
	foreach $line (@prot){
		print LOG $line;
    }
    close(LOG);
	
	if($ENV{'SDBINSTLOGFILE'} =~ /\S/){
		print2stdout("saving installation logfile as ".$ENV{'SDBINSTLOGFILE'}."...\n");
		copy($fullname,$ENV{'SDBINSTLOGFILE'});
	}
	 
}

sub DESTROY{
	my ($self)=@_;
	$self->writeLog;
	$self->SAPDB::Install::Base::DESTROY;
	WriteTrace();
}


1;