#!/usr/bin/perl
#
# $Header: //sapdb/V76/c_00/b_32/sys/src/install/perl/SAPDB/Install/PackageObj.pm#1 $
# $DateTime: 2006/07/06 16:50:32 $
# $Change: 130190 $
#
# Desc: 


package SAPDB::Install::PackageObj;

use SAPDB::Install::Tools;
use SAPDB::Install::Base;
use SAPDB::Install::Values;
use SAPDB::Install::MD5Sum;
use SAPDB::Install::Trace;
use SAPDB::Install::System;
use SAPDB::Install::StdIO;

our @ISA = qw (SAPDB::Install::Base);

my %members=(
	ID => undef,
	InterfaceVersion => undef,
	Name => undef,						# unique package type identifier
	Version => undef,
	PackageVersion => undef,
	DbgPackage => undef,
	DbgHandle => undef,
	DbgSize => undef,
	DbgMd5 => undef,
	DbgFiles => undef,
	Mode => undef,
	Type => undef,
	MinVersion => undef,
	Profiles => undef,
	TopOf => undef,
	Require => undef,
	Required => undef,
	FileList => undef,
	Script => undef,
	TestFile => undef,
	MagicMode => undef,
	AllowSkip => undef,
	preinstall => undef,
	preunpack => undef,
	postprepare => undef,
	postinstall => undef,
	register => undef,
	unregister => undef,
	framestart => undef,
	framestop => undef,
	Paths => undef,
	MainPath => undef,
	Archive => undef,
	Opts => undef,
	RegData=> undef,
	Usage => undef,
	UsageDesc => undef,
	Skip => undef,
	Update => undef,
	User => undef,
	UID => undef,
	Group => undef,
	GID => undef,
	MagicString => undef,
	Untgz => undef,
	IsSubPackage => undef,			# boolean value 
	SubDir => undef,				# part of package path for subpackage
	SubPackages => undef,				# array of subpackage objects
	ParentPackage => undef,			# parent package object, only if IsSubPackage is set 
	FlushIntoReg => 0,				# flag to activate / deactivate changes in regpackobj, needed for prepare phase
	CheckSum => undef,				# checksum of archive file
	WithParent => undef, 			# subpackage will be installed together with its parent package
	SubPackObjs => undef,			# list of subpackage objects that will be installed togeter with this package
	DispName => undef,				# name of pakages visible from outside
	IgnoreBusyFiles => undef,		# list of files which won't break unpack check
	Size => undef,
	BuildString => undef,
	MakeId => undef,
	QALevel => undef,
	Purpose => undef,
	ChangeList => undef,
	MakeState => undef,
	Desc => undef
);


$members{'MappedFiles'} = undef if ($^O =~ /mswin/i);


my $id=0;

sub evalScript{
	my ($self) = @_;
	defined $self->Script or return 0;
	my $newpackagename='SAPDB::Install::Pkg'.$self->ID;
	sub getPerlPackageName{
		my ($package,$filename,$line)=caller;
		return $package;
	}
	my $packagename=getPerlPackageName();	
	my $varname='SELFOBJ['.$self->ID.']';
	#my $svid=$self->ID;
	#my $script= "package $newpackagename;\n local *SELFOBJ=\$$packagename\::refTmp;\n".${$self->Script};
	my $script= "package $newpackagename;\n".${$self->Script};
	$script=~s/\$packobj/\$$packagename\::$varname/g;
	#$script=~s/\$packobj/\$SAPDB::Install::StartInstall::packageref[$svid]/g;
	$script=~s/\$instRegistry/\$SAPDB::Install::StartInstall::instRegistry/g;
	if($DEBUG){
		my @text= split("\n",$script);
		my $linecount=1;
		foreach my $line (@text){
			print "$linecount: $line\n";
			$linecount++;
		}
	}
	defined(%{"${newpackagename}::"}) and $SAPDB::Install::Values::log->SetMsg("WRN: generated package exists in mem - now overwrite it\n");
	eval($script) or print $self->Name."ERR: $@\n" and return 0;
	foreach my $func ('preinstall','preunpack','postprepare','postinstall','register','unregister','framestart','framestop'){
		if(defined &{"${newpackagename}::$func"}){
			TraceMsg("MSG: set function reference \"$func\" in package ".$self->Name."\n",3,\$DEBUG);
			$self->$func(\&{"${newpackagename}::$func"});
		}
		else{
			TraceMsg("MSG: no function \"$func\" in package ".$self->Name."\n",3,\$DEBUG);
		}
	}
	defined ${"${newpackagename}::usage"} and $self->Usage(${"${newpackagename}::usage"});
	defined ${"${newpackagename}::usage_desc"} and $self->UsageDesc(${"${newpackagename}::usage_desc"});
	defined ${"${newpackagename}::main_path"} and $self->MainPath(${"${newpackagename}::main_path"});
	defined ${"${newpackagename}::skip"} and $self->Skip(\${"${newpackagename}::skip"});
	my @newArray;
	defined @{"${newpackagename}::paths"} and $self->Paths(\@{"${newpackagename}::paths"}) or $self->Paths(\@newArray);
	defined @{"${newpackagename}::opts"} and $self->Opts(\@{"${newpackagename}::opts"}) or $self->Opts(\@newArray);

}

sub initData{
	my ($self,$rhData)=@_;
	ref($rhData) ne 'HASH' and return 0;
	my %data = %$rhData;
	$self->InterfaceVersion($data{'INTERFACE_VERSION'});
	$self->Name($data{'PACKAGE_NAME'});
	$self->Version($data{'SOFTWARE_VERSION'});
	$self->PackageVersion($data{'PACKAGE_VERSION'});
	$self->Mode($data{'MODE'});
	$self->Type($data{'TYPE'});
	my @profiles=split(',',$data{'PART_OF'});
	$self->Profiles(\@profiles);
	my @require=split(',',$data{'REQUIRE'});
	$self->Require(\@require);
	$self->Archive($data{'ARCHIVE_NAME'});
	$self->MinVersion($data{'MIN_VERSION'});
	$self->AllowSkip($data{'ALLOW_SKIP'});
	$self->TopOf($data{'IS_TOP_OF'});
	$self->IsSubPackage($data{'IS_SUBPACKAGE'});
	$self->SubDir($data{'SUB_DIR'});
	$self->MagicMode($data{'MAGIC_MODE'});
	$self->Desc($data{'DESC'});
	$self->Size($data{'SIZE'});
	$self->BuildString($data{'BUILD_STRING'});
	if($data{'BUILD_STRING'}){
		my ($make_state,$qa_level,$purpose,$change) = ($data{'BUILD_STRING'} =~ /-(\d)(\d)(\d)-(\d{3}-\d{3})$/);
		$change =~ s/-//g;
		$change =~ s/^0*//;
		$self->ChangeList($change);
		if($make_state == 1){
			$make_state = 'productive';
		}
		else{
			$make_state = 'test/development';
		}
		$self->MakeState($make_state);
		if($qa_level == 0){
			$qa_level = 'DEV';
		}
		elsif($qa_level == 1){
			$qa_level = 'COR';
		}
		elsif($qa_level == 2){
			$qa_level = 'RAMP';
		}
		elsif($qa_level == 3){
			$qa_level = 'HOT';
		}
		elsif($qa_level == 4){
			$qa_level = 'ANA';
		}
		$self->QALevel($qa_level);
		my @purpose;
		if($purpose == 0){
			push @purpose,'no classification';
		}else{
			if(($purpose & 1) != 0){
				push @purpose,'OLTP';
			}
			if(($purpose & 2) != 0){
				push @purpose,'liveCache';
			}		
		}
		$self->Purpose(join(' & ',@purpose));
	}
	
	
	$self->MakeId($data{'MAKE_ID'});
	unless($^O=~/mswin/i){
		$self->User($data{'DEFAULT_USER'});
		$self->Group($data{'DEFAULT_GROUP'});
	}
	$self->Untgz($data{'UNTGZ'});

	if($data{'TEST_FILE'}){
		$self->TestFile($data{'TEST_FILE'});
		$self->MagicString($data{'MAGIC'});
	}
	if(defined $data{'DISPLAY_NAME'}){
		$self->DispName($data{'DISPLAY_NAME'});
	}
	else{
		$self->DispName($data{'PACKAGE_NAME'});
	}

	my $text;
	if(defined $data{'SCRIPT_VALUE'}){
		$text = $data{'SCRIPT_VALUE'}; 	
	}
	else{
		$text = readFileInTGZ($data{'UNTGZ'},$data{'SCRIPT'});
	}
	$self->Script(\$text);
	
	unless($SAPDB::Install::Values::GUI_MODE){
		my $md5sum = MD5Sum($data{'ARCHIVE_NAME'});
		if(defined $md5sum && $md5sum =~ /^[0-9a-f]{32}$/){
			$self->CheckSum($md5sum);
		}
		if (defined $data{'DBG_PACKAGE'} and $data{'MAKE_ID'} =~ /^0+$/){
			unless ($data{'DBG_MD5SUM'} eq $md5sum){
				$SAPDB::Install::Values::log->SetMsg ("WRN: debug package for ".$data{'PACKAGE_NAME'}." isn\'t valid: md5sum doesn\'t match\n");
				$SAPDB::Install::Values::log->SetMsg ("WRN: $md5sum vs. ".$data{'DBG_MD5SUM'}."\n");
				delete $data{'DBG_PACKAGE'};
			}
		}
	}
	
	if(defined $data{'DBG_PACKAGE'}){
			$self->DbgPackage($data{'DBG_PACKAGE'});
			$self->DbgHandle($data{'DBG_TGZHANDLE'});
			$self->DbgSize($data{'DBG_SIZE'});
			$self->DbgMd5($data{'DBG_MD5SUM'});
			if (exists $data{'DBG_FILELIST_VALUE'}){
				$self->DbgFiles($data{'DBG_FILELIST_VALUE'});
			}
			else{
				my $filelist = readFileInTGZ($data{'DBG_TGZHANDLE'},'FILELIST');
				my %md5sums;
				foreach my $file (split("\n",$filelist)){
					$file=~/^\s*"(\S.*\S)"\s*([0-9a-z]{32})\s*$/ and $md5sums{$1}=$2;
				}
				$self->DbgFiles(\%md5sums);
			}
	}
		
		
	if(defined $data{'FILELIST_VALUE'}){
		$self->FileList($data{'FILELIST_VALUE'});
	}
	else{
		my $filelist = readFileInTGZ($data{'UNTGZ'},$data{'FILELIST'});
		my %md5sums;
		foreach my $file (split("\n",$filelist)){
			$file=~/^\s*"(\S.*\S)"\s*([0-9a-z]{32})\s*$/ and $md5sums{$1}=$2;# and print "#####MATCH#####\n" or print "#####DONT MATCH#####\n";
		}
		$self->FileList(\%md5sums);
	}
	$self->evalScript; 
}


sub PrepareMSCS{
	my ($self,$clu_files,$sys_files) = @_;
	my $path = $self->MainPath->{'value'};
	my @refresh_md5;		
	makedir($path.'/noncluser',0555) unless -d ($path.'/noncluser');
	foreach my $clu_file (@$clu_files){
	
		#
		# save originals in noncluser/
		#
			
		if(rename($path.'/pgm/'.$clu_file.'.exe',$path.'/noncluser/'.$clu_file.'.exe')){
			$SAPDB::Install::Values::log->SetMsg ('saved pgm/'.$clu_file.'.exe as noncluser/'.$clu_file.".exe\n");
		}
		else{
			print2stderr("cannot save ".$path.'/pgm/'.$clu_file.".exe: $!\n");
			return undef;
		}
			
		#
		# copying cluser files
		#
		
		if(copy($path.'/cluster/'.$clu_file.'_clu.exe',$path.'/pgm/'.$clu_file.'.exe',{'binmode' => 1})){
			#$SAPDB::Install::Values::log->SetMsg ('copy cluster/'.$clu_file.'_clu.exe -> pgm/'.$clu_file.".exe\n");
		}
		else{
			print2stderr('cannot copy '.$path.'/cluster/'.$clu_file."_clu.exe: $!\n");
			return undef;	
		}
		push @refresh_md5,('pgm/'.$clu_file.'.exe','noncluser/'.$clu_file.'.exe'); 
	}	
	
	
	if(defined $sys_files){
		my $windir = getWinDir();
		my $sysdir = getSysPath(); # system32
		if (-d $windir){
			makedir($windir.'/cluster',0555) unless(-d $windir.'/cluster');
			foreach my $sys_file (@$sys_files){
				unless(	copy($path.'/cluster/'.$sys_file,$sysdir.'/'.$sys_file,{'binmode' => 1})
						and 
					copy($path.'/cluster/'.$sys_file,$windir.'/cluster/'.$sys_file,{'binmode' => 1}))
				{
					print2stderr('cannot copy '.$path.'/cluster/'.$sys_file."\n");
					return undef;
				}
			}
		}
		else{
			print2stderr("cannot determine windows system folder\n");
			return undef;
		}
	} 

	#
	# refresh md5 sums
	#		
	$self->RegData->updateMd5Sum(@refresh_md5);	
	
	return 1;
}




our @SELFOBJ;


sub new{
	my ($type) = @_;
	
	if ($id == 0){
		my $level = 0;
		my $found_cleaner = 0;
		while(1){
			my @all_info_caller_package = (caller($level));
			$#all_info_caller_package == -1  and last;
			$level++;
			my $package = $all_info_caller_package[0];
			if(defined ${"${package}::cleaner"} and	ref(${"${package}::cleaner"}) eq 'SAPDB::Install::Cleaner'){ 
				TraceMsg("found cleaner object \$${package}::cleaner\n",3,\$DEBUG);
				${"${package}::cleaner"}->SetRef(\@SELFOBJ);
				$found_cleaner = 1;
				last;
			}
		}
		
		unless($found_cleaner){
			print2stderr("WRN: no cleaner object of type SAPDB::Install::Cleaner found - possible installation data loss\n");
		}
	}

	#$id==0 and $SAPDB::Install::StartInstall::cleaner->SetRef(\@SELFOBJ);
	my $self= SAPDB::Install::Base->new;
	foreach my $key (keys(%members)){
		$self->{$key}=$members{$key};
	}
	$self = bless $self,$type;
	$SELFOBJ[$id]=$self;
	$self->ID($id++);
	$self->SubPackObjs([]);
	$self->SubPackages([]);
	return $self;
}


sub freeSELFOBJ{
	#workaround for problem with simple perl garbage collection (no cycle detection)  
	my ($self)=@_;
	$SELFOBJ[$self->ID]=undef; # free $SELFOBJ[ID] 
}


sub setDataInRegistry{
	my ($self)=@_;
	$self->FlushIntoReg or TraceMsg("data not flushable - flag disabled\n",3,\$DEBUG) and return 1;
	$self->RegData->Version($self->Version);
	$self->RegData->Require($self->Require);
	$self->RegData->User($self->User);
	$self->RegData->Group($self->Group);
	$self->RegData->PackageVersion($self->PackageVersion);
	$self->RegData->Profiles($self->Profiles);
	$self->RegData->Pathes($self->Paths);
	$self->RegData->Size($self->Size);
	$self->RegData->BuildString($self->BuildString);
	$self->RegData->MakeId($self->MakeId);
	defined $self->CheckSum and $self->RegData->CheckSum($self->CheckSum);

	$self->RegData->TestFile($self->TestFile) if $self->TestFile =~ /\S/;
	$self->RegData->Desc($self->Desc) if $self->Desc =~ /\S/;

	if($self->DispName ne $self->Name){
		$self->RegData->DispName($self->DispName);
	}
	$self->RegData->Mode($self->Mode);
	$self->RegData->IsSubPackage($self->IsSubPackage);
	#$self->RegData->SubPackages($self->SubPackages);
	$self->RegData->ParentPackage($self->ParentPackage);
	#if (defined $self->ParentPackage){
	#	if(ref($self->ParentPackage) =~ /SAPDB::Install::InstallRegistry::Package/){
	#			$self->RegData->ParentPackage([${$self->ParentPackage}->Name,${$self->ParentPackage}->Path]);
	#	}
	#	elsif(ref($self->ParentPackage) =~ /SAPDB::Install::PackageObj/){
	#			$self->RegData->ParentPackage([${$self->ParentPackage}->Name,${%{${$self->ParentPackage}->MainPath}}{'value'}]);
	#	}
	#}
}

sub DESTROY{
	my ($self)=@_;
	defined $self->RegData && $self->setDataInRegistry;
	$self->SAPDB::Install::Base::DESTROY;
}


1;

