#!/usr/bin/perl
#
# $Header: //sapdb/V76/c_00/b_32/sys/src/install/perl/SAPDB/Install/Portable/Registry.pm#1 $
# $DateTime: 2006/07/06 16:50:32 $
# $Change: 130190 $
#
# Desc: functions to find MaxDB RTE Registrations in portable mode  

package SAPDB::Install::Portable::Registry;

use SAPDB::Install::Registry;
use SAPDB::Install::Portable::Helper;

require SDB::Common::Exporter;
our @ISA = qw (SDB::Common::Exporter);

our @EXPORT = qw (	deleteSAPDBTechPortableRoot
					getInstalledReleasesPortableRoot
					getInstancesPortableRoot
					readRuntimesPortableRoot
					readIndepPathPortableRoot);	
					



sub deleteSAPDBTechPortableRoot{
	$SAPDB::Install::Values::owner = undef;
	$SAPDB::Install::Values::group = undef;
	
	my $glob_path = $SAPDB::Install::Config{'SDB_PORTABLE_ROOT'}.$SAPDB::Install::Values::glob_path;
	
	if(-f $glob_path){

		my $globals = readGlobals();
 		if (exists $globals->{'SdbOwner'}){
 			my $uid = getpwnam($globals->{'SdbOwner'});
 			if (defined $uid and $uid){
 				deleteUser($globals->{'SdbOwner'});				
 			}
 			my $gid = getgrnam($globals->{'SdbGroup'});
 			if (defined $gid and $gid){
 				deleteGroup($globals->{'SdbGroup'});				
 			}
 		}

		unless(unlink($glob_path)){
			$SAPDB::Install::Values::log->SetMsg("WRN: cannot delete \"$glob_path\"\n")
				if defined $SAPDB::Install::Values::log;
		}
	}
	else{
		$SAPDB::Install::Values::log->SetMsg("WRN: cannot delete \"$glob_path\" - file dont exist\n")
			if defined $SAPDB::Install::Values::log;
	}
	
	$SAPDB::Install::Values::log->SetMsg("MSG: removed SAP DB Registry successfully\n")
		if defined $SAPDB::Install::Values::log;
	return 1;
}

sub getInstalledReleasesPortableRoot{
    my %returnvalue;
    my $indep_data = (readIndepPath())[0];
	my $path = $indep_data.'/config/Installations.ini';
	
	if(-f $path){
		my %hash = readini($path);
		return undef unless defined %hash;
		foreach $inst (keys( %{$hash{'Installations'}})){
			my $full_inst = getAbsPath ($inst);
			unless (defined $full_inst){
				$full_inst = $inst;
			}
			next if exists $returnvalue{$full_inst};
			$_=$hash{'Installations'}{$inst};	
			/([0-9][0-9]*\.[0-9][0-9]*\.[0-9][0-9]*\.[0-9][0-9]*),/ and $returnvalue{$full_inst}=$1;
		}
	}	
	return %returnvalue;
}

sub getInstancesPortableRoot{
	my %returnvalue;
	my $indep_data = (readIndepPath())[0];
	my $path  = $indep_data.'/config/Databases.ini';
	if(-f $path){
		my %hash=readini($path);
		my $tmp = $hash{'Databases'};
		my %dbroot=%$tmp;
		foreach my $db (keys(%dbroot)){
			#filter service databases ( '_' for < 7.4 )
			unless($db=~/^_/ || $db=~/^\./){
					my $abs_path = getAbsPath ($dbroot{$db});
					$dbroot{$db} = $abs_path if (defined $abs_path);
					$returnvalue{$db}= $dbroot{$db} unless exists $returnvalue{$db};
			}
		}
	}	
	return %returnvalue;
}

sub readRuntimesPortableRoot{
	my %returnvalue;
	my $indep_data = (readIndepPath())[0];
	my $path = $indep_data.'/config/Runtimes.ini';
	return () unless -f $path;
	my %hash = readini($path) or print2stderr("cannot read $path\n") and return ();
	my %releases = %{$hash{'Runtime'}};
	foreach my $rpath (keys(%releases)){
		my $full_path = getAbsPath ($rpath);
		next if exists $returnvalue{$full_path};
		$returnvalue{$full_path}=$releases{$rpath};
		$returnvalue{$full_path}=~s/,.*$//;
	}
	return %returnvalue;
}

sub readIndepPathPortableRoot{
	my $globs = readGlobals();
	return () unless defined $globs;
	return $globs->{'IndepData'}, $globs->{'IndepPrograms'};
}



1;

