package SAPDB::Install::Templates::InstUnit;


use SAPDB::Install::Tools;
use SAPDB::Install::Templates::Properties;
use SAPDB::Install::Templates::InstTemplate;
use SAPDB::Install::Templates::Util;
use SAPDB::Install::Values;
use SAPDB::Install::MD5Sum;
use SAPDB::Install::Trace;
use SAPDB::Install::Registry;
use SAPDB::Install::System;
use strict;


#you can manually set debug levels from 0 - 4
my $sdbinstDebugLevel=0;

#ifdef SCRIPTING
#require Exporter;
#our @ISA = qw (SAPDB::Install::Templates::Properties);
#our @EXPORT = qw (createObjectFromTGZ);
#endif
#because SDBINST
require SDB::Common::Exporter;
our @ISA = qw (SAPDB::Install::Templates::Properties);
our @EXPORT = qw (createObjectFromTGZ);
#endbecause






sub readAFileInTGZ{
  my($tgzhandle,$readfile)=@_;
  my $returnvalue;
  my $info = "start";
  $tgzhandle->Rewind(); 
  my $retVal = 0;
  my $file=undef;
  while ($info) {
    $info = $tgzhandle->Next();
    $retVal = $tgzhandle->GetErr() and return ($retVal,undef);
    unless($info){
      next;
    }
    $info =~ /(\S+)$/;
    my $aFileName = $1;
    unless($aFileName){
      next;
    }
    if ($aFileName eq $readfile) {
      $retVal =  $tgzhandle->ExtractScalar($file) and return ($retVal,undef);
      last;
    }
  }
  $file or $retVal = "Could not find $readfile\n";
  return ($retVal,$file);
}

sub createObjectFromTGZ{
  my $untgz = shift;
  my ($retVal,$instConfig,$rawData); 
  ($retVal,$instConfig) = SAPDB::Install::Templates::InstTemplate::createObject();
  $retVal and return ($retVal,undef);
  ($retVal,$rawData) = readAFileInTGZ($untgz,'PACKAGEDATA');
  $retVal and return ($retVal,undef);
  my %data = %SAPDB::Install::Values::package_interface;
  $rawData or return ("No package data found!",undef);
  my @filedata=split("\n",$rawData);
  @filedata or return ("Package data file is empty!",undef);
  foreach my $line (@filedata){
    foreach my $key (keys(%data)){
      if($line =~ /^\s*$key\s*=\s*"(.*)"\s*$/){
        $data{$key} = $1;
        last;
      }
    }
  }
  keys %data or return ("Package data file contains no valid data!",undef);
  $data{'UNTGZ'}=$untgz;
  my $newpackage = SAPDB::Install::Templates::InstUnit::new(\%data,$instConfig);
  return ($retVal,$newpackage);
}

sub createObjectFromRegistry{
  my $reg = shift;
  my %data  ;
  my ($retVal,$instConfig);
  ($retVal,$instConfig) = SAPDB::Install::Templates::InstTemplate::createObject();
  $retVal and return ($retVal,undef);
  $data{'PACKAGE_NAME'}=$reg->Name;
  $data{'SOFTWARE_VERSION'}=$reg->Version;
  $data{'DESC'}=$reg->Desc;
  my $packobj = SAPDB::Install::Templates::InstUnit::new(\%data,$instConfig);
  $packobj->setMainPath($reg->Path);
  $packobj->setFileList($reg->FileList);
  return ($retVal,$packobj);
}










# delete a directory tree
sub delTree{
  my ($this,$dir) = @_;
  -d $dir || return 0;
  opendir(DH,$dir) || return 0;
  my @content = readdir(DH);
  closedir(DH);
  my @files = grep { -f "$dir/$_" } @content;
  my @dirs = grep { -d "$dir/$_" && ! /^\.{1,2}$/} @content;
  unless($^O =~ /mswin/i){
    if ($#files > -1 || $#dirs > -1) {
      my @statbuf = stat($dir);
      my $mask;
      if ($> == $statbuf[4]) {
        $mask = 0200;
      } else {
        my $found;
        foreach my $gid (split(' ',$) )) {
          if ($gid == $statbuf[5]) {
            $found = 1;
            last;
          } 
        }
        if ($found) {
          $mask = 020;
        } else {
          $mask = 002;
        }
      }
      if (($statbuf[2] & $mask) == 0) {
        chmod($statbuf[2] | $mask,$dir) || $this->warnit("cannot enable write permissions for $dir\n");
      }
    }
  }

  foreach (@dirs) {
    -l "$dir/$_" and next;
    delTree("$dir/$_") || return 0;
  }
  foreach (@files) {
    -l "$dir/$_" and next;
    -f "$dir/$_" and (unlink("$dir/$_") or $this->warnit("cannot delete $dir/$_\n") and return 0);
  }
  rmdir($dir) or $this->warnit("cannot delete $dir\n") and return 0;
  return 1;
}



sub testaccess{
  my ($this,$node,$username,$access) = @_;
  $^O =~ /mswin/i and return 0;
  -e $node or return "Access failed: no such file or directory: $node";
  my @passwd_buf = getpwnam($username);
  my $uid = $passwd_buf[2];
  defined $uid or return "unknown user $username"; 
  $access =~ /^[rwx]{1,3}$/i or return "wrong access mode format '$access'";  
  my @statbuf = stat ($node) or return "Cannot stat $node: $!"; 
  my $mode = $statbuf[2];
  if ($statbuf[4] == $uid) { # user is owner of node:check owner permissions
    if ($access =~ /r/i) {
      ($mode & 0400) == 0 and return 'owner has no r bit'; 
    }
    if ($access =~ /w/i) {
      ($mode & 0200) == 0 and return 'owner has no w bit';
    }
    if ($access =~ /x/i) {
      ($mode & 0100) == 0 and return 'owner has no x bit';
    } 
    return 0;
  }
  # try to find username in group of node 
  my $found = 0;
  my $prim_gid == $statbuf[5] and $found = 1; # group is primary group of user 
  foreach my $member (split(' ',(getgrgid($statbuf[5]))[3])) {
    $found and last;            # group is primary group of user
    $member eq $username and $found = 1 and last; 
  } 
  if($found){
    if ($access =~ /r/i) {
      ($mode & 040) == 0 and return 'group has no r bit';
    }
    if ($access =~ /w/i) {
      ($mode & 020) == 0 and return 'group has no w bit' ;
    }
    if ($access =~ /x/i) {
      ($mode & 010) == 0 and return 'group has no x bit' ;
    } 
    return 0;
  } else { # user is not owner and not member of group: check permissions for others
    if ($access =~ /r/i) {
      ($mode & 04) == 0 and return 'others have no r bit';
    }
    if ($access =~ /w/i) {
      ($mode & 02) == 0 and return 'others have no w bit'; 
    }
    if ($access =~ /x/i) {
      ($mode & 01) == 0 and return 'others have no x bit'; 
    } 
    return 0;
  }
  return "Not found $node"; 
}


sub crelease2num{
  my $this = shift;
  $_=shift; 
  my $cor_digit_num = 5;
  unless(/^(\d)+\.(\d+)\.(\d+)\.(\d+)$/ || /^(\d)\.(\d)([A-Z|\d]+)\.(\d+)$/){
    return 0;
  } 
  my ($major, $minor,$cor_lev,$pat_lev) = ($1,$2,$3,$4);
  if ($cor_lev =~ /\D/) {
    $cor_lev = ord($cor_lev) - ord('A');
  }
  if ($cor_digit_num < length($cor_lev)) {
    return 0;
  }
  length($minor)==1 and $minor="0$minor";
  $cor_lev = ('0' x ($cor_digit_num - length($cor_lev))) . $cor_lev;
  if (length($pat_lev) == 1) {
    $pat_lev="00$pat_lev";
  } elsif (length($pat_lev) == 2) {
    $pat_lev="0$pat_lev";
  } elsif (length($pat_lev) > 3) {
    return 0;
  }
  my $num="$major$minor$cor_lev$pat_lev";
  return $num;
}




#our $keys->{'MappedFiles'} = {} if ($^O =~ /mswin/i);

sub GetUID{
  $^O =~ /mswin/i and return 0;
  my $this = shift;
  my $instConfig = $this->getInstTemplate();
  my $uid = getpwnam($instConfig->getOwnerName());
  return $uid;
}
sub GetGRID{
  $^O =~ /mswin/i and return 0;
  my $this = shift;
  my $instConfig = $this->getInstTemplate();
  my $grid = getgrnam($instConfig->getOwnerGroup());
  return $grid;
}

sub GetRelease{
  my $this = shift;
  my ($release) = ($this->getVersion() =~ /(\d+\.\d+)/); 
  return $release;
}

sub warnit{
  my ($this,$message)  = @_;
  my $lt; 
  $lt = $this->internalGetListeners('warning') or return 1;
  foreach(@$lt){ &$_($message);}
  return 0;
}
sub logit{
  my ($this,$message)  = @_;
  my $lt; 
  $lt = $this->internalGetListeners('log') or return 1;
  foreach(@$lt){ &$_($message);}
  return 0;
}

sub debugit{
  my ($this,$level,$message)  = @_;
  my $lt; 
  $lt = $this->internalGetListeners('debug') or return 1;
  my $name = $this->getName();
  foreach(@$lt){ &$_($level, "($name) " . $message);}
  return 0;
}
sub internalGetListeners{
  my ($this,$type) = @_;
  my $instConfig = $this->getInstTemplate();
  $instConfig or return undef;
  return $instConfig->GetListeners($type);
}


sub SetInstTemplate{
  my ($this,$value) = @_;
  if($value){
    my $indepProg = $value->getIndepProg();
    if ($indepProg) {
      $this->setIndepProg($indepProg);
    }
    my $indepData = $value->getIndepData();
    if ($indepData) {
      $this->setIndepData($indepData);
    }
    $this->setSysProgPath($value->getSysProgPath());
    $this->setUser($value->getOwnerName());
    $this->setGroup($value->getOwnerGroup());
  }
  return $this->setInstTemplate($value);
}
	
sub initData{
  my ($this,$rhData)=@_;
  $this->SUPER::initData();
  my %data = %$rhData;
  $this->setInterfaceVersion($data{'INTERFACE_VERSION'});
  $this->setName($data{'PACKAGE_NAME'});
  $this->setVersion($data{'SOFTWARE_VERSION'});
  $this->setPackageVersion($data{'PACKAGE_VERSION'});
  $this->setMode($data{'MODE'});
  $this->setType($data{'TYPE'});
  if($data{'PART_OF'}){
    my @profiles=split(',',$data{'PART_OF'});
    $this->setProfiles(\@profiles);
  }
  if ($data{'REQUIRE'}) {
    my @require=split(',',$data{'REQUIRE'});
    $this->setRequire(\@require);
  } else {
    $this->setRequire([]);
  }
  $this->setArchiveName($data{'ARCHIVE_NAME'});
  $this->setArchiveDir($data{'ARCHIVE_DIR'});
  $this->setMinVersion($data{'MIN_VERSION'});
  $this->setAllowSkip($data{'ALLOW_SKIP'});
  $this->setTopOf($data{'IS_TOP_OF'});
  $this->setIsSubPackage($data{'IS_SUBPACKAGE'});
  $this->setSubDir($data{'SUB_DIR'});
  $this->setMagicMode($data{'MAGIC_MODE'});
  $this->setDesc($data{'DESC'});
  $this->setSize($data{'SIZE'});
  $this->setBuildString($data{'BUILD_STRING'});
  if ($data{'BUILD_STRING'}) {
    my ($make_state,$qa_level,$purpose,$change) = ($data{'BUILD_STRING'} =~ /-(\d)(\d)(\d)-(\d{3}-\d{3})$/);
    $change =~ s/-//g;
    $change =~ s/^0*//;
    $this->setChangeList($change);
    if ($make_state == 1) {
      $make_state = 'productive';
    } else {
      $make_state = 'test/development';
    }
    $this->setMakeState($make_state);
    if ($qa_level == 0) {
      $qa_level = 'DEV';
    } elsif ($qa_level == 1) {
      $qa_level = 'COR';
    } elsif ($qa_level == 2) {
      $qa_level = 'RAMP';
    } elsif ($qa_level == 3) {
      $qa_level = 'HOT';
    }
    $this->setQALevel($qa_level);
    my @purpose;
    if ($purpose == 0) {
      push @purpose,'no classification';
    } else {
      if (($purpose & 1) != 0) {
        push @purpose,'OLTP';
      }
      if (($purpose & 2) != 0) {
        push @purpose,'liveCache';
      }		
    }
    $this->setPurpose(join(' & ',@purpose));
  }
	
	
  $this->setMakeId($data{'MAKE_ID'});
  unless($^O=~/mswin/i){
    $this->setUser($data{'DEFAULT_USER'});
    $this->setGroup($data{'DEFAULT_GROUP'});
  }
  $this->setUntgz($data{'UNTGZ'});

  if ($data{'TEST_FILE'}) {
    $this->setTestFile($data{'TEST_FILE'});
    $this->setMagicString($data{'MAGIC'});
  }
  if (defined $data{'DISPLAY_NAME'}) {
    $this->setDispName($data{'DISPLAY_NAME'});
  } else {
    $this->setDispName($data{'PACKAGE_NAME'});
  }

  my $scriptName = $data{'SCRIPT'};
  if ($scriptName) {
    my $text = readFileInTGZ($data{'UNTGZ'},$scriptName);
    $this->setScript(\$text);
  }
	
  my $archiveName = $this->getArchiveName();
  my $archiveDir = $this->getArchiveDir();
  if($archiveDir and $archiveName){
    my $archive = catfile($archiveDir,$archiveName);
    my $md5sum = MD5Sum($archive);
    if (defined $md5sum && $md5sum =~ /^[0-9a-f]{32}$/) {
      $this->setCheckSum($md5sum);
    }
    my %md5sums;
    my $fileListName = $data{'FILELIST'};
    if ($fileListName) {
      $this->debugit(4,"Reading file list $fileListName");
      my ($retVal,$filelist) = readAFileInTGZ($data{'UNTGZ'},$fileListName);
      $retVal and $this->logit("Error reading $archiveName: $retVal");
      
      foreach my $file (split("\n",$filelist)) {
        if( $file=~/^\s*"(\S.*\S)"\s*([0-9a-z]{32})\s*$/){
          $md5sums{$1}=$2;  
        }
        elsif($file =~/^\s*#/){
        }
        elsif($file =~/\S/){
          $this->debugit(1,"Invalid entry in file list: $file");
        }
      }
    }
    $this->setFileList(\%md5sums);
  }
  $this->setPaths([]);
  $this->setOpts([]);
  $this->MergeKeys($this->myKeys());
}



sub preinstall {return 0;}
sub preuninstall {return 0;}
sub preunpack {return 0;}
sub postprepare {return 0;}
sub postinstall {return 0;}
sub postuninstall {return 0;}
sub register {return 0;}
sub unregister {return 0;}
sub framestart {return 0;}
sub framestop {return 0;}


sub uninstall{
  my $this = shift;
  my @undeleted;
  my $regmd5sums = $this->getFileList();
  foreach my $file (keys(%$regmd5sums)) {
    my $fullpath = catfile($this->getMainPath(),$file);
    -f $fullpath or $this->warnit("Cannot delete file \"$fullpath\": no such file\n") and next; 
    if ($regmd5sums->{$file} eq MD5Sum($fullpath)){
      if(unlink($fullpath)){
      } else {
        $this->warnit("Not deleted: \"$fullpath\": $!\n");
      }
    } else {
      $this->warnit("Modified:      \"$fullpath\"\n");
    }
  }
}

my %packageClasses =
  (
   '^Base$' => "SAPDB::Install::Templates::PackageBase",
   '^Database Manager$' => "SAPDB::Install::Templates::PackageDBM",
   '^Server Utilities$' => "SAPDB::Install::Templates::PackageServerUtil",
   '^Database Kernel$' => "SAPDB::Install::Templates::PackageDBKernel",
   '^SQL Studio$' => "SAPDB::Install::Templates::PackageSQLS",
   '^CPC Base$' => "SAPDB::Install::Templates::PackageCPCBase",
   '^DB Analyzer$' => "SAPDB::Install::Templates::PackageDBAnalyzer",
   '^Database Connectivity$' => "SAPDB::Install::Templates::PackageDBConnect",
   '^JDBC$' => "SAPDB::Install::Templates::PackageJDBC",
   '^LC SDK$' => "SAPDB::Install::Templates::PackageLCSDK",
   '^PCR \d\d\d\d$' => "SAPDB::Install::Templates::PackagePCR",
   '^CPC SDK \d\d\d\d$' => "SAPDB::Install::Templates::PackageCPC",
   '^DB Analyzer$' => "SAPDB::Install::Templates::PackageDBAna",
   '^Redist Python$' => "SAPDB::Install::Templates::PackagePython",
   '^Webtools$' => "SAPDB::Install::Templates::PackageWebtools",
   '^XML Indexing Engine$' => "SAPDB::Install::Templates::PackageXML",
   '^APO COM$' => "SAPDB::Install::Templates::PackageAPO"
  );


sub classForName{
  my ($name)  = @_;
  my $retVal;
  foreach my $key (keys %packageClasses) {
    if ($name =~ /$key/) {
      $retVal = $packageClasses{$key};
      last;
    }
  }
  return $retVal;
}



sub new{
  my ($data,$instConfig) = @_;
  my $keys={
            'iD' => {},
            'interfaceVersion' => {},
            'name' => {},       # unique package type identifier
            'version' => {},
            'packageVersion' => {},
            'mode' => {},
            'type' => {},
            'minVersion' => {},
            'profiles' => {},
            'topOf' => {},
            'require' => {},
            'required' => {},
            'fileList' => {},
            'script' => {},
            'testFile' => {},
            'magicMode' => {},
            'allowSkip' => {},
            'paths' => {},
            'mainPath' => {},
            'archiveName' => {},
            'archiveDir' => {},
            'opts' => {},
            'regData'=> {},
            'skip' => {},
            'user' => {},
            'uID' => {},
            'group' => {},
            'gID' => {},
            'magicString' => {},
            'untgz' => {},
            'isSubPackage' => {}, # boolean value 
            'subDir' => {},     # part of package path for subpackage
            'subPackages' => {}, # array of subpackage objects
            'parentPackage' => {}, # parent package object, only if IsSubPackage is set 
            'flushIntoReg' => {}, # flag to activate / deactivate changes in regpackobj, needed for prepare phase
            'checkSum' => {},   # checksum of archive file
            'withParent' => {}, # subpackage will be installed together with its parent package
            'subPackObjs' => {}, # list of subpackage objects that will be installed togeter with this package
            'dispName' => {},   # name of pakages visible from outside
            'ignoreBusyFiles' => {}, # list of files which won't break unpack check
            'size' => {},
            'buildString' => {},
            'makeId' => {},
            'qALevel' => {},
            'purpose' => {},
            'changeList' => {},
            'makeState' => {},
            'indepData' => {},
            'indepProg' => {},
            'sysProgPath' => {},
            'desc' => {},
            'fSUsage' => {},
            'instTemplate' => {},
            'date' => {}
           };
  my $this = {'KEYS' => $keys};
  my $aName =  $data->{'PACKAGE_NAME'}; 
  my $aClass = classForName($aName);
  if ($aClass) {
    bless $this,$aClass;
  } else {
    bless $this;
  }
  $instConfig and $this->SetInstTemplate($instConfig);
  $this->initData($data);
  return  $this;
}



sub prepare{
  my ($this) = @_;
  my $result = $this->preinstall(); 
  if($result){return $result;}
  $result  = $this->preprepare();
  if($result){return $result;}
  $result = $this->postprepare();
  return $result;
}

sub preprepare{
  my $this = shift;
  my $result = $this->handleTestExtraction();
  unless($result) {
    my $sp = $this->handleSpaceCheck();
    if ($sp) {
      $result =  $sp;
    }
  }
  return $result;
}

sub handleTestExtraction{
  my $this = shift;
  my $retVal=0;
  my $mainPath= $this->getMainPath();
  my $mk = $this->pmakedir($mainPath,0555);
  if ($mk) {
    $retVal =    "\nError creating directory '$mainPath': $mk\n"; 
  } else {
    my $testResult = $this->unpackIt(1);
    if ($testResult) {
      $retVal = "Error in test extraction:\n$testResult\n";
    }
  }
  return $retVal;
}


sub handleSpaceCheck{
  my $this = shift;
  my $fsusage =  $this->getFSUsage();
  my $retVal = 0;
  my %fsall;
  foreach my $fs (@$fsusage) {
    my $name = $fs->{'FSNAME'};
    my $afs = $fsall{$name};
    unless(defined $afs){
      $afs = { 'AVAIL' => 0, 'NEEDED' => 0, 'FSNAME' => $name};
      $fsall{$name}=$afs;
    }
    my $avail = $fs->{'AVAIL'};
    my $needed = $fs->{'NEEDED'};
    my $allNeeded =  $afs->{'NEEDED'}+$needed;
    $afs->{'NEEDED'} = $allNeeded;
    $afs->{'AVAIL'} = $avail;
  }
  foreach my $fs (values %fsall) {
    my $avail = $fs->{'AVAIL'};
    my $needed = $fs->{'NEEDED'};
    my $name = $fs->{'FSNAME'};
    if ($needed > $avail) {
      $retVal =  "Not enough space on $name  available $avail, needed $needed\n";
    } else {
    }
  }
  return $retVal;
}




sub displayString{
  my $this = shift;
  my $aName = $this->getDispName ? $this->getDispName : $this->getName;
#  my $aVersion = $this->getVersion ? $this->getVersion : "";
  my $tmpstring=$aName;
  my $aPath = $this->getMainPath();
  if($this->getMode){
    $tmpstring .= ' ('.$this->getMode.' BIT)' if $this->getMode =~ /32|64/;
  }
#  $tmpstring .= " $aPath";
  return  $tmpstring;
}





sub GetAbsoluteFileList{
  my ($this)=@_;
  my %fileList = %{$this->getFileList}; # md5sums of files in tgz archive
  my $mainPath =  $this->getMainPath();
  my @retVal;
  foreach(keys %fileList){
    my $aPath = normalizeAPath(catfile($mainPath,$_));
    push @retVal,($aPath);
  }
  return @retVal;
}

sub backupFile{
  my $file = shift;
  my $backupFile = $file;
  my $counter = 1;
  while(-f $backupFile){
    $backupFile = $file . ".bak." . $counter;
    $counter++;
  }
#  rename ($file,$backupFile);
  my $result = copyFile($file,$backupFile);
  return $result;
}



  # $tgzname: full parth to archive
  # $destdir: destination directory
  # $testrun: if true no files will be extracted, only try to open for writing
  # @list: empty -> all files of archive will be extracted; set -> only files in @list will be extracted
sub unpackIt{
  #sub extract archive
  my ($this,$testrun)=@_;
  my @list = keys (%{$this->getFileList()});
  my $listCount = @list;
  my $name = $this->getName();
  $this->debugit(1,"Start extracting files for $name ($listCount files).");
  my $destdir = $this->getMainPath();
  my $cwd;
  my $tgzhandle = $this->getUntgz;
  my $action="";
  $tgzhandle->SetOwner($this->getUser,$this->getGroup) unless($^O =~ /mswin/i);
  $tgzhandle->Rewind();
  if ($testrun) {
    $tgzhandle->SetReadOnly(1);
    $action = "Testextracting";
  } else {
    $tgzhandle->SetReadOnly(0);
    $action = "Extracting";
  }
  unless(defined $destdir and -d $destdir){
    my $displ = defined $destdir ? $destdir : "<UNDEFINED>";
    return "Destination Directory \"$displ\" not found";
  }
  unless($tgzhandle->SetDestDir($destdir)==0){
    return "Cannot set destination directory \"$destdir\": ".join(': ',$tgzhandle->GetErr);
  }
  $this->debugit(1,"Destination directory for unzipping is '$destdir'\n");
  my $info = "start"; 
  while ($info) {
    $info = $tgzhandle->Next();
    unless ($info) {
      last;
    }
    $_=$info;
    if (/^d/) {
      $tgzhandle->ExtractFile;
      next;
    }
    /([^\ ]*)$/	and my $file=$1;
    $file=~s/^\.\///;           #remove "./" at begin of filename
    my ($rc,$errtext);
    if (@list) {
      foreach my $extfile (@list) { 
        if ($extfile eq $file) {
          $tgzhandle->ExtractFile;
          $this->debugit(4,"$action '$file'\n");
        }
      }
    } else {
      $tgzhandle->ExtractFile($testrun);	
      $this->debugit(1,"$action '$file'\n");
    }
    if ($tgzhandle->GetErr) {
      return "Extraction error: ".join(': ',$tgzhandle->GetErr);
    }
  }
  my @fsusage = $tgzhandle->GetFSUsage; 
  $this->setFSUsage(\@fsusage); 
  return 0;
}

sub install{
  my $this = shift;
  my $result = $this->doUpdate();
  return $result;
}
sub doUpdate{
  my ($this,$oldPackage)  = @_;
  my $result = 0;
  if($oldPackage){
    $result = $oldPackage->unregister();
    if ($result) {$this->warnit("Unregister failed:\n" . $result);}
  }
  $result = $this->update($oldPackage) and return "Error in update: \n" . $result;
  $result =   $this->postinstall() and return "Error in postinstall:\n" .$this->getDispName . "\n". $result;
  $result =   $this->register() and return "Error in register:\n".$result;
  return $result;
}
sub update{
  my ($this,$oldPackage)=@_;
  my $result = 0;
  my $oldFileList = {};
  if($oldPackage){
    $oldFileList = $oldPackage->getFileList();
  }
  my $newFileList = $this->getFileList; 
  my $mainPath = $this->getMainPath();
  #Backup changed files:
  foreach my $aKey (keys %$oldFileList) {
    my $aFile = catfile($mainPath,$aKey);
    if (-f $aFile) {
      my $newSum=MD5Sum($aFile);
      my $fileInfo = $oldFileList->{$aKey};
      my $oldSum ; 
      if(ref $fileInfo eq 'HASH'){
        $oldSum = $fileInfo->[0];
      }
      else{
        $oldSum = $fileInfo;
      }
      if ( $oldSum ne $newSum) {
        $result = backupFile($aFile);
        if ($result) {
          return $result;
        }
      }
    }
  }
  #Delete old files
   foreach my $aKey (keys %$oldFileList) {
     unless( exists $newFileList->{$aKey}){
       my $aFile = catfile($mainPath,$aKey);
       if(-f $aFile){
         unlink ($aFile);
       }
     }
   }
  $result =  $this->preunpack();
  if ($result) { return $result; }
  $result = $this->unpackIt(0);
  if ($result) { return $result; }
  return $result;
}



sub pmakedir{
  my ($this,$dir,$perm) = @_;
  my $retVal=0;
  $perm = 0777 unless $perm;
  unless($dir){
    return "Directory not specified";
  }
  makedir($dir,$perm,$this->GetUID(),$this->GetGRID());
  unless(-d $dir){
    $retVal = $!;
  }
  return $retVal;
}

sub GetPaths{
  return ();
}



sub GetDbmcliExe{
  my $this = shift;
  my $dbmcli ; 
  if($^O =~ /mswin/i){
    $dbmcli = catfile($this->getIndepProg(),"pgm","dbmcli");
  }
  else{
    $dbmcli = catfile($this->getIndepProg(),"bin","dbmcli");
  }
  return $dbmcli;
}


sub call_dbmcli{
  my ($this,$command,$result) = @_;
  $command = $this->GetDbmcliExe() . $command;
  my $retVal=0;
  my $error=1;
  my $output="";
  my $instConfig = $this->getInstTemplate();
  $this->logit("[".$command."]");
  callSystem($command, sub{
               my $line = $_[0];
               unless(defined $line){
                 return ;
               }
               $output .= $line;
               $result and push @$result,($line);
               if ( $line =~ /OK/) {
                 $error = 0;
               }
               return 1;
             }
            );
  if ($error) {
    $retVal = "ERROR in '$command'\n" . $output;
  }
  return $retVal;
}




sub call_dbmcli_script{
  my ($this,$script,$user,$db) = @_;
  my $retVal = 0;
  $user = $user ? "-u $user" : '';
  $db = $db ? "-d $db" : '';
  my $tmpDir = $ENV{'TMP'} ? $ENV{'TMP'} : $ENV{'TEMP'};
  -d $tmpDir or return "Cannot write to temporary directory $tmpDir";
  my $tmpFile = catfile($tmpDir,"tmpscript.dbm");
  open TMPSCRIPT , ">$tmpFile" or return "Cannot open $tmpFile: $!";
  print TMPSCRIPT $script;
  close TMPSCRIPT;
  my $cmd = $this->GetDbmcliExe() . " $user $db -i \"$tmpFile\"";
  $this->logit("[".$cmd."]");
  $this->logit("\nBEGIN SCRIPTCONTENTS\n${script}END SCRIPTCONTENTS");
  #The first error of the script is reported:
  my ($scriptError,$errorCode,$output) = (0,0,'');

  $retVal = callSystem($cmd, sub{
               my ($line) = @_;
               $line or return;
               $output .= $line;
               $line =~ /^\s*-?\d+,ERR/ and  not $errorCode and $errorCode = $line;
               $line =~ /^\s*ERR/ and  $scriptError = 1;
             }
            ); 
  $scriptError and $this->logit("\nBEGIN SCRIPTOUTPUT\n${output}END SCRIPTOUTPUT");
  $errorCode and return $errorCode;
  $scriptError and return "Unknown error in script occured.";
  return $retVal;
}







my $x_server_results = {
               '19710' => "INFO: Service 'XServer' removed successfully" , 
               '19709' => "INFO: Service 'XServer' installed successfully" , 
               '19712' => "INFO: Service 'XServer' is already installed" , 
               '18640' => "ERROR: XSERVER not active" , 
               '19720' => "INFO: XSERVER started" , 
               '18641' => "ERROR: XSERVER is already running!" ,
               '18581' => "ERROR: Service 'XServer' not installed!"
               };
sub call_x_server{
  my ($this,$arg,$retVals) = @_;
  my $retVal=0;
  my $exec  = catfile($this->getIndepProg(), "bin","x_server") ;
  my $command = $exec . " $arg";
  my $instConfig = $this->getInstTemplate();
  $this->logit("[".$command."]");
  callSystem($command,
             sub{
               my $line = $_[0];
               $this->logit($line);
               if ( $line =~ /(^\d+)\s.*$/
                    or $line =~ /^ERR\s+(\d+)/
                  ) {
                 my $id = $1;
                 my $error = 1;
                 foreach (@$retVals) {
                   if ($id eq $_ ) {
                     $error = 0;
                   }
                 }
                 if ($error) {
                   $retVal = $line;
                 }
               }
               return 1;
             });
  return $retVal;
}




package SAPDB::Install::Templates::PackageBase;
use SAPDB::Install::Registry;
use SAPDB::Install::System;
use SAPDB::Install::Templates::Util;
use SAPDB::Install::System::Win32::Registry;
use SAPDB::Install::DBMCmd;
unless($^O =~ /mswin/i){
  use SAPDB::Install::System::Unix;
  use SAPDB::Install::System::Unix::Dir;
}

our @ISA = qw (SAPDB::Install::Templates::InstUnit);


my %datapath;
my %progpath;
$datapath{'name'} = 'independent data path';
$datapath{'mode'}=0775;
$datapath{'opt'}='indep_data=s';
$progpath{'name'} = 'independent program path';
$progpath{'opt'} = 'indep_prog=s';
$progpath{'update'}=0;

my  $initPath = '/usr/spool/sql/ini/SAP_DBTech.ini';
my $globPath = '/etc/opt/sdb';


sub initData{
  my ($this,$rhData)=@_;
  $this->SUPER::initData($rhData);
  $datapath{'default'} = catfile(getAppDataPath(),'sdb/data');
  $progpath{'default'}=catfile(getSysProgPath(),'sdb/programs');
}


sub postuninstall{
  my $this = shift;
  $this->SUPER::postuninstall();
  if($^O =~ /mswin/i){
    my $key;
    my $tech =  'SOFTWARE\SAP\SAP DBTech';
    my $sap = 'Software\SAP';
    my $sapKey = 'SAP DBTech';
    my $warn = 0;
    my $res  = RegOpenKeyEx(&HKEY_LOCAL_MACHINE,$tech, 0, &KEY_ALL_ACCESS , $key );  
    if($res){ $this->warnit(  "WARNING: Could not  open registry key $tech.\n"); }
    $res = RegDeleteValue($key,'IndepData'); 
    if($res){ $this->warnit(  "WARNING: Could not  delete registry value 'IndepData'.\n"); }
    $res = RegDeleteValue($key,'IndepPrograms'); 
    if($res){ $this->warnit(  "WARNING: Could not  delete registry value 'IndepPrograms'.\n"); }
    $res = RegCloseKey($key); 
    if($res){ $this->warnit(  "WARNING: Could not  closing registry key $tech.\n"); }
    $res = RegOpenKeyEx(&HKEY_LOCAL_MACHINE, $sap, 0, &KEY_ALL_ACCESS , $key); 
    if($res){ $this->warnit(  "WARNING: Could not  open registry key $sap.\n"); }
    $res = RegDeleteKey($key,$sapKey); 
    if($res){ $this->warnit(  "WARNING: Could not  delete registry key $sapKey.\n"); }
    $res = RegCloseKey($key); 
    if($res){ $this->warnit(  "WARNING: Could not close registry key.\n"); }
  }
  else{
    -f $initPath and unlink($initPath); 
    -f $initPath and $this->warnit("WARNING: Cannot delete '$initPath'\n");
    -f $globPath and unlink($globPath); 
    -f $globPath and  $this->warnit("WARNING: Cannot delete '$globPath'\n");
  }
  return 0;
}


sub GetPaths{
  return (\%datapath,\%progpath);
}

sub myKeys{
  return {
          mainPath =>{
                      'DEFAULT' => '${indepProg}'
                     }
         };
}

sub GetServicePath{
  my $service_path;
  if ($^O=~/.*mswin.*/i) {
    my %hash = readValues(0,"SYSTEM\\CurrentControlSet\\Control\\Windows");
    if ($hash{'SystemDirectory'}) {
      $service_path="$hash{'SystemDirectory'}/drivers/etc/services";
      $service_path=~s/\\/\//g;			
    }
  } else {
    $service_path="/etc/services";
  }
  return $service_path;
}

sub unregister{
  my $this = shift;
  $this->SUPER::register();
#   unless($^O =~ /mswin/i){
#     -f '/etc/opt/sdb' and unlink '/etc/opt/sdb'; 
#     -f '/etc/opt/sdb' and $this->warnit("Could not delete /etc/opt/sdb: $!");
#   }
  return 0;
}

sub register{
  my $this = shift;
  $this->SUPER::register();
  if($^O =~ /mswin/i){
    return $this->registerWin();
  }
  else{
    return $this->registerUx();
  }
}
sub registerWin{
  my $this = shift;
  my $retVal = 0;
  my $command = " -s dbm_setpath IndepProgPath \\\"". $this->getIndepProg() . "\\\"";
  $retVal = $this->call_dbmcli($command);
  if($retVal){return $retVal;}
  $command = " -s dbm_setpath IndepDataPath \\\"" . $this->getIndepData() . "\\\"";
  $retVal = $this->call_dbmcli($command);
  return $retVal;
}

sub registerUx{
  my $this = shift;
  my $retVal =0;
  my $conf = {};
  my $indepProg = $this->getIndepProg();
  my $indepData = $this->getIndepData();
  my $sdbconfig = catfile($indepProg,"bin","sdbconfig");
  my $user = $this->getUser();
  my $group = $this->getGroup();
  if (-f $globPath && open (CONFIG, $globPath)) {
    while (my $line = <CONFIG>) {
      chomp $line;
      foreach my $key 
        ('IndepData', 'IndepPrograms', 'SdbOwner', 'SdbGroup') {
          if ($line =~ /^$key=/) {
            $line =~ s/^$key=//;
            $conf->{$key} = $line;
          }
        }
    }
    close (CONFIG);
  }
  unless (
          exists $conf->{IndepData} && $conf->{IndepData} eq $indepData &&
          exists $conf->{IndepPrograms} && $conf->{IndepPrograms} eq $indepProg &&
          exists $conf->{SdbOwner} && $conf->{SdbOwner} eq $user &&
          exists $conf->{SdbGroup} && $conf->{SdbGroup} eq $group) {
    my $path = '/etc/opt';
    makedir($path,0555) unless -d $path;
    my $res ='';
    callSystem("$sdbconfig IndepData=$indepData",sub{$res.=$_[0];}) == 0
      or return "ERROR: Failed command was '$sdbconfig IndepData=$indepData'\n$res";
    $res='';
    callSystem("$sdbconfig IndepPrograms=$indepProg",sub{$res.=$_[0];}) == 0
      or return "ERROR: Failed command was '$sdbconfig IndepPrograms=$indepProg'\n$res";
    $res='';
    callSystem("$sdbconfig SdbOwner=$user",sub{$res.=$_[0];})== 0
      or return "ERROR: Failed command was '$sdbconfig SdbOwner=$user'\n$res";
    $res='';
    callSystem("$sdbconfig SdbGroup=$group",sub{$res.=$_[0];})== 0
      or return "ERROR: Failed command was '$sdbconfig SdbGroup=$group'\n$res";
    $res='';
  }
  return $retVal;
}

sub writeServices{
  my $service_path = GetServicePath;
  open(SRV,$service_path) or return $!;
  my %port=(
            'sql6' => '7210/tcp',
            'sapdbni72' => '7269/tcp'
           );
  my %service_exist;
  while (<SRV>) {
    foreach my $service (keys(%port)) {
      if (/^$service\s/) {
        $service_exist{$service}=1;
      }
    }
  }
  close(SRV);
  # append SAP DB services to etc/services
  open(SRV,">>$service_path") or return $!;
  foreach my $service (keys(%port)) {
    unless ($service_exist{$service}==1) {
      print SRV "$service\t\t$port{$service}\n";
    }
  }
  close(SRV);
  return 0;
}

sub preinstall(){
  my $this = shift;
  my $retVal =0;
  $retVal = writeServices;
  $datapath{'value'}=$this->getIndepData;
  $progpath{'value'}=$this->getIndepProg;
  if($retVal){return $retVal;}
  return $retVal;
}


#Base package: postinstall
sub postinstall{
  my $this = shift;
  my $retVal=0;
  $retVal = $this->SUPER::postinstall(@_) and return $retVal;
  my $instConfig = $this->getInstTemplate();
  my $indepData = $instConfig->getIndepData();
  my $mainPath = $this->getMainPath();
  my $grid = $this->GetGRID();
  my $uid = $this->GetUID();
  my $version = $this->getVersion();
  my $aDir = catfile($indepData,'config');
  not -d $aDir and $retVal = $this->pmakedir($aDir,0770) and  return "Error creating directory $aDir:\n".$retVal;
  $aDir = catfile($indepData,'config/install');
  not -d $aDir and $retVal = $this->pmakedir($aDir,0770) and  return "Error creating directory $aDir:\n".$retVal;
  $aDir = catfile($indepData,'config/wrk');
  not -d $aDir and $retVal = $this->pmakedir($aDir,0770) and  return "Error creating directory $aDir:\n".$retVal;
  unless ($^O =~ /mswin/i){
    my $aFile = catfile($mainPath,'bin/starter');
    chmod (0500, $aFile) or return "Cannot chmod directory '$aFile' to 0500:\n$!";
    $aFile = catfile($mainPath,'bin/starter/sdbstarter');
    chown (0, $grid, $aFile) or return "Cannot chown file '$aFile' to 0,$grid:\n$!";
    chmod (04510, $aFile) or return "Cannot chmod file '$aFile' to 04510:\n$!";
    # create directory tree <data>/...
    foreach my $dir ('config','ipc','fifo','diag','dbspeed','pipe','pid','ppid','pid/serverpids') {
      my $mode = $dir eq 'config' ? 0775 :
        $dir eq 'ipc'    ? 0775 :
          $dir eq 'fifo'   ? 0775 :
            0770;
      my $longdir = catfile($indepData,$dir);
      if (-d $longdir) {
        chmod($mode,$longdir) or return "Cannot chmod '$longdir' to ".sprintf ("%04o", $mode).":\n$!";    
        chown($uid,$grid,$longdir) or return "Cannot chown '$longdir' to $uid,$grid:\n$!";
      } else {
        $this->pmakedir($longdir,$mode,$uid,$grid) and return "Cannot create directory '$longdir':\n$!" ;
      }
    }
    my $longdir = "/usr/spool/sql";
    if (-d $longdir) {
      chmod(0775,$longdir) or return "Cannot chmod '$longdir' to 775\n";
      chown($uid,$grid,$longdir) or return "Cannot chown '$longdir' to $uid,$grid:\n$!";    
    } else {
      $this->pmakedir ($longdir,0775,$uid,$grid) and return "Cannot create directory '$longdir'\n$!"; 
    }
    my $db_ini = catfile($indepData,'config/Databases.ini');
    unless(-f $db_ini){
      open(DBR,'>'.$db_ini) or return "cannot create '$db_ini':\n$!";
      close(DBR);
    }
    chmod(0660,$db_ini)or return "Cannot chmod '$db_ini' to 0660:\n$!";
    chown($uid,$grid,$db_ini) or return "Cannot chown '$db_ini' to $uid,$grid:\n$!";
    foreach my $dir ('ipc','fifo','diag','dbspeed','pid','ppid','pipe') {
      my $fullname = catfile("/usr/spool/sql",$dir);
      my $ais = catfile($indepData,$dir);
      if (-l $fullname) {
        unlink($fullname);
      } else {
        $this->delTree($fullname);
      }
      symlink ($ais, $fullname) or return "Cannot create symlink '$fullname' -> '$ais'.\n$!"; 
      lchown ($uid,$grid,$fullname) or return "Cannot chown of symlink '$fullname' to $uid,$grid.\n"; 
    }
    $longdir = "/usr/spool/sql/ini";
    if (-d $longdir) {
      chmod(0777,$longdir) or return "Cannot chmod '$longdir' to 0777:\n$!";
      chown($uid,$grid,$longdir) or return "Cannot chown '$longdir' to $uid,$grid.\n$!";
    } else {
      $this->pmakedir ($longdir,0777) and return "Cannot create directory '$longdir'\n"; 
    }

    my $inifile = "/usr/spool/sql/ini/SAP_DBTech.ini";
    unless (-f $inifile) {
      open (INI, ">$inifile") or return "cannot create SAP_DBTech.ini:\n$!";
      print INI "[Globals]\n";
      print INI "IndepData=$indepData\n";
      print INI "IndepPrograms=$mainPath\n";
      close INI;
    }

    chown ($uid,$grid,$inifile) or return "Cannot chown '$inifile' to $uid,$grid.\n" ;
    chmod (0666,$inifile) or return "Cannot chmod '$inifile' to 0666\n"; 
    if ($this->crelease2num($version) < $this->crelease2num('7.5.00.00')) {
      #  setServerRootPermissions ($packobj->User) or return "Cannot change permissions of server software root directories\n";
    }
  }
  return 0;
}


package   SAPDB::Install::Templates::PackageDBM;
use SAPDB::Install::Templates::Util;
use SAPDB::Install::Registry ;
use SAPDB::Install::System::Win32::Registry ;

our @ISA = qw (SAPDB::Install::Templates::InstUnit);



sub myKeys{
my $instScript = <<"INSTFILE";
[InstallShield Silent]
Version=v6.00.000
File=Response File
[File Transfer]
OverwrittenReadOnly=NoToAll
[\{38E77B11-839B-4138-B002-598026C48116\}-DlgOrder]
Dlg0=\{38E77B11-839B-4138-B002-598026C48116\}-SdWelcome-0
Count=5
Dlg1=\{38E77B11-839B-4138-B002-598026C48116\}-SdAskDestPath-0
Dlg2=\{38E77B11-839B-4138-B002-598026C48116\}-SdSelectFolder-0
Dlg3=\{38E77B11-839B-4138-B002-598026C48116\}-SdStartCopy-0
Dlg4=\{38E77B11-839B-4138-B002-598026C48116\}-SdFinish-0
[\{38E77B11-839B-4138-B002-598026C48116\}-SdWelcome-0]
Result=1
[\{38E77B11-839B-4138-B002-598026C48116\}-SdAskDestPath-0]
szDir=\${winInstPath}
Result=1
[\{38E77B11-839B-4138-B002-598026C48116\}-SdSelectFolder-0]
szFolder=MySQL MaxDB
Result=1
[\{38E77B11-839B-4138-B002-598026C48116\}-SdStartCopy-0]
Result=1
[Application]
Name=Database Manager
Version=\${version}
Company=SAP AG
Lang=0009
[\{38E77B11-839B-4138-B002-598026C48116\}-SdFinish-0]
Result=1
bOpt1=0
bOpt2=0
[\{765537B8-DA4C-480E-917E-E25CAA26CE99\}-DlgOrder]
Count=0
[\{B464AF4C-EFBF-4488-82DA-E1CF26531D34\}-DlgOrder]
Count=0
[\{9829D65A-1914-418C-98B6-48008C42FFD2\}-DlgOrder]
Count=0
[\{4E709CE6-A288-462A-AE8B-E76CB9AD0342\}-DlgOrder]
Count=0
INSTFILE
  my $uninstScript = <<"UNINSTFILE";
[InstallShield Silent]
Version=v6.00.000
File=Response File
[File Transfer]
OverwrittenReadOnly=NoToAll
[\{38E77B11-839B-4138-B002-598026C48116\}-DlgOrder]
Dlg0=\{38E77B11-839B-4138-B002-598026C48116\}-SdWelcomeMaint-0
Count=3
Dlg1=\{38E77B11-839B-4138-B002-598026C48116\}-SprintfBox-0
Dlg2=\{38E77B11-839B-4138-B002-598026C48116\}-SdFinish-0
[\{38E77B11-839B-4138-B002-598026C48116\}-SdWelcomeMaint-0]
Result=303
[\{38E77B11-839B-4138-B002-598026C48116\}-SprintfBox-0]
Result=1
[Application]
Name=Database Manager
Version=\${version}
Company=SAP AG
Lang=0009
[\{38E77B11-839B-4138-B002-598026C48116\}-SdFinish-0]
Result=1
bOpt1=0
bOpt2=0
[\{765537B8-DA4C-480E-917E-E25CAA26CE99\}-DlgOrder]
Count=0
[\{B464AF4C-EFBF-4488-82DA-E1CF26531D34\}-DlgOrder]
Count=0
[\{9829D65A-1914-418C-98B6-48008C42FFD2\}-DlgOrder]
Count=0
[\{4E709CE6-A288-462A-AE8B-E76CB9AD0342\}-DlgOrder]
Count=0
UNINSTFILE

  my $updateScript = <<"UPDATEFILE";
[InstallShield Silent]
Version=v6.00.000
File=Response File
[File Transfer]
OverwrittenReadOnly=NoToAll
[{38E77B11-839B-4138-B002-598026C48116}-DlgOrder]
Dlg0={38E77B11-839B-4138-B002-598026C48116}-SdWelcome-0
Count=3
Dlg1={38E77B11-839B-4138-B002-598026C48116}-SdStartCopy-0
Dlg2={38E77B11-839B-4138-B002-598026C48116}-SdFinish-0
[{38E77B11-839B-4138-B002-598026C48116}-SdWelcome-0]
Result=1
[{38E77B11-839B-4138-B002-598026C48116}-SdStartCopy-0]
Result=1
[Application]
Name=Database Manager
Version=\${version}
Company=SAP AG
Lang=0009
[{38E77B11-839B-4138-B002-598026C48116}-SdFinish-0]
Result=1
bOpt1=0
bOpt2=0
[{AAE16ED0-5751-4059-A3AC-C8CEEA9EC206}-DlgOrder]
Count=0
[{CC2821B5-2907-4B12-AA15-73F545F171D2}-DlgOrder]
Count=0
[{21D3C994-574E-4D5C-8B5A-13A8CB209AAB}-DlgOrder]
Count=0
[{38C6A71B-4176-49C2-814E-D078808CCF5B}-DlgOrder]
Count=0
UPDATEFILE
  return {
          winInstPath =>{ 'DEFAULT' => '${indepProg}\DBM' },
          winExePath =>{ 'DEFAULT' => '${indepProg}\misc' },
          mainPath =>{ 'DEFAULT' => '${indepProg}' },
          instScript =>{ 'DEFAULT' => $instScript },
          uninstScript =>{ 'DEFAULT' => $uninstScript },
          updateScript =>{ 'DEFAULT' => $updateScript },
          exePrefix =>{ 'DEFAULT' => 'DBM' },
          versionKey => { 'DEFAULT' => 'SOFTWARE\SAP AG\Database Manager\7.6'},
          winID => { 'DEFAULT' => 'DBMGui3.Application'},
          classID => {},# must be read from registry
          classIDKey => { 'DEFAULT' => '${winID}\Clsid'},
          uninstKey => { 'DEFAULT' => 'SOFTWARE\Microsoft\Windows\CurrentVersion\Uninstall\${classID}'}
         };
}






sub getInstalledVersionForRegistryKey(){
  my ($this,$aRegistryKey) = @_;
  my  %aValueHash=readValues(0,$aRegistryKey);
  my $installedVersion = $aValueHash{'Version'};
  return $installedVersion;
}

sub getInstalledVersionViaReg(){
  my ($this) = @_;
  my $aVersionKey = $this->getVersionKey();
  my $installedVersion = $this->getInstalledVersionForRegistryKey($aVersionKey);
  if(not $installedVersion){
    my $r74=$aVersionKey;
    $r74 =~ s/\d\.\d$/7\.4/;
    my $r75 = $aVersionKey;
    $r75 =~ s/\d\.\d$/7\.5/;
    my $r76 = $aVersionKey ;
    $r76 =~ s/\d\.\d$/7\.6/;
    foreach my $key ($r74,$r75,$r76){
      unless($key eq $aVersionKey){
        $installedVersion = $this->getInstalledVersionForRegistryKey($key);
        if($installedVersion){
          last;
        }
      }
    }
  }
  return $installedVersion;
}


sub writeLog{
  my ($message) = @_;
  chomp $message;
  $message =~ s/\r$//;
  my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = localtime(time);
  $mon += 1;
  $year += 1900;
  my $dateString = sprintf("%02d/%02d/%02d %02d:%02d:%02d", $mon,$mday,$year % 100,$hour,$min,$sec);
  $message = "[".$dateString."] " . $message;
  $message =~ /\n$/ or $message = $message . "\n";
#because SDBINST
  $SAPDB::Install::Values::log->SetMsg($message);
#endbecause
  #ifdef SCRIPTING
#  print $message;
  #endif
}


sub addLogs{
  my $instance = shift;
  my $instConfig = $instance->getInstTemplate();
  push @{$instConfig->GetListeners('log')},(sub {writeLog($_[0]);});
  push @{$instConfig->GetListeners('warning')},(sub {writeLog("WARNING: " . $_[0]);});
  push @{$instConfig->GetListeners('debug')},(sub {
                                                  if($sdbinstDebugLevel and $sdbinstDebugLevel ge $_[0]){
                                                    writeLog( $_[1]);
                                                  }});
}

sub createExternalInstallerInstanceFromPackobj{
  my $packobj = shift;
  my ($retVal,$instance);
  ($retVal,$instance) = SAPDB::Install::Templates::InstUnit::createObjectFromTGZ($packobj->Untgz());
  my $aPathStruct = $packobj->MainPath();
  my $aPath ; 
  $aPathStruct and 
    $aPath = $aPathStruct->{'value'} and $instance->setMainPath($aPath);
  $retVal and return ($retVal,undef);
  addLogs($instance);
  my $mainPath = $instance->getMainPath();
  $instance->debugit(1,"Installation path is '$mainPath'");
  return ($retVal,$instance);
}

sub createExternalInstallerInstanceFromReg{
  my $reg = shift;
  my ($retVal,$instance);
  ($retVal,$instance) = SAPDB::Install::Templates::InstUnit::createObjectFromRegistry($reg);
  $retVal and return ($retVal,undef);
  addLogs($instance);
  return ($retVal,$instance);
}





sub preuninstall{
  my ($this,$retVal) = (shift,0);
  $retVal = $this->SUPER::preuninstall(@_) and return $retVal;
  #Get parameters
  my ($mainPath , $version , $exePrefix , $responseFile , $logFile , $aExe , $name ) = $this->getMyParams();
  #determine already installed version
  my $installedVersion;
  ($retVal,$installedVersion) = $this->getInstalledVersion();
  $retVal and return $retVal;
  #Check if same version is installed:
  #if not: return
  my $aScript;  
  if (not defined $installedVersion){
    $this->logit("$name is not installed (Skipping)");
    return 0;
  }
  elsif($installedVersion ne $version){
    return "$name is installed in Version $installedVersion, cannot uninstall Version $version";
  }
  else{
    $aScript = $this->getUninstScript();
  }
  #find executable  uninstaller executable
#   my $classIDKey = $this->getClassIDKey();
#   my %values = readValues(&HKEY_CLASSES_ROOT,$classIDKey);
#   $this->logit("$classIDKey =  '" . join(",",keys %values)."'");
#   exists $values{'(Default)'} or return "Cannot get classID for $name to uninstall";
#   my $clsid = $values{'(Default)'};
#   $this->setClassID($clsid);
#   my $uninstKey = $this->getUninstKey();
#   %values = readValues(&HKEY_LOCAL_MACHINE,$uninstKey);
#   print("$uninstKey =  '" . join(",",keys %values)."'");
#   exists $values{'(Default)'} or return "Cannot get uninstaller for $name ";
#   my $uninstValue = $values{'(Default)'};
#   ($aExe) = ($uninstValue =~ /^(\S)\s/);
#   $this->logit("uninstaller executable is '$aExe'");

  #Execute the external uninstaller
  $retVal = $this->callExternalExe($responseFile,$aExe,$aScript,$logFile,$name) and return $retVal;
  $this->logit("Execution of external uninstaller for  $name was successfull");
  return $retVal;
}
sub staticpreuninstall{
  my $regpackobj = shift;
  my ($retVal,$obj)=(0);
  ($retVal,$obj) = createExternalInstallerInstanceFromReg($regpackobj);
  $retVal and return $retVal;
  $retVal = $obj->preuninstall() and return $retVal;
  return $retVal;
}

sub postinstall{
  my ($this,$retVal) = (shift,0);
  $retVal = $this->SUPER::postinstall(@_) and return $retVal;
  #Get parameters
  my ($mainPath , $version , $exePrefix , $responseFile , $logFile , $aExe , $name ) = $this->getMyParams();
  #determine already installed version
  my $installedVersion;
  ($retVal,$installedVersion) = $this->getInstalledVersion();
  $retVal and return $retVal;
  #Determine response file depending on installed version:
  my $aScript;  
  if (not defined $installedVersion){
    $aScript =  $this->getInstScript();
  }
  elsif($installedVersion lt $version){
    $aScript =  $this->getUpdateScript();
    $this->logit("$name already installed in version $installedVersion. Upgrading to $version.");
  }
  elsif($installedVersion eq $version){
    $this->logit("$name already installed in version $installedVersion, skipping.");
    return 0;
  }
  elsif($installedVersion gt $version){
    return "Cannot downgrade $name $installedVersion -> $version";
  }
  #Execute the external installer with the response file
  $retVal = $this->callExternalExe($responseFile,$aExe,$aScript,$logFile,$name) and return $retVal;
  $this->logit("Execution of external installer for  $name was successfull");
  return $retVal;
}

sub staticpostinstall{
  my $packobj = shift;
  my ($retVal,$obj)=(0);
  ($retVal,$obj) = createExternalInstallerInstanceFromPackobj($packobj);
  $retVal and return $retVal;
  $retVal = $obj->postinstall() and return $retVal;
  return $retVal;
}



sub callExternalExe{
  my $retVal =0;
  my ($this,$responseFile,$aExe,$aScript,$logFile,$name) = @_;
  open FH, ">$responseFile" or return "Cannot open install response file '$responseFile': $!";
  print FH ($aScript);
  close FH;
  my $command = "\"$aExe\"" . " -s -a -s  -f1\"" . $responseFile . "\" -f2\"" . $logFile . "\"";
  $this->logit("[".$command."]"); 
  $retVal = callSystem($command, sub{}) and return "Error installing $name: $retVal";
  open FH, "<$logFile" or return "Cannot read log file '$logFile': $!";
  my $resultCode;
  foreach(<FH>){
    if(/^\s*ResultCode\s*=\s*([-0-9]+)\s*$/){
      $resultCode = $1;
      last;
    }
  }
  close FH;
  defined $resultCode or return "Cannot get resultcode in '$logFile'";
  $this->logit("ResultCode from installer log: $resultCode");
  $resultCode eq '0' or return "Error executing installer of $name";
  unlink($aScript);
  unlink($logFile);
  return $retVal;
}


sub getInstalledVersion{
  my ($this) = @_;
  my ($retVal,$version)=(0,'');
  $version = $this->getInstalledVersionViaReg();
  return ($retVal,$version);
}

sub getMyParams{
  my $this = shift;
  my $mainPath = $this->getMainPath();
  my $exePrefix =  $this->getExePrefix();
  my $version =  $this->getVersion();

  return ( $mainPath,
           , $version
           , $exePrefix
           , catfile($mainPath,"misc","inst$exePrefix.iss")
           , catfile($mainPath,"misc","inst$exePrefix.log")
           , catfile($mainPath,"misc",join( "" , ($exePrefix, ($version =~ /^(\d+)\.(\d+)\./), ".exe")))
           ,$this->getName());
}



package   SAPDB::Install::Templates::PackageServerUtil;
use SAPDB::Install::System;
use SAPDB::Install::Templates::Util;
our @ISA = qw (SAPDB::Install::Templates::InstUnit);

sub myKeys{ return { mainPath =>{ 'DEFAULT' => '${indepProg}' } }; }

my $progpath;
$progpath{'name'} = 'independent program path';
$progpath{'list'} = 'utils.lst';
$progpath{'opt'} = 'indep_prog=s';
$progpath{'mode'}=0775;
$progpath{'update'}=0;
sub GetPaths{ return (\%progpath); }
sub initData{
  my ($this,$rhData)=@_;
  $this->SUPER::initData($rhData);
  $progpath{'default'}=catfile(getSysProgPath(),'sdb/programs');
}

sub preinstall{
  my $this = shift;
  my $result = $this->SUPER::preinstall();
  if($result ){
    return $result;
  }
  $progpath{'value'}=$this->getIndepProg;
  return $result;
}


sub register{
  my $this = shift;
  $this->SUPER::register();
  unless($^O =~ /mswin/i){ return 0; }
  my $retVal=$this->call_x_server("install",['19712','19709']);
  return $retVal;
}

sub unregister{
  my $this = shift;
  $this->SUPER::unregister();
  my $retVal=$this->call_x_server("stop",['18640']);
  unless($^O =~ /mswin/i){ return $retVal; }
  $retVal=$this->call_x_server("remove",['19710','18581']);
  return $retVal;
}

package SAPDB::Install::Templates::PackageDBKernel;
use SAPDB::Install::System;
use SAPDB::Install::Templates::Util;
use SAPDB::Install::Instance;
use SAPDB::Install::MD5Sum;
unless($^O =~ /mswin/i){
  use SAPDB::Install::System::Unix;
  use SAPDB::Install::System::Unix::Dir;
}
our @ISA = qw (SAPDB::Install::Templates::InstUnit);
sub myKeys{
  return {
          releaseId =>{
                       'DEFAULT' => sub{
                         my ($this) = @_;
                         my $aRelease  = $this->getVersion();
                         if($aRelease){
                           $aRelease =~ s/\.//g;
                           $aRelease =~ s/\d\d$//;
                         }
                         return $aRelease;
                       }
                      },
          mainPath =>{
                      'DEFAULT' => '${sysProgPath}/sdb/${releaseId}'
                     },
          kernelPackage => {},
          oldKernelPackage => {}
         };
}
my %deppath;
$deppath{'name'} = 'dependent path';
$deppath{'list'} = 'kernel.lst';
$deppath{'opt'} = 'depend=s';
$deppath{'mode'}=0775;
$deppath{'update'}=0;
sub GetPaths{ return (\%deppath); }
sub initData{
  my ($this,$rhData)=@_;
  $this->SUPER::initData($rhData);
  my $depPath = $this->getMainPath();
  $progpath{'default'}=$depPath;
  $progpath{'value'}=$depPath;
}

sub SetInstTemplate{
  my ($this,$value) = @_;
  $this->SUPER::SetInstTemplate($value);
  $this->setMainPath($value->getDependendPath());
}


sub instanceErrorChecker{
  my ($this, $instance,$successStates) = @_;
  my $errorState = $instance->get_errorstate;
  my $msgtxt = $instance->{msgtxt};
  my $retVal = $errorState . ":\n" . $msgtxt;
  foreach (@$successStates) {
    $errorState eq $_ and return 0
  }
  return $retVal;
}


sub postprepare{
  my ($this,$oldPackage) = @_;
  my $retVal=0;
  $retVal = $this->SUPER::postprepare($oldPackage) and return $retVal;
  my $version = $this->getVersion();
  my $instConfig = $this->getInstTemplate();
  my $indep_data = $instConfig->getIndepData();
  my $mainPath = $this->getMainPath();
  my $user = $this->getUser();
  my $grp = $this->getGroup();
  my $uid = $this->GetUID();
  my $gid = $this->GetGRID();
  unless($^O =~ /mswin/i or $this->crelease2num($version) < $this->crelease2num('7.5.00.00')){
    my @nodes = (
                 {
                  'path' => catfile($indep_data,"config"),
                  'access' => 'rwx'
                 },
                 {
                  'path' => catfile($indep_data,"config","Databases.ini"),
                  'access' => 'rw'
                 },
                 {
                  'path' => catfile($indep_data,"config","Installations.ini"),
                  'access' => 'r'
                 }

		);
		
    foreach my $ref_node (@nodes) {
      my %node = %$ref_node; 
      my $aPath = $node{'path'};
      my $aAcc= $node{'access'};
      -l $aPath and $aPath = readlink($aPath);
      -e $aPath or next;
      $retVal=$this->testaccess ($aPath,$user,$aAcc) and return $retVal; 
    }
  }

  unless ($^O =~ /mswin/i) {
    chown ($uid, $gid, $mainPath) == 1 or
      return ("Cannot chown '$mainPath' to user $user and group $grp"); 
    chmod (0555, $mainPath) == 1 or return  ("cannot chmod '$mainPath' to 0555\n") ;
    # check access of parent directories
    unless (uaccess ($user, $mainPath) == 0) {
      my @ancestors =	SAPDB::Install::System::Unix::Dir::getancestors ($mainPath);
      foreach my $dir (@ancestors) {
        next if (-l $dir);
        my ($dev, $ino, $mode) = stat ($dir);
        next if (($mode & 0111) == 0111);
        chmod (($mode & 07777) | 00111, $dir);
      }
    }
    uaccess ($user, $mainPath) == 0 or return ("user $user cannot access directory $mainPath\n");
  }
  return $retVal;
}# kernel postprepare

#get internal parameters for the upgrade;
sub GetUpgradeParams{
  my ($this,$oldPackage) = @_;
  my $instConfig = $this->getInstTemplate();
  my $packageName = $this->getName();
  my $dbname = $instConfig->getDBName();
  my $path = $this->getMainPath();
  my $dbmuser = $instConfig->getDBMUser();
  my $version = $this->getVersion();
  my ( $apoTargetRelease,$kernelPackage,$oldKernelPackage,$targetRelease) = $this->GetInternalParams($oldPackage);
  my $oldVersion = $oldPackage ? $oldPackage->getVersion() : undef;
  my $upgradeFileName = $instConfig->getUpgradeFile();
  my $upgradeFile = SAPDB::Install::Templates::UpgradeFile::new();
  return ( $instConfig , $packageName , $kernelPackage,$oldKernelPackage, $dbname ,
           $path , $dbmuser , $version, $apoTargetRelease , $targetRelease , $oldVersion ,
           $upgradeFileName , $upgradeFile); 
}


#get internal parameters for the upgrade;
#kernel specific
sub GetInternalParams{
  my ($this,$oldPackage) = @_;
  my $apoTargetRelease = undef;
  my ($kernelPackage,$oldKernelPackage)  = ($this,$oldPackage); 
  my $targetRelease =  $this->getVersion;
  return ($apoTargetRelease,$kernelPackage,$oldKernelPackage,$targetRelease);
}


sub isUpgrade{
  my ($this,$oldPackage)  = @_;
  unless($this->instanceExists ) {
    $this->warnit("Database instance not found\n");
    return (0,0);
  }
  my ($isResponsible,$retVal) = $this->isResponsible();
  $retVal and return (undef,"Could not check instance type:\n".$retVal);
  not $isResponsible and return (0,0);
  unless($oldPackage){
    $this->logit("No instance check necessary (in 'framestart')\n");
    return (0,0);
  }
  return (1,0);
}




# kernel framestart:
sub framestart{
  my ($this,$oldPackage) = @_;
  $this->SUPER::framestart($oldPackage);
  my ($retVal,$isUpgrade)=(0,0);
  my ( $instConfig , $packageName , $kernelPackage,$oldKernelPackage, $dbname ,
       $path , $dbmuser , $version, $apoTargetRelease , $targetRelease , $oldVersion ,
       $upgradeFileName , $upgradeFile)  = $this->GetUpgradeParams($oldPackage);
  -f $upgradeFileName and $retVal = $upgradeFile->parseFile($upgradeFileName) and return $retVal;
  if($upgradeFile->getInstanceCheckOK){
    $this->logit("Instance check was successfull last time\n");
    return 0;
  }
  $retVal=$this->call_x_server("start",['18641','19720']) and return $retVal;
  ($isUpgrade,$retVal) = $this->isUpgrade($oldPackage);
  $retVal and return $retVal;
  not $isUpgrade and return 0;
  $this->debugit(1,"Checking for multiple instances");
  $this->findInstances > 1 and 
    return "Multiple instance update not supported -  multiple instances on \"".$this->getMainPath . "\"\n";

  $this->debugit(1,"Creating upgrade object");
  my  $instance =$this->createUpgradeObject();
  my %sessiondata =  ( 'apo_target_release' => $apoTargetRelease,
                       'dbname' => $dbname,
                       'userpwd' => $dbmuser ,
                       'target_release' => $targetRelease,
                       'additional' => undef,
                       'logpos' => $upgradeFile->getLogPos,
                       'backup_logpos' => $upgradeFile->getBackupLogPos);
  $this->debugit(1,"Creating session object from upgrade object");
  my $dbm =$instance->session (%sessiondata);
  $retVal = $this->instanceErrorChecker($instance,['OK']) and return $retVal;
  $this->debugit(1,"Starting instance preparation now");
  $instance->prepare();
  #save instance date here ....
  $upgradeFile->setApoStartingRelease($instance->{'apo_starting_release'});
  $upgradeFile->setStartingRelease($oldKernelPackage->getBuildString());
  $upgradeFile->setInstanceCheckOK($instance->get_errorstate () eq 'OK' ? 1 : 0);
  $upgradeFile->setLogPos($instance->{'logpos'});
  $upgradeFile->setBackupLogPos($instance->{'backup_logpos'});
  $upgradeFile->setMigrationStrategy($instance->{'migration_strategy'});
  $upgradeFile->setLCOwner($instance->{'lcowner'});
  $upgradeFile->setDefaultCode($instance->{'default_code'});
  $upgradeFile->setMCOD($instance->{'mcod'});

  
  $retVal = $upgradeFile->saveToFile($upgradeFileName) and return "Could not save upgrade status: '" . $retVal . "'";
  $retVal = $this->instanceErrorChecker($instance,['OK']) and return $retVal;
  #TODO
  
  $instance->shutdown();
  $retVal = $this->instanceErrorChecker($instance,['OK']) and return $retVal;
  $instance->disconnect();
  $instance = undef;
  $retVal=$this->call_x_server("stop",['18640']) and return $retVal;
  return $retVal;
}

#kernel framestop
sub framestop{
  my ($this,$oldPackage) = @_;
  $this->SUPER::framestop($oldPackage);
  my ($retVal,$isUpgrade)=(0,0);
  $retVal=$this->call_x_server("start",['18641','19720']) and return $retVal;
  ($isUpgrade,$retVal) = $this->isUpgrade($oldPackage);
  $retVal and return $retVal;
  not $isUpgrade and return 0;
  my ( $instConfig , $packageName , $kernelPackage,$oldKernelPackage, $dbname ,
       $path , $dbmuser , $version, $apoTargetRelease , $targetRelease , $oldVersion ,
       $upgradeFileName , $upgradeFile)  = $this->GetUpgradeParams($oldPackage);
  not -f $upgradeFileName and return "Upgrade data file '$upgradeFileName' not found.";
  $retVal = $upgradeFile->parseFile($upgradeFileName) and return "Could not read data in upgrade file '$upgradeFile':\n$retVal";
  #get instance data from registry
  my  $instance = $this->createUpgradeObject();
  $instance->chvolperm($dbname, $upgradeFile->getStartingRelease(), $targetRelease);
  $retVal = $this->instanceErrorChecker($instance,['OK']) and return $retVal;
  my %sessionData =  	 (
                          'apo_target_release' => $apoTargetRelease,
                          'apo_starting_release' => $upgradeFile->getApoStartingRelease,
                          'default_code' => $upgradeFile->getDefaultCode,
                          'lcowner' => $upgradeFile->getLCOwner,
                          'dbname' => $dbname, 
                          'userpwd' => $dbmuser,
                          'target_release' => $targetRelease,
#                          'instancetype' => 'LCA',
                          'starting_release' => $upgradeFile->getStartingRelease,
                          'mcod' => $upgradeFile->getMCOD,
                          'migration_strategy' => $upgradeFile->getMigrationStrategy);
  my $dbm =$instance->session (%sessionData);
  $retVal = $this->instanceErrorChecker($instance,['OK']) and return $retVal;
  $instance->finalize();
  $retVal = $this->instanceErrorChecker($instance,['OK']) and return $retVal;
  $instance->disconnect();
  if ($instance->get_keep_instance_data) {
  } else {
    unlink $upgradeFileName or return "Could not delete upgrade file '$upgradeFileName': $!";  
  }
  return $retVal;
}


sub isResponsible{
  my ($this) = @_;
  my ($isLiveCache,$retVal) = $this->isLiveCache;
  $retVal and return (undef,$retVal);
  if($isLiveCache){
    $this->logit("Kernel package ignores liveCache upgrade\n");
    return (0,$retVal);
  }
  return (1,$retVal);
}




sub  createUpgradeObject{
  my ($this) = @_;
  my $instConfig = $this->getInstTemplate();
  my  $instance = SAPDB::Install::Instance->new (); 
  my $lt = $instConfig->GetListeners('log');
  $instance->{msg0} = sub { my $msg = join "\n",@_ ; foreach(@$lt){&$_($msg);}};
  $instance->{msg1} = sub { my $msg = join "\n",@_ ; foreach(@$lt){&$_($msg);}};

  my $statusListeners = $instance->{status_listeners};
  $statusListeners or $statusListeners = [];
  my @listeners = @$statusListeners;
  push @listeners, (sub { my $msg = join "\n",@_ ; foreach(@$lt){&$_($msg);}});
  $instance->{status_listeners} = $statusListeners;
  return $instance;
}



sub preuninstall{
  my $this = shift;
  $this->SUPER::preuninstall();
  my %instances = SAPDB::Install::Registry::getInstances();  	
  foreach my $instance_name (keys(%instances)){
    my $dbroot = normalizeAPath($instances{$instance_name});
    my $mainPath = normalizeAPath($this->getMainPath());
    if($dbroot eq $mainPath){
      return "Found instance $instance_name in '$dbroot'\n";
    }
  }
  return 0;
}
sub register{
  my $this = shift;
  $this->SUPER::register();
  my ($retVal,$cmd,$release)=(0,undef,$this->GetRelease());
  $release gt '7.3' and $retVal = $this->do_dbmsrvctl() and return $retVal;
  $retVal = $this->do_instreg() and return $retVal;
  $release le '7.3' and $retVal = $this->do_xregcomp('-c') and return $retVal;
  return $retVal;
}


sub do_xregcomp{
  my ($this,$opt) = @_;
  my ($retVal,$cmd) = (0,undef);
  my $mainPath = $this->getMainPath();
  my $success=0;
  my $checker = sub { my $text=shift;
                      $this->debugit(1,$text);
                      $text =~ /Registration\s+done/ and $success=1;		
                      $text =~ /Registration\s+cleared/ and $success=1; };
  my $cmd1 = "$mainPath/bin/xregcomp $opt \\\"$mainPath/pgm/dbpinstall\\\"";
  my $cmd2 = "$mainPath/bin/xregcomp $opt \\\"$mainPath/sap/sphsapdb\\\"";
  if( $^O =~ /mswin/i){
    $cmd1 =~ s/\//\\/g;
    $cmd2 =~ s/\//\\/g;
  } 
  $this->logit("[$cmd1]");
  $retVal = callSystem($cmd1,$checker) and return "Error in xregcomp: $retVal";
  $success or return "Error in xregcomp\n";
  $success=0;
  $this->logit("[$cmd2]");
  $retVal = callSystem($cmd2,$checker) and return "Error in xregcomp: $retVal";
  $success or return "Error in xregcomp\n";
  return $retVal;
}


sub do_instreg{
  my ($this) = @_;
  my ($retVal,$cmd) = (0,undef);
  my $mainPath = $this->getMainPath();
  if( $^O =~ /mswin/i){
    $mainPath=~s/\//\\/g;
    $cmd= " -R \"" . $mainPath . "\" inst_reg -k \\\"" . $mainPath . "\\\"";
  }
  else{
    $cmd= " -R \"" . $mainPath . "\" inst_reg -k \"" . $mainPath . "\"";
  }
  $retVal = $this->call_dbmcli($cmd) and return $retVal;
  return $retVal;
}
sub do_dbmsrvctl{
  my $this = shift;
  my $retVal = 0;
  my $dbmsrvctl = catfile( $this->getMainPath(),"bin","dbmsrvctl");
  my $cmd = $dbmsrvctl . " start";
  $retVal = callSystem($cmd, sub{ }) and return "Error in '$cmd'\nReturn code $retVal\n";
  return $retVal;
}


sub unregister{
  my $this = shift;
  $this->SUPER::unregister();
  my ($release,$retVal) = ($this->getRelease(),0);
  $release le '7.3' and $retVal = $this->do_xregcomp('-u') and return $retVal;
  my $mainPath = $this->getMainPath();
  my $cmd= " -R \"" . $mainPath . "\" inst_unreg";
  $retVal = $this->call_dbmcli($cmd) and return $retVal;
  return $retVal;
}

sub postinstall{
  my $this = shift;
  my $retVal=0;
  $retVal = $this->SUPER::postinstall(@_) and return $retVal;
  my $instConfig = $this->getInstTemplate();
  my $indepData = $instConfig->getIndepData();
  my $mainPath = $this->getMainPath();
  my $grid = $this->GetGRID();
  my $uid = $this->GetUID();
  my $version = $this->getVersion();
  #SKIPPED: remove instance data from earlier installations

  my @demofiles;                # files which have to be modified
  if ($^O=~/mswin/i) {
    @demofiles = ('misc/create_demo_db.cmd','misc/drop_demo_db.cmd');
  } else {
    @demofiles = ('misc/create_demo_db.sh','misc/drop_demo_db.sh'
                  ,'doc/FirstSteps/index.html', 'doc/FirstSteps/sapdbenv.sh','doc/FirstSteps/Perl/HowToPerl.html',
                  'doc/FirstSteps/Python/HowToPython.html','doc/FirstSteps/ODBC/HowToODBC.html',
                  'doc/FirstSteps/Java/HowToJava.html'
                 );
  }
  my $regmd5sums = $this->getFileList();
  foreach my $file (@demofiles) {
    my $longfile=catfile($mainPath,$file);
    if (-f $longfile) {
      my @statbuf = stat($longfile);
      open(FD,$longfile) or $this->warnit("Cannot open demo database script file \"$longfile\"\n") and next;
      my @newfile;              # array of newlines for file
      while (my $line = <FD>) {
        chomp($line);
        my $path=$mainPath;
        $^O=~/mswin/i and $path=~s/\//\\/g;
        $line=~s/\$DEP/$path/g;
        $path=$mainPath;
        $^O=~/mswin/i and $path=~s/\//\\/g;
        $line=~s/\$IND/$path/g;
        push(@newfile,$line);
      }
      close(FD);
      open(FD,">$longfile") or $this->warnit("Cannot create new demo database script file \"$longfile\"\n") and next;
      foreach my $line (@newfile) {
        print FD "$line\n";
      }
      close(FD);
      utime($statbuf[9],$statbuf[9],$longfile);
      foreach (keys %$regmd5sums) {
        if ($_ eq $file) {
          $regmd5sums->{$_} = MD5Sum($longfile);
        }
      }
    } else {
      $this->warnit("Cannot modify \"$longfile\"\n");
    }
  }
  #SKIPPED: Deletion of unsupported files.

  unless($^O=~/mswin/i){
    my $file =catfile($mainPath,'pgm/dbmverifypw');
    chown(0,$grid,$file) or $this->warnit("Cannot change owner to \"root\" for file \"$file\" - please doit manually\n");
    chmod(04550,$file) or $this->warnit("Cannot chmod file \"$file\"  to 04550- please doit manually\n");
    $file = catfile($mainPath,'pgm/starter');
    chmod (0500, $file) or $this->warnit("Cannot chmod \"$file\ to 0500 - please doit manually\n");;
    $file = catfile($mainPath,'pgm/starter/sdbstarter');
    chown (0, $grid, $file)or $this->warnit("Cannot chown \"$file\ to root," . $this->getGroup() . " - please doit manually\n");
    chmod (04510, $file) or $this->warnit("Cannot chmod \"$file\ to 04510 - please doit manually\n");
  }

  if ($^O =~ /mswin/i) {
    #old loader code:
    my $p1 = catfile($mainPath,'/env/LOADER.py');
    my $p2 = catfile($mainPath,'/env/LOADER.pyc');
    foreach ($p1,$p2) {
      -f $_ and (unlink $_ or $this->warnit("Cannot delete $_, please do it manually"));
    }
  }

  foreach my $file ('lcinit.log','lcinit.his','adatmp','adatmp2') {
    my $fullpath = catfile($mainPath,$file); 
    -f $fullpath and unlink($fullpath);
  }
  #SKIPPED: Migration registrations
  # 	migrateDBRegistrations($deppath{'value'}) unless($^O =~ /mswin/i);
  return $retVal;
}                               #postinstall





sub dirtyInstanceCheck{
  my $this = shift;
  my $retVal=0;
  my $instConfig = $this->getInstTemplate();
  my $dbname = $instConfig->getDBName();
  if($this->instanceExists ) {
    $this->warnit("Database $dbname already exists\n");
    return 0;
  }
  $retVal=$this->call_x_server("start",['18641','19720']) and return $retVal;
  return $retVal;
}



sub isLiveCache{
  my ($this) = @_;
  my $instConfig = $this->getInstTemplate();
  my $dbname = $instConfig->getDBName();
  my $dbmuser = $instConfig->getDBMUser();
  my ($retVal,$result,$instanceType,$isLiveCache)=(0,[],undef,0);

  $retVal = $this->call_dbmcli(" -u $dbmuser -d $dbname param_directget INSTANCE_TYPE",$result)
    and return (undef,$retVal);
  foreach(@$result){
    /^\s*INSTANCE_TYPE\s+(\w+)\s*$/ and $instanceType = $1;
  }
  $instanceType or return (undef,"Could not determine instance type:\n" . join("\n",@$result));
  if ('LVC' eq $instanceType) {
    $isLiveCache =1;
  }
  return ($isLiveCache,$retVal);
}


# creates an oltp instance
sub createInstance{
  my $this = shift;
  my $retVal=0;
  $retVal= $this->dirtyInstanceCheck() and return  $retVal;
  my $release = $this->GetRelease();
  $release le '7.3' and return $this->createInstance73();
  my $instConfig = $this->getInstTemplate();
  my $depPath = $instConfig->getDependendPath();
  $depPath =~ s/\//\\/g if $^O =~ /mswin/i;
  my $dbname = $instConfig->getDBName();
  my $dbmuser = $instConfig->getDBMUser();
  my $dbauser = $instConfig->getDBAUser();
  $retVal =  $this->call_dbmcli(" -R \"$depPath\" db_create  $dbname $dbmuser") and return $retVal; 
  my $dbm = SAPDB::Install::DBMCmd->new ('', $dbname) or return "DBMCmd not initialized!";
  $retVal = $this->dbmcmd($dbm,"user_logon $dbmuser") and return $retVal;
  $retVal = $this->dbmcmd($dbm,"param_startsession") and return $retVal;
  $retVal = $this->dbmcmd($dbm,"param_init") and return $retVal;
  $retVal = $this->dbmcmd($dbm,"param_checkall") and return $retVal;
  $retVal = $this->dbmcmd($dbm,"param_commitsession") and return $retVal;
  my $params = $instConfig->getDBParams();
  foreach(keys %$params){
    $retVal = $this->dbmcmd($dbm,"param_directput " . $_ . " " . $params->{$_}) and return $retVal;
  }
  $retVal = $this->dbmcmd($dbm,"param_checkall") and return $retVal;
  $retVal = $this->addVolumes($dbm) and return $retVal;
  $retVal = $this->dbmcmd($dbm,"db_admin") and return $retVal;
  $retVal = $this->dbmcmd($dbm,"db_activate $dbauser") and return $retVal;
  $retVal = $this->dbmcmd($dbm,"load_systab -u $dbauser -ud domain") and return $retVal;
#  $retVal = $this->dbmcmd($dbm,"load_systab -u $dbauser") and return $retVal;
  return $retVal;
}

sub createInstance73{
  my ($this) = @_;
  my ($retVal) = (0);

  my $instConfig = $this->getInstTemplate();
  my $depPath = $instConfig->getDependendPath();
  $depPath =~ s/\//\\/g if $^O =~ /mswin/i;
  my $dbname = $instConfig->getDBName();
  my $dbmuser = $instConfig->getDBMUser();
  my $dbauser = $instConfig->getDBAUser();

  #do db_create
  $retVal =  $this->call_dbmcli(" -R \"$depPath\" db_create  $dbname $dbmuser") and return $retVal; 
  #Generate script:
  my $script = << "SCRIPTEND";
user_logon $dbmuser
param_startsession
param_init
param_checkall
param_commitsession
SCRIPTEND

  #Add database parameters to script:
  my $params = $instConfig->getDBParams();
  foreach (keys %$params) {
    $script .= "param_directput " . $_ . " " . $params->{$_} . "\n";
  }
  #Add a param_checkall if necessary
  keys %$params and $script .= "param_checkall\n";
  
  #Add the volumes:
  my $vols = $instConfig->getDBVolumes();
  foreach(@$vols){
    my ($num,$type,$path,$kind,$size)= split/;/; 
    $path =~ s/\\/\//g unless $^O =~ /mswin/i;
    $path =~ s/\//\\/g if $^O =~ /mswin/i;
    $retVal = $this->pmakedir(dirname($path),0777) and return $retVal;
    $script .= "param_adddevspace $num $type $path $kind $size\n";
  }

  $script .= << "SCRIPTEND2";
db_cold
util_connect $dbmuser
util_execute init config
util_activate  $dbauser 
util_release
load_systab -u $dbauser -ud domain
db_warm
SCRIPTEND2

  #Execute the script:
   $retVal =  $this->call_dbmcli_script($script,$dbmuser,$dbname) and return $retVal; 
  return $retVal;
}




sub addVolumes{
  my($this,$dbm) = @_;
  my $retVal = 0;
  my $instConfig = $this->getInstTemplate();
  my $vols = $instConfig->getDBVolumes();
  foreach(@$vols){
    my ($num,$type,$path,$kind,$size)= split/;/; 
    $path =~ s/\\/\//g unless $^O =~ /mswin/i;
    $path =~ s/\//\\/g if $^O =~ /mswin/i;
    $retVal = $this->pmakedir(dirname($path),0777) and return $retVal;
    $retVal = $this->dbmcmd($dbm,"param_addvolume $num $type \"$path\" $kind $size") and return $retVal;
  }
  return $retVal;
}


sub deleteInstance{
  my $this = shift;
  my $retVal=0;
  $this->GetRelease() le '7.3' and return $this->deleteInstance73();
  my $instConfig = $this->getInstTemplate();
  my $dbname = $instConfig->getDBName();
  unless($this->instanceExists ) {
    $this->warnit("Database $dbname not found\n");
    return 0;
  }
  my $dbmuser = $instConfig->getDBMUser();
  my $dbm = SAPDB::Install::DBMCmd->new ('', $dbname) or return "DBMCmd not initialized!";
  $retVal = $this->dbmcmd($dbm,"user_logon $dbmuser") and return $retVal;
  $retVal = $this->dbmcmd($dbm,"db_offline") and return $retVal;
  $retVal = $this->dbmcmd($dbm,"db_drop") and return $retVal;
  return $retVal;
}

sub deleteInstance73{
  my $this = shift;
  my $retVal=0;
  my $instConfig = $this->getInstTemplate();
  my $dbname = $instConfig->getDBName();
  unless($this->instanceExists ) {
    $this->warnit("Database $dbname not found\n");
    return 0;
  }
  my $dbmuser = $instConfig->getDBMUser();
  $this->call_dbmcli_script("db_offline\ndb_drop\n",$dbmuser,$dbname);
  return $retVal;
}

sub instanceExists{
  my $this = shift;
  my $instConfig = $this->getInstTemplate();
  my $dbname = $instConfig->getDBName();
  my @instances = $this->findInstances();
  my $f=0;
  foreach(@instances){
    if((uc $_) eq (uc $dbname)){
      $f=1;
      last;
    }
  }
  return $f;
}

sub findInstances{
  my $this = shift;
  my $instConfig = $this->getInstTemplate();
  my $mainPath = normalizeAPath($instConfig->getDependendPath());
  my %instances = SAPDB::Install::Registry::getInstances();  	
  my @retVal;
  foreach my $instance_name (keys(%instances)){
    my $dbroot = normalizeAPath($instances{$instance_name});
    if($dbroot eq $mainPath ){
      push(@retVal,($instance_name));
    }
  }
  return @retVal;
}





sub dbmcmd{
  my ($this,$dbm,$cmd) = @_;
  my $instConfig = $this->getInstTemplate();
  $this->logit("[".$cmd."]");
  my $output = $dbm->exec($cmd);
  if($output){
    my $result =  (split('\n', $output))[0];
    $result =~ /^OK/ and return 0;
  }
  unless($output){
    my $lastmsg = $dbm->lastmsg();
    my $lasterr = $dbm->lasterr();
    my $errorReturn = "$lastmsg\n$lasterr\n";
    if($errorReturn =~ /^\s*$/){
      $errorReturn = "NO ERROR MESSAGE AVAILABLE";
    }
    return $errorReturn;
  }
  return $output;
}



package   SAPDB::Install::Templates::PackageSQLS;
our @ISA = qw (SAPDB::Install::Templates::PackageDBM);

sub myKeys{
my $instScript = <<"INSTFILE";
[InstallShield Silent]
Version=v6.00.000
File=Response File
[File Transfer]
OverwrittenReadOnly=NoToAll
[\{6A729C9E-BC1A-4F20-9EA8-95E1570545F1\}-DlgOrder]
Dlg0=\{6A729C9E-BC1A-4F20-9EA8-95E1570545F1\}-SdWelcome-0
Count=5
Dlg1=\{6A729C9E-BC1A-4F20-9EA8-95E1570545F1\}-SdAskDestPath-0
Dlg2=\{6A729C9E-BC1A-4F20-9EA8-95E1570545F1\}-SdSelectFolder-0
Dlg3=\{6A729C9E-BC1A-4F20-9EA8-95E1570545F1\}-SdStartCopy-0
Dlg4=\{6A729C9E-BC1A-4F20-9EA8-95E1570545F1\}-SdFinish-0
[\{6A729C9E-BC1A-4F20-9EA8-95E1570545F1\}-SdWelcome-0]
Result=1
[\{6A729C9E-BC1A-4F20-9EA8-95E1570545F1\}-SdAskDestPath-0]
szDir=\${winInstPath}
Result=1
[\{6A729C9E-BC1A-4F20-9EA8-95E1570545F1\}-SdSelectFolder-0]
szFolder=MySQL MaxDB
Result=1
[\{6A729C9E-BC1A-4F20-9EA8-95E1570545F1\}-SdStartCopy-0]
Result=1
[Application]
Name=SQL Studio
Version=\${version}
Company=SAP AG
Lang=0009
[\{6A729C9E-BC1A-4F20-9EA8-95E1570545F1\}-SdFinish-0]
Result=1
bOpt1=0
bOpt2=0
[\{BDF33DF1-74DD-42F7-B1E4-5FB009F322CC\}-DlgOrder]
Count=0
[\{40314691-03B7-4446-BD07-455BB5065FB8\}-DlgOrder]
Count=0
[\{1266F848-EDF7-482A-AC08-8572932505A1\}-DlgOrder]
Count=0
[\{3002A6A5-63C2-41ED-85DE-3438A737AF4C\}-DlgOrder]
Count=0
[\{F2AEC4C5-45F1-46D6-9E04-28C59D5CAA30\}-DlgOrder]
Count=0
[\{B75C64BC-0006-4437-9F51-641B18904F8B\}-DlgOrder]
Count=0
[\{05F72A77-4A2A-4B9E-B5E7-E53AE156FDF3\}-DlgOrder]
Count=0
INSTFILE
  my $uninstScript = <<"UNINSTFILE";
[InstallShield Silent]
Version=v6.00.000
File=Response File
[File Transfer]
OverwrittenReadOnly=NoToAll
[\{6A729C9E-BC1A-4F20-9EA8-95E1570545F1\}-DlgOrder]
Dlg0=\{6A729C9E-BC1A-4F20-9EA8-95E1570545F1\}-SdWelcomeMaint-0
Count=3
Dlg1=\{6A729C9E-BC1A-4F20-9EA8-95E1570545F1\}-SprintfBox-0
Dlg2=\{6A729C9E-BC1A-4F20-9EA8-95E1570545F1\}-SdFinish-0
[\{6A729C9E-BC1A-4F20-9EA8-95E1570545F1\}-SdWelcomeMaint-0]
Result=303
[\{6A729C9E-BC1A-4F20-9EA8-95E1570545F1\}-SprintfBox-0]
Result=1
[Application]
Name=SQL Studio
Version=\${version}
Company=SAP AG
Lang=0009
[\{6A729C9E-BC1A-4F20-9EA8-95E1570545F1\}-SdFinish-0]
Result=1
bOpt1=0
bOpt2=0
[\{BDF33DF1-74DD-42F7-B1E4-5FB009F322CC\}-DlgOrder]
Count=0
[\{40314691-03B7-4446-BD07-455BB5065FB8\}-DlgOrder]
Count=0
[\{1266F848-EDF7-482A-AC08-8572932505A1\}-DlgOrder]
Count=0
[\{3002A6A5-63C2-41ED-85DE-3438A737AF4C\}-DlgOrder]
Count=0
[\{F2AEC4C5-45F1-46D6-9E04-28C59D5CAA30\}-DlgOrder]
Count=0
[\{B75C64BC-0006-4437-9F51-641B18904F8B\}-DlgOrder]
Count=0
[\{05F72A77-4A2A-4B9E-B5E7-E53AE156FDF3\}-DlgOrder]
Count=0
UNINSTFILE

  my $updateScript = <<"UPDATEFILE";
[InstallShield Silent]
Version=v6.00.000
File=Response File
[File Transfer]
OverwrittenReadOnly=NoToAll
[{6A729C9E-BC1A-4F20-9EA8-95E1570545F1}-DlgOrder]
Dlg0={6A729C9E-BC1A-4F20-9EA8-95E1570545F1}-SdWelcome-0
Count=3
Dlg1={6A729C9E-BC1A-4F20-9EA8-95E1570545F1}-SdStartCopy-0
Dlg2={6A729C9E-BC1A-4F20-9EA8-95E1570545F1}-SdFinish-0
[{6A729C9E-BC1A-4F20-9EA8-95E1570545F1}-SdWelcome-0]
Result=1
[{6A729C9E-BC1A-4F20-9EA8-95E1570545F1}-SdStartCopy-0]
Result=1
[Application]
Name=SQL Studio
Version=\${version}
Company=SAP AG
Lang=0009
[{6A729C9E-BC1A-4F20-9EA8-95E1570545F1}-SdFinish-0]
Result=1
bOpt1=0
bOpt2=0
[{CDEE62EA-7780-438C-BEE2-BEFF1C4E0D90}-DlgOrder]
Count=0
[{C5813181-443D-4786-8E7A-8942D357B7F0}-DlgOrder]
Count=0
[{74D357CA-ACF2-4BBD-AD30-9F873A8C4E6B}-DlgOrder]
Count=0
[{28488D74-4BBF-463F-9F83-10E38E5EFB10}-DlgOrder]
Count=0
[{E5DDC44F-C6A7-421B-AEC1-0AD3C883EB64}-DlgOrder]
Count=0
[{D6BA7E1B-4B0B-48C8-ACB4-4E5CC3754376}-DlgOrder]
Count=0
[{19CD7770-1B50-49BA-9025-72586042A74D}-DlgOrder]
Count=0
UPDATEFILE
  return {
          winInstPath =>{ 'DEFAULT' => '${indepProg}\SQLStudio' },
          winExePath =>{ 'DEFAULT' => '${indepProg}\misc' },
          mainPath =>{ 'DEFAULT' => '${indepProg}' },
          instScript =>{ 'DEFAULT' => $instScript },
          uninstScript =>{ 'DEFAULT' => $uninstScript },
          updateScript =>{ 'DEFAULT' => $updateScript },
          exePrefix =>{ 'DEFAULT' => 'SQLSTD' },
          versionKey => { 'DEFAULT' => 'SOFTWARE\SAP AG\SQL Studio\7.6'},
          winID => { 'DEFAULT' => 'SQLStudio.SQLSTO'},
          classID => {},# must be read from registry
          classIDKey => { 'DEFAULT' => '${winID}\Clsid'},
          uninstKey => { 'DEFAULT' => 'SOFTWARE\Microsoft\Windows\CurrentVersion\Uninstall\${classID}'}
         };
}







package SAPDB::Install::Templates::PackageCPCBase;
our @ISA = qw (SAPDB::Install::Templates::InstUnit);
sub myKeys{ return { mainPath =>{ 'DEFAULT' => '${indepProg}' } }; }
package SAPDB::Install::Templates::PackageDBAnalyzer;
our @ISA = qw (SAPDB::Install::Templates::InstUnit);
sub myKeys{ return { mainPath =>{ 'DEFAULT' => '${indepProg}' } }; }
package SAPDB::Install::Templates::PackageDBConnect;
our @ISA = qw (SAPDB::Install::Templates::InstUnit);
sub myKeys{ return { mainPath =>{ 'DEFAULT' => '${indepProg}' } }; }
package SAPDB::Install::Templates::PackageJDBC;
our @ISA = qw (SAPDB::Install::Templates::InstUnit);
sub myKeys{ return { mainPath =>{ 'DEFAULT' => '${indepProg}' } }; }
package SAPDB::Install::Templates::PackageLCSDK;
our @ISA = qw (SAPDB::Install::Templates::InstUnit);
sub myKeys{ return { mainPath =>{ 'DEFAULT' => '${indepProg}' } }; }
package SAPDB::Install::Templates::PackagePCR;
use SAPDB::Install::SetUser;
use SAPDB::Install::Templates::Util;
our @ISA = qw (SAPDB::Install::Templates::InstUnit);
sub myKeys{ return { mainPath =>{ 'DEFAULT' => '${indepProg}' } }; }



sub GetPCRVersion{
  my $this = shift;
  my $name = $this->getName();
  return $this->getName() =~ /^PCR\s* (\d+)$/ ? $1 : undef; 
}



sub registerWin{
  my $this = shift;
  my $mainPath = $this->getMainPath();
  $mainPath=~s/\//\\/g;
  my $retVal=0;
  my $pcrVersion = $this->GetPCRVersion();
  my $callstring =
    ($pcrVersion >= 7500) ?
      "\"$mainPath\\runtime\\$pcrVersion\\bin\\irconf\" -g -i -p \"$mainPath\\runtime\\$pcrVersion\"" :
        "\"$mainPath\\runtime\\$pcrVersion\\bin\\irconf\" -i -p \"$mainPath\\runtime\\$pcrVersion\"";
  $retVal = callSystem($callstring, sub{}) and return $retVal;
  return $retVal;
}

sub registerUX{
  my $this = shift;
  my $instConfig = $this->getInstTemplate();
  my $indepData = $instConfig->getIndepData();
  my $inifile = catfile($indepData,'/config','Runtimes.ini');
  my $uid = $this->GetUID();
  my $grid = $this->GetGRID();
  my $mainPath = $this->getMainPath();
  my $retVal=0;
  my $pcrVersion = $this->GetPCRVersion();
   if (-f $inifile) {
     chmod (0444, $inifile);
     chown ($uid,$grid,$inifile);
   }
  SetUser($uid) == -1 and return "Cannot set effective uid and gid\n";
#   callSystem('id',sub{return 1;});
   my $callstring =
     ($pcrVersion >= 7404) ?
       "$mainPath/runtime/$pcrVersion/bin/irconf -g -i -p $mainPath/runtime/$pcrVersion" :
         "$mainPath/runtime/$pcrVersion/bin/irconf -i -p $mainPath/runtime/$pcrVersion";
  $retVal = callSystem($callstring, sub{}) and return $retVal;
  SetUser() == -1 and return "cannot set effective uid and gid back to root\n";
  return $retVal;
}

sub register{
  my $this = shift;
  $^O =~ /^MsWin/i ? $this->registerWin() : $this->registerUX();
}

sub unregister{
  my $this = shift;
  $^O =~ /^MsWin/i ? $this->unregisterWin() : $this->unregisterUX();
}

sub unregisterWin{
  my $this = shift;
  my $instConfig = $this->getInstTemplate();
  my $mainPath = $this->getMainPath();
  my $retVal=0;
  my $pcrVersion = $this->GetPCRVersion();
  my $callstring =
    ($pcrVersion >= 7404) ?
      "\"$mainPath\\runtime\\$pcrVersion\\bin\\irconf\" -g -r -p \"$mainPath\\runtime\\$pcrVersion\"" :
        "\"$mainPath\\runtime\\$pcrVersion\\bin\\irconf\" -r -p \"$mainPath\\runtime\\$pcrVersion\"";
  $callstring=~s/\//\\/g;
  $retVal = callSystem($callstring, sub{}) and return $retVal;
  return $retVal;

}
sub unregisterUX{
  my $this = shift;
  my $instConfig = $this->getInstTemplate();
  my $mainPath = $this->getMainPath();
  my $retVal=0;
  my $pcrVersion = $this->GetPCRVersion();
  my $callstring =
    ($pcrVersion >= 7404) ?
      "$mainPath/runtime/$pcrVersion/bin/irconf -g -r -p $mainPath/runtime/$pcrVersion" :
        "$mainPath/runtime/$pcrVersion/bin/irconf -r -p $mainPath/runtime/$pcrVersion";
  $retVal = callSystem($callstring, sub{}) and return $retVal;
  return $retVal;
}


package SAPDB::Install::Templates::PackageCPC;
our @ISA = qw (SAPDB::Install::Templates::InstUnit);
sub myKeys{ return { mainPath =>{ 'DEFAULT' => '${indepProg}' } }; }
package SAPDB::Install::Templates::PackageDBAna;
our @ISA = qw (SAPDB::Install::Templates::InstUnit);
sub myKeys{ return { mainPath =>{ 'DEFAULT' => '${indepProg}' } }; }
package SAPDB::Install::Templates::PackagePython;
our @ISA = qw (SAPDB::Install::Templates::InstUnit);
sub myKeys{ return { mainPath =>{ 'DEFAULT' => '${indepProg}' } }; }
package SAPDB::Install::Templates::PackageWebtools;
our @ISA = qw (SAPDB::Install::Templates::InstUnit);
sub myKeys{ return { mainPath =>{ 'DEFAULT' => '${indepProg}' } }; }
package SAPDB::Install::Templates::PackageXML;
our @ISA = qw (SAPDB::Install::Templates::InstUnit);
sub myKeys{ return { mainPath =>{ 'DEFAULT' => '${indepProg}' } }; }

package SAPDB::Install::Templates::PackageAPO;
use SAPDB::Install::Templates::Util;
our @ISA = qw (SAPDB::Install::Templates::PackageDBKernel);
sub myKeys{
  my $this = shift;
  my $retVal = $this->SUPER::myKeys();
  $retVal->{'mainPath'} = {
                           'DEFAULT' => '${sysProgPath}/sdb/${releaseId}/sap'
                          };
  return $retVal;
}
sub preinstall {return 0;}
sub preuninstall {return 0;}
sub preunpack {return 0;}
sub postprepare {return 0;}
sub postinstall {return 0;}
sub postuninstall {return 0;}
sub register {return 0;}
sub unregister {return 0;}

sub SetInstTemplate{
  my ($this,$value) = @_;
  $this->SUPER::SetInstTemplate($value);
  $this->setMainPath(catfile($this->SUPER::getMainPath(),'sap'));
}



sub isResponsible{
  my ($this) = @_;
  my ($isLiveCache,$retVal) = $this->isLiveCache;
  $retVal and return (undef,$retVal);
  if(not $isLiveCache){
    $this->logit("APO package ignores oltp upgrade\n");
    return (0,$retVal);
  }
  return (1,$retVal);
}






#get internal parameters for the upgrade;
#apo specific:
sub GetInternalParams{
  my ($this,$oldPackage) = @_;
  my $apoTargetRelease = $this->getVersion;
  $apoTargetRelease =~ s/\.(\d+)$/\ Build\ $1/; 
  my ($kernelPackage,$oldKernelPackage) = ($this->getKernelPackage,$this->getOldKernelPackage); 
  my $targetRelease =  $kernelPackage->getBuildString; 
  return ( $apoTargetRelease,$kernelPackage,$oldKernelPackage,$targetRelease);
}




#create a livecache instance
sub createInstance{
  my $this = shift;
  my $retVal=0;
  $retVal= $this->dirtyInstanceCheck() and return  $retVal;
  my $instConfig = $this->getInstTemplate();
  my $depPath = $instConfig->getDependendPath();
  $depPath =~ s/\//\\/g if $^O =~ /mswin/i;
  my $dbname = $instConfig->getDBName();
  my $dbmuser = $instConfig->getDBMUser();
  my $dbauser = $instConfig->getDBAUser();
  my $lcauser = $instConfig->getLCAUser();
  my $lcaencoding = $instConfig->getLCAEncoding();
  $retVal =  $this->call_dbmcli(" -R \"$depPath\" db_create  $dbname $dbmuser") and return $retVal; 
  my $dbm = SAPDB::Install::DBMCmd->new ('', $dbname) or return "DBMCmd not initialized!";
  $retVal = $this->dbmcmd($dbm,"user_logon $dbmuser") and return $retVal;
  $retVal = $this->dbmcmd($dbm,"param_startsession") and return $retVal;
  $retVal = $this->dbmcmd($dbm,"param_init lvc") and return $retVal;
  $retVal = $this->dbmcmd($dbm,"param_checkall") and return $retVal;
  $retVal = $this->dbmcmd($dbm,"param_commitsession") and return $retVal;
  my $params = $instConfig->getDBParams();
  foreach(keys %$params){
    $retVal = $this->dbmcmd($dbm,"param_directput " . $_ . " " . $params->{$_}) and return $retVal;
  }
  $retVal = $this->dbmcmd($dbm,"param_checkall") and return $retVal;
  $retVal = $this->addVolumes($dbm) and return $retVal;
  $retVal = $this->dbmcmd($dbm,"db_admin") and return $retVal;
  $retVal = $this->dbmcmd($dbm,"db_activate $dbauser") and return $retVal;
  $retVal = $this->dbmcmd($dbm,"load_systab -u $dbauser -ud domain") and return $retVal;
  $retVal = $this->dbmcmd($dbm,"db_offline") and return $retVal;
  $retVal = $this->dbmcmd($dbm,"exec_lcinit init -uSQL $lcauser -e $lcaencoding") and return $retVal;
  $retVal = $this->dbmcmd($dbm,"sql_connect  $lcauser") and return $retVal;
#  $retVal = $this->dbmcmd($dbm,"sql_execute enable write transaction") and return $retVal;
  $retVal = $this->dbmcmd($dbm,"sql_execute create table \"__\$__LCFLAG\" " .
                          "(flag CHAR(1),begin CHAR(23), end CHAR(23),version CHAR(12))") and return $retVal;
  $retVal = $this->dbmcmd($dbm,"sql_execute insert into \"__\$__LCFLAG\" values ('X','nix', 'nix','nix')") and return $retVal;
  $retVal = $this->dbmcmd($dbm,"sql_execute grant select on \"__\$__LCFLAG\" to public") and return $retVal;
  $retVal = $this->dbmcmd($dbm,"sql_release") and return $retVal;
  return $retVal;
}



package SAPDB::Install::Templates::UpgradeFile;
our @ISA = qw (SAPDB::Install::Templates::Properties);
sub myKeys{ return { mainPath =>{ 'DEFAULT' => '${indepProg}' } }; }


sub new(){
  my $this = { 'KEYS' =>{
                         'apoStartingRelease'  => { },
                         'startingRelease'  => { },
                         'instanceCheckOK'  => { },
                         'logPos'  => { },
                         'backupLogPos'  => { },
                         'mCOD'  => { },
                         #default character encoding of liveCache-Owner
                         'defaultCode'  => { },
                         #liveCache-Owner
                         'lCOwner'  => { },
                         'migrationStrategy'  => { }
                        }};
  bless $this ;
  $this->SUPER::initData();
  return  $this;
}




# Local Variables: ***
# sdb-current-src:  "/develop3/src"  ***
# sdb-current-main: "SAPDB::Install::Templates::InstallationTest::main" ***
# sdb-current-template:  "/develop3/Install.config"  ***
# sdb-current-exe:  "c:\\WINNT\\system32\\rsh is0031 -l lcroot  chroot  /rel/76_64_D022195/root   /develop3/SDBINST"  ***
# eval:  (setq compile-command (format "%s --inc %s --main %s -template  %s" sdb-current-exe sdb-current-src sdb-current-main sdb-current-template)) ***
# End: ***



1;

