#!/usr/bin/perl
package SAPDB::Install::Templates::Setup;
#ifdef SCRIPTING
#use strict;
#use Getopt::Long;
#use SAPDB::Install::Templates::MobileInfra;
#use SAPDB::Install::Templates::MobileWorkstation;
#use SAPDB::Install::Templates::OpensourceOltpWorkstation;
#use SAPDB::Install::Templates::Base;
#use SAPDB::Install::Templates::Simple;
#use SAPDB::Install::Templates::Util;
#use SAPDB::Install::Templates::WindowsInstaller;
#use SAPDB::Install::FSUsage qw(GetFSSize GetFSName );
#require Exporter;
#our @ISA = qw(Exporter);
#our @EXPORT = qw (  initializeTemplateOptions 
#                    framestart
#                    framestop
#                    postprepare
#                    register
#                    postinstall 
#                    getActiveInstanceTemplate
#                    setActiveInstanceTemplate
#                    preinstall
#                    fsinfo_templates 
#                    getTemplateDescriptions createTemplate  
#                    createInstallTemplateFromConfigLines 
#                    createInstallTemplateFromString 
#                    createInstallTemplateFromFile 
#                    writeInstallTemplateToFile
#                 );
#endif
#because SDBINST
require SDB::Common::Exporter;

use SDB::Common::Getopt::Long;
use SAPDB::Install::Templates::MobileInfra;
use SAPDB::Install::Templates::MobileWorkstation;
use SAPDB::Install::Templates::OpensourceOltpWorkstation;
use SAPDB::Install::Templates::Simple;
use SAPDB::Install::Templates::Util;
use SAPDB::Install::Templates::WindowsInstaller;
use SAPDB::Install::StdIO;
use SAPDB::Install::Registry;
use SAPDB::Install::FSUsage;
use SAPDB::Install::Base;

our @ISA = (SDB::Common::Exporter);

our @EXPORT = qw (initializeTemplateOptions 
                  getActiveInstanceTemplate 
                  setActiveInstanceTemplate 
                  fsinfo_templates
                  getTemplateDescriptions 
                  createTemplate 
                  createInstallTemplateFromConfigLines 
                  createInstallTemplateFromString 
                  createInstallTemplateFromFile
                  writeInstallTemplateToFile);

#endbecause


##############################################################################################
#ERROR HANDLING
##############################################################################################

#Dummy error output
sub myerror{
  #ifdef SCRIPTING
#  print $_[0] ;
  #endif
#because SDBINST
  print2stderr ( $_[0]);
#endbecause
}
#myerror


#Dummy warn
sub mywarn{
  #ifdef SCRIPTING
#  print $_[0];
  #endif
#because SDBINST
  print2stderr ($_[0]);
#endbecause
}
#mywarn


sub onInvalidTemplate($){
  my $message=
    "*********************************************************************\n" .
    "INVALID TEMPLATE SPECIFICATION: \"$_[0]\"\n\n" .
    "Specify a legal value!\n" .
    "At the moment, the only legal value is \"mi\",\n"  .
    "which is the template for the mobile infrastructure.\n" .
    "*********************************************************************\n" ;
#because SDBINST
  print2stdout("\n$message");
#endbecause
  #ifdef SCRIPTING
#  print "$message\n";
  #endif
#  kackAb();
}
#onInvalidTemplate


#Tracing


#use Trace;
#TraceMsg("Message\n",
# 5, # Trace level > 1
# \$DEBUG, # Nie ndern
#0 # caller level
#);



sub logger{
#because SDBINST
 $SAPDB::Install::Values::log->SetMsg($_[0]); 
#endbecause
#ifdef SCRIPTING
#print "SDBINST: ". $_[0];
#endif
}

sub status{
#because SDBINST
 print2stdout("\n" . $_[0]);
#endbecause
#ifdef SCRIPTING
#print "SDBINST: ". $_[0];
#endif
}










##############################################################################################
#Singleton instance vor the active Template
##############################################################################################

#+field(TemplateRef:the active template)
my $activeTemplate ; 
my $setup_opt_instrootpath;
my $setup_opt_configfile;
my $setup_opt_drive;
my $setup_opt_template;
my $setup_opt_dbname;

#Get the active Template as Object instance of type Template
#+sub():TemplateRef:Get the active template
sub getActiveInstanceTemplate(){
  return $activeTemplate;
}
sub setActiveInstanceTemplate{
  my ($aTemplate) = @_;
  $activeTemplate = $aTemplate;
}
#getActiveInstanceTemplate


#Set the active template by specifiying a template type as a string
#undef is a legal value and causes no action
sub setActiveInstanceTemplateByType($){
  my $aTemplateType = $_[0];
  my $aTemplate = createInstanceTemplateByType($aTemplateType);
  if (isMobileInfraType($aTemplateType)) {
    $activeTemplate = $aTemplate;
    if (defined $setup_opt_configfile) {
      $activeTemplate->configureFromFile($setup_opt_configfile);
    }
    if (defined $setup_opt_instrootpath) {
      $activeTemplate->setAdvicedInstRootPath($setup_opt_instrootpath);
    }
    if (defined $setup_opt_drive) {
      $activeTemplate->setDrive($setup_opt_drive);
    }
    $activeTemplate->setStatusCallback(\&status);
    $activeTemplate->setLogCallback(\&logger);
  }
  elsif (isSimpleWorkstationType($aTemplateType)) {
    $activeTemplate = $aTemplate;
    if (defined $setup_opt_configfile) {
      $activeTemplate->configureFromFile($setup_opt_configfile);
    }
    $activeTemplate->setStatusCallback(\&status);
    $activeTemplate->setLogCallback(\&logger);
  }
  elsif (defined $aTemplateType) {
    return undef;
  }
  else {
    return undef;
  }
}
#setActiveInstanceTemplateByType

sub createInstanceTemplateByType{
  my $aTemplateType = $_[0];
  my $template;
  if (isMobileInfraType($aTemplateType)) {
    $template = SAPDB::Install::Templates::MobileInfra::new();
  }
  elsif (isSimpleWorkstationType($aTemplateType)) {
    $template = SAPDB::Install::Templates::Simple::new();
  }
  return $template;
}




sub isMobileInfraType($){
  return ( defined $_[0] and $_[0]  eq "mi");
}

sub isSimpleWorkstationType($){
my $aType = $_[0];
  return ( defined $aType and $aType  eq "simple");
}


sub isSimpleWorkstationTemplate($){
  my $aTemplate = $_[0];
  return (defined $aTemplate and isSimpleWorkstationType($aTemplate->getType()));
}



sub isMobileInfraTemplate($){
  my $aTemplate = $_[0];
  return ( defined $aTemplate and isMobileInfraType(  $aTemplate->getType()));
}







##############################################################################################
#Package script methods
##############################################################################################






sub initializeTemplateOptions(){
#ifdef SCRIPTING
#  GetOptions( 'config=s' =>\$setup_opt_configfile , 
#              'instrootpath=s' , \$setup_opt_instrootpath,
#               'drive=s', \$setup_opt_drive,
#               'template=s', \$setup_opt_template,
#                'd=s', \$setup_opt_dbname);
#endif

#because SDBINST
  $setup_opt_configfile  = $SAPDB::Install::StartInstall::opt_config;
   $setup_opt_instrootpath = $SAPDB::Install::StartInstall::opt_instrootpath;
   $setup_opt_drive = $SAPDB::Install::StartInstall::opt_drive;
   $setup_opt_template = $SAPDB::Install::StartInstall::opt_template;
    $setup_opt_dbname = $SAPDB::Install::StartInstall::opt_d;
#endbecause

  if( defined $setup_opt_template){
    setActiveInstanceTemplateByType($setup_opt_template);
  }

  if( defined $setup_opt_configfile){
    if( (not -f $setup_opt_configfile)){
      $setup_opt_configfile=undef;
      kackAb("Configuration file not found: $setup_opt_configfile\n");
    }
    else{
      my $section;
      my $templateType;
      open CONFIGFILE, "<$setup_opt_configfile";
      while(<CONFIGFILE>){
        parseConfigLine
            ( $_,
              sub{ $section =$_[0];},
              sub {
                my $key = $_[0];
                my $val = $_[1];
                parseKey($section,$key,$val,\$templateType,"Template","type");
              });
      }#while
      close CONFIGFILE;
      if(defined $templateType){
        setActiveInstanceTemplateByType($templateType);
      }
    }#else
  }#if
}
#initializeTemplateOptions

sub parseKey{
my ($section,$key,$val,$valueRef,$realSection,$realKey)=@_;
#print "gotcha:Section: $section: $key=$val\n";
if( defined $section and $section eq $realSection and defined $key and $key =~ /$realKey/i ){
  if( defined $val and  $val =~ /^\s*(\w+)\s*$/ ){
    $$valueRef=$1;
  }}
}

sub createInstallTemplateFromConfigLines{
  my ($linesRef) = @_;
  my @lines= @$linesRef;
  my $template=undef;
  my $instanceTemplate=undef;
  my $instanceTemplateType=undef;
  my $templateType=undef;
  my $isDBMGUI=0;
  my $isSQLStudio=0;
  my ($section,$key,$value);
  foreach my $line (@lines) {
    parseConfigLine
        ( $line,
          sub{
            $section =$_[0];
            if( $section eq "SQL Studio"){
              $isSQLStudio=1;
            }
            elsif( $section eq "Database Manager"){
              $isDBMGUI=1;
            }
          },
          sub {
            $key = $_[0];
            $value = $_[1];
            parseKey($section,$key,$value,\$instanceTemplateType,"DBInstance","type");
            parseKey($section,$key,$value,\$templateType,"Template","TEMPLATECODE");
          });
  }
  if(defined $templateType){
    $template = createTemplate($templateType);
    if(defined $instanceTemplateType){
      $instanceTemplate = createInstanceTemplateByType($instanceTemplateType);
      $template->setIsCreateInstance("yes");
      $template->setDBInstance($instanceTemplate);
      $instanceTemplate->configureFromLines(\@lines);
    }
    if($isSQLStudio){
      my $sqlStudio = SQLSTUDIO();
      $template->setIsSQLStudio($isSQLStudio);
      $sqlStudio->configureFromLines(\@lines);
      $template->setSQLStudio($sqlStudio);
    }
    if($isDBMGUI){
      my $dbmGui = DBM();
      $template->setIsDBMGui($isDBMGUI);
      $dbmGui->configureFromLines(\@lines);
      $template->setDBMGui($dbmGui);
    }
    $template->configureFromLines(\@lines);
  }
  return $template;
}
#createInstallTemplateFromConfigLines



sub createInstallTemplateFromString{
  my ($aString) = @_;
  my @lines = split /\n/ ,$aString;
  return createInstallTemplateFromConfigLines(\@lines);
}

#+sub(File:fileName|Sub:callBack):Template:Liest ein Install Template von einer Datei
sub createInstallTemplateFromFile{
  my ($file,$callBack) = @_;
  my $template=undef;
  if( open CONFIG, $file){
    my @lines = <CONFIG>;
    $template =  createInstallTemplateFromConfigLines(\@lines);
    close CONFIG;
  }
  elsif (defined $callBack){
    &$callBack("Cannot open install template $file\n");
  }
  return $template;
}





#+sub(Template:aTemplate|File:fileName|CallBack|aCallBack):Template:Schreibt ein Install-Template in eine Datei.
sub writeInstallTemplateToFile{
  my ($template, $file,$callBack) = @_;
  my $retVal=undef;
  unless(defined $template){
    $retVal=0;
    return;
  }
  if( open CONFIG, ">".$file){
    my $aWrite = sub{ print CONFIG $_[0];};
    $template->writeConfig($aWrite);
    close CONFIG;
    $retVal=1;
  }
  elsif (defined $callBack){
    &$callBack("Cannot open $file to write:   $!\n");
    $retVal=0;
  }
  else{
    $retVal=0;
  }
  return $retVal;
}

#+sub(HashRef:paths|HashRef:config):void:Called from SDBINST during package installation
sub framestart($$){
  initializeTemplateOptions();
  my $configref = $_[1];
  my %config = %$configref;
  my $aTemplateType =$config{'template'};
  my $anActiveInstanceTemplate = getActiveInstanceTemplate();

  if( (defined $aTemplateType) && (defined $anActiveInstanceTemplate) ){
    my $currentType = $anActiveInstanceTemplate->getType();
    unless( $currentType eq  $aTemplateType){
      mywarn "framestart: Template type is already set to \"$currentType\" " . 
        "and cannot be changed to \"$aTemplateType\"!\n";
    }
  }
  elsif (defined $aTemplateType){
    setActiveInstanceTemplateByType($aTemplateType);
  }

#Now the active template must be there !!!!!

  $anActiveInstanceTemplate = getActiveInstanceTemplate();


  if(isMobileInfraTemplate($anActiveInstanceTemplate)){
     mobileInfraFramestart($_[0],$_[1]);
   }
  elsif( isSimpleWorkstationTemplate($anActiveInstanceTemplate)){
     simpleWorkstationFramestart($_[0],$_[1]);
   }
  return 1;
}
#framestart




 sub mobileInfraFramestart{
  my $configref = $_[1];
  my %config = %$configref;
  my $aArchiveName =$config{'archive_name'};
  my $aBackupName =$config{'backup_name'};
  my $aTgzHandler =$config{'tgz_handle'};
  my $anActiveInstanceTemplate = getActiveInstanceTemplate();
  my $sloppy = $anActiveInstanceTemplate->getGeneric("sloppy");

  if( defined $aArchiveName){
    $anActiveInstanceTemplate->setArchive($aArchiveName);
  }
  if( defined $aBackupName){
    $anActiveInstanceTemplate->setBackup($aBackupName);
  }
  if( defined $setup_opt_dbname){
    $anActiveInstanceTemplate->setDBName($setup_opt_dbname);
  }

  my ($indep_data,$indep_prog)=readIndepPath(); # try to get independent paths from earlier installation

  if(defined $indep_prog and $indep_prog =~ /\S/){

     my $aFilename = basename($indep_prog);
     my $aDirname = dirname($indep_prog);

     unless(defined $sloppy){
       unless( $aFilename =~ /^indep_prog$/i or $aFilename =~ /^programs$/i){
         kackAb("ERROR: Independend Programs path \"$indep_prog\" not supported for this Profile!");
       }
       unless ($aDirname =~ /^\s*([a-z]:)?[\\\/]s(ap)?db[\\\/]?\s*$/i){
         kackAb("ERROR: Independend Programs path \"$indep_prog\" not supported for this Profile!");
       }
     }

    if( $aFilename =~ /^indep_prog$/i or $aFilename =~ /^programs$/i){
      if ($aDirname =~ /\S\s\S/){
        mywarn "WARNING: I ignored independend programs path that does not follow filename conventions:\n\"$indep_prog\"\n";
      }
      else{
        my $root = $aDirname;
        $anActiveInstanceTemplate->setAdvicedInstRootPath($root); 
      }
    }
    else{
      mywarn "WARNING: I ignored independend programs path that does not follow filename conventions:\n\"$indep_prog\"\n";
    }
  }else{
    $SAPDB::Install::StartInstall::opt_indep_prog =
      catfile( $anActiveInstanceTemplate->getInstRootPath() ,  "programs");
  }


  if(defined $indep_data and $indep_data =~ /\S/){
  }else{
    my $newIndepData = 
      catfile( $anActiveInstanceTemplate->getInstRootPath() ,  "data");
    $SAPDB::Install::StartInstall::opt_indep_data = $newIndepData;
  }

  $SAPDB::Install::StartInstall::opt_depend=
    $anActiveInstanceTemplate->getDependendPath();
}
#mobileInfraFramestart


 sub simpleWorkstationFramestart{
  my $configref = $_[1];
  my %config = %$configref;
  my $anActiveInstanceTemplate = getActiveInstanceTemplate();
  if( defined $setup_opt_dbname){
    $anActiveInstanceTemplate->setDBName($setup_opt_dbname);
  }
  my ($indep_data,$indep_prog)=readIndepPath(); # try to get independent paths from earlier installation
  #What will we do here?
  if(defined $indep_prog and $indep_prog =~ /\S/){
   }
}
#simpleWorkstationFramestart




#ifdef SCRIPTING
#sub readIndepPath{
## ("c:\\Sdb\\Data", "k:\\Sapdb\\Programs");
#}
#endif



#+sub(HashRef:paths|HashRef:config):void:Called from SDBINST during uninstall
sub unregister{
  return 1;
}
#unregister




#+sub(HashRef:paths|HashRef:config):void:Called from SDBINST during uninstall
sub preuninstall{
#print "Hello world in preuninstall\n";
return 1;
}
#preuninstall
#+sub(HashRef:paths|HashRef:config):void:Called from SDBINST during uninstall
sub postuninstall{
#print "Hello world in postuninstall\n";
return 1;
}
#postuninstall

#+sub(HashRef:paths|HashRef:config):void:Called from SDBINST during sdbverify and installation
sub verify{
#print "Hello world in verify\n";
  return 1;
}
#verify







#+sub(HashRef:paths|HashRef:config):void:Called from SDBINST during package installation
sub preinstall{
#  my ($pathlistref, $configref) = @_;
#  my ($pathspecref, %config) = (@$pathlistref[0],%$configref);
#  my %pathspec = %$pathspecref;
  return 1;
}#preinstall

#Ab hier dependend path gesetzt.
#ifdef sb:postprepare
#<p>Sets the <code>dependend path</code> in the active template.</p>
#endif
#+sub(HashRef:paths|HashRef:config):void:Called from SDBINST during package installation
sub postprepare{
  my $pathlistref = $_[0];
  my $pathspecref = @$pathlistref[0];
  my %pathspec = %$pathspecref;
  my $dependend = $pathspec{'value'};
  my $anActiveInstanceTemplate = getActiveInstanceTemplate();
  if (defined $anActiveInstanceTemplate && defined $dependend) {
    $anActiveInstanceTemplate->setDependendPath($dependend);
  }
  return 1;
}
#postprepare

#+sub(HashRef:paths|HashRef:config):void:Called from SDBINST during package installation
sub postinstall($$){


}
#postinstall






#registrierung z.B. in Windows-Registry
#+sub(HashRef:paths|HashRef:config):void:Called from SDBINST during package installation
sub register($$){
  return 1;
}
#register

#ifdef sb:framestop
#<p>Creates the database instance.</p>
#endif
#+sub(HashRef:paths|HashRef:config):void:Called from SDBINST during package installation
sub framestop($$){
  my $pathlistref = $_[0];
  my $configref = $_[1];
  my $anActiveInstanceTemplate = getActiveInstanceTemplate();
  $anActiveInstanceTemplate->writeConfig(\&logger);
  my $result = $anActiveInstanceTemplate->createInstance();
  unless(defined $result){
    kackAb("Error creating database instance!");
  }
  return $result;
}
#framestop



sub fsinfo_templates{
  my ($hFileSystemInfoRef) =@_;
  my $activeTemplate = getActiveInstanceTemplate();
  if (isMobileInfraTemplate($activeTemplate)) {
    my %hFileSystemInfo = %$hFileSystemInfoRef;
    foreach my $fsname (keys(%hFileSystemInfo)) {
      my $needed = ${%{$hFileSystemInfo{$fsname}}}{'NEEDED'};
      my $available = ${%{$hFileSystemInfo{$fsname}}}{'AVAIL'};
      # ${%{$hFileSystemInfo{$fsname}}}{'USED'}=9999;
      my $used = ${%{$hFileSystemInfo{$fsname}}}{'USED'};


      my $volumes =  $activeTemplate->getDBVolumes();
      foreach my $volume (@$volumes) {
        my $avolumeKind =  @$volume[1];
        my $avolumePath =  @$volume[2];
        my $avolumeType =  @$volume[3];
        my $avolumeSize =  @$volume[4];
        #
        my $avolumeSizeInKb = ($avolumeSize+1) / 8; 

        if("F" eq $avolumeType){
          #Strip fsName to existing
          my $lastPath ;
          my $existingPartOfVolumePath = $avolumePath;
          while ( $existingPartOfVolumePath and not -e $existingPartOfVolumePath){
            $existingPartOfVolumePath = dirname($existingPartOfVolumePath);
            if(defined $lastPath and $lastPath eq $existingPartOfVolumePath){
              $existingPartOfVolumePath = undef;
            }
            $lastPath=$existingPartOfVolumePath;
          }
          if(defined $existingPartOfVolumePath){
            my $fsName = GetFSName($existingPartOfVolumePath);
            #print "Volume (full,existing,fsname) (\"$avolumePath\",\"$existingPartOfVolumePath\",$fsName)\n" . 
            #      "       (needed,available,used): ($needed,$available,$used,)\n";
          }
          else{
            mywarn("WARNING: Cannot find Filesystem for path \"$avolumePath\"\n" . 
                   "Is this a valid path?\n");
          }
          #else
        }
        #if
      }
      #foreach
    }
    #foreach
  }
  #if
}
#fsinfo_templates()


my $statusCallback=undef;
#+ Anwendung beenden
sub kackAb(){
#because SDBINST
  diesoft $_[0];	
#endbecause
  #ifdef SCRIPTING
#  die $_[0];
  #endif
}#kackAb



  my $templateListRef=undef;

#+sub():return:TemplateList

#+sub(void):ListReference:Get the list of template descriptions
sub getTemplateDescriptions{
my $desktopDescr = <<"AEND";
Typical installation for a single user environment. 
Can be used during development of simple database applications, for 
the MaxDB tutorial or for testing purposes.
AEND
$desktopDescr =~ s/\n//g;

my $customDescr = <<"AEND";
Customizable installation of a databases instance. 
Default values for memory usage, cpu usage and other parameters are calculated and can be adapted.
 Any number of data volumes on multiple hard disks can be specified. 
AEND
$customDescr =~ s/\n//g;


  my $retVal = 
    [
     {
      "label" => "General",
      "data" =>
      [
       {
        "TemplateCode" => "templ_desktop",
        "ShortDescription" => "Desktop PC / Laptop",
        "LongDescription" =>  $desktopDescr
       }
      ]
     } ,
     {
      "label" => "Custom",
      "data" =>
      [
       {
        "TemplateCode" => "templ_server_advanced",
        "ShortDescription" => "Custom",
        "LongDescription" => $customDescr 
       }
      ]
     }
    ] ;


  return $retVal;
}


#+sub(String:templateCode):Template:Create an instance of a template
sub createTemplate{
  my ($templateCode) = @_;
  my $aTemplate = undef;
  if ( $templateCode eq "templ_desktop") {
    $aTemplate = SAPDB::Install::Templates::OpensourceOltpWorkstation::new();
    $aTemplate->setTemplateCode("templ_desktop");
  }
  elsif ( $templateCode eq "templ_server_small") {
    $aTemplate = SAPDB::Install::Templates::OpensourceOltpWorkstation::new();
    $aTemplate->setTemplateCode("templ_server_small");
  }
  elsif ( $templateCode eq "templ_server_medium") {
    $aTemplate = SAPDB::Install::Templates::OpensourceOltpWorkstation::new();
    $aTemplate->setTemplateCode("templ_server_medium");
  }
  elsif ( $templateCode eq "templ_server_advanced") {
    $aTemplate = SAPDB::Install::Templates::OpensourceOltpWorkstation::new();
    $aTemplate->setTemplateCode("templ_server_advanced");
  }
  elsif( $templateCode eq "templ_mi_workstation") {
    $aTemplate = SAPDB::Install::Templates::MobileWorkstation::new();
    $aTemplate->setTemplateCode("templ_mi_workstation");
  }
  return $aTemplate;
}



1;



#Reihenfolge der Aufrufe der Package-Schnittstelle
#
#framestart
#preinstall (Bedeutet praktisch prepare)
#postprepare (Ab hier dependend path gesetzt)
#postinstall
#register
#framestop



#ifdef doc
#<h1>SAPDB::Install::Templates::Setup </h1>
#<h2>Usage </h2>
#<h3>From the installation package</h3>
#<pre>
#sub framestart{
#  Setup.framestart(..);
#}
#sub preinstall{
#  Setup.preinstall(..);
#}
##And so on
#</pre>
#<h3>Access the active template</h3>
#<pre>
#my $activeTemplate = Setup.getActiveInstanceTemplate();
#</pre>
#<h2>What it does</h2>
#<ol>
#<li>Entry points for SDBINST-Package "DB Instance":
#<ul>
#<li><code>framestart()</code></li>
#<li><code>preinstall()</code></li>
#<li><code>and so on ...</code></li>
#</ul>
#</li>
#<li>Manage Singleton Instance of  <a href="Base.html">Install Template</a>:
#<ul class="c1">
# <li>At runtime of SDBINST, there is at most one <strong>Singleton</strong>-Instance of
# the <code>Install Template</code></li>
# <li>Name this singleton <strong>Active Template</strong></li>
# <li>Access the <strong>Active Template</strong> via <code>Setup.getActiveInstanceTemplate()</code></li>
# </ul>
# </li>
#<li>Configure the <strong>Active Template</strong> (Orderer by priority):
#<ol class="c2">
#<li> via command line parameters</li>
#<li> via ini file</li>
#<li> via info passed by the install package sub methods</li>
#<li> via default parameters from the template module</li>
#</ol>
#  </li>
#</ol>
#
#<h2>How it works</h2>
#<ol>
# <li> SDBINST starts</li>
# <li> <code>framestart()</code> in package script  is called and calls ...
#       <ul>
#    <li> <code>Setup.framestart()</code>,which handles the Active Template</li>
#       </ul>
#</li>
# <li> <code>preinstall()</code> in package script is  called and calls ...
#   <ul>
# <li> <code>Setup.preinstall()</code>,which  handles the Active Template</li>
#  </ul>
#  </li>
# <li> and so on ...
# <strong>(At every step, configuration data may be transferred and actions may be triggered.)</strong>
# </li>
#</ol>
#endif



# sdbinst-testfile: "SetupTestSimple.pl" ***
# sdbinst-testfile: "TemplateTest.pl" ***
# sdbinst-testfile: "SetupTestSimple.pl" ***



# Local Variables: ***
# perl-dbg-flags:"-w      -I../../../../../../wrk/script -I../../../sdbrun  " ***
# sdbinst-defines: "-define SCRIPTING=YES -define PERL5.6.1=YES" ***
# sdbinst-testfile: "TemplateTest.pl" ***
# sdbinst-script-directory:  "sys/wrk/script/SAPDB/Install/Templates" ***
# End: ***

