package SAPDB::Install::Templates2::Installation;
use SDB::Common::Getopt::Long;
use SAPDB::Install::System;
use SAPDB::Install::Registry;
use SAPDB::Install::Values;
use SAPDB::Install::SetDebug;
use SAPDB::Install::Tools;
use SAPDB::Install::SysInfo;
use SAPDB::Install::Templates2::InstRegistry;
use SAPDB::Install::Misc;
use SAPDB::Install::Templates2::Util;
use SAPDB::Install::Templates2::InstUnit;
use SAPDB::Install::Templates2::Archiver;
use SAPDB::Install::Templates2::Debug;
use strict;

our $DEBUG=1;
my $callerDir;
my $installerDir;
my $instRegistry;
our $instConfig;
#runtime variables;
my $packageInfo;
my $APO = 'APO COM';
my $KERNEL = 'Database Kernel';

our @ISA;
if($SCRIPTING){
  require Exporter;
  push @ISA,qw(Exporter);
}
else{
  require SDB::Common::Exporter;
  push @ISA,qw(SDB::Common::Exporter);
}
our @EXPORT = qw(  );



sub getPackageInfo(){
  my $retVal=0;
  unless($packageInfo){
    my $archiveDir = $instConfig->getArchiveDir();
    ($retVal,$packageInfo) = getAllPackageData(defined $archiveDir ? genAbsPath($archiveDir) : undef);	
  }
  return ($retVal,$packageInfo);
}
sub getPackages(){
  my ($retVal,$aPackInfo) = getPackageInfo();
  $retVal and return ($retVal,undef);
  my @packList = sortPacks($aPackInfo->{'packages'});
  return ($retVal,\@packList);
}
sub getProfiles(){
  my ($retVal,$aPackInfo) = getPackageInfo();
  $retVal and return ($retVal,undef);
  my $packList = $aPackInfo->{'profiles'};
  return ($retVal,$packList);
}


sub handleTrace{
  my $opt_trace;
  my %opt_ctrl=( 'trace=s' => \$opt_trace );
  GetOptions(%opt_ctrl);
  if ($opt_trace) {
    TraceLevel($opt_trace);
    TraceMsg("Trace is $opt_trace\n",1,\$DEBUG,1);
  }
}
sub listIt{
  my ($retVal,$packRef,$profRef) = (0);
  initialize();
  status( "\n");
  showEnvironment();
  my @installedPackages = $instRegistry->GetAllPackageObjects($instConfig);
  if(@installedPackages){
    status( "\n");
    status("[ALREADY INSTALLED PACKAGES]\n");
    foreach(@installedPackages){
      my $name = $_->getName();
      my $path = $_->getMainPath();
      my $version = $_->getVersion();
      status( "$name $version [$path]\n");
    }
  }
  status( "\n");

  my %instances = SAPDB::Install::Registry::getInstances();  	
  if(keys %instances){
    status( "[INSTANCES ON LOCAL HOST]\n");
  }
  foreach my $instance_name (keys(%instances)){
    my $path = $instances{$instance_name};
    status( "$instance_name $path\n");
  }
  status( "\n");
  status( "[PROFILES IN INSTALLATION DIRECTORY]\n");
  ($retVal,$profRef)= getProfiles();
  $retVal and return $retVal;
  my @profiles = @$profRef;
  ($retVal,$packRef)= getPackages();
  $retVal and return $retVal;
  my @packList = @$packRef;
  my $profD = join("\n",@profiles);
  foreach (@profiles) {
    my @aList = listProfile($_,\@packList);
    status( "(".join(',',@aList) . ")");
    status( "\n");
  }
  status( "\n\n");
  status( "[PACKAGES IN INSTALLATION DIRECTORY]\n");
  foreach my $packdata (@packList) {
    my $displayName = $packdata->{'DISPLAY_NAME'};
    my $packageName = $packdata->{'PACKAGE_NAME'}  ;
    my $isSub = $packdata->{'IS_SUBPACKAGE'}  ;
    my @deps =  GetDepPacks($packdata,\@packList);
    my $version =  $packdata->{'SOFTWARE_VERSION'}  ;
    status(   $packageName);
    status(  " ($version)\n") ;
    status( " Requires: (");
    my @depNames;
    foreach (@deps) {
      my $aName = $_->{"PACKAGE_NAME"};
      if ($isSub) {
        $aName .= " [Subpackage]"
      } else {
      }
      push @depNames,($aName);
    }
    status( join(",",@depNames));
    status( ")\n");
  }
  status( "[VERSION INFORMATION]\n");
  printVersion();
  return $retVal;
}                               #handleLOption





sub listProfile{
  my ($profName,$l) = @_;
  my @retVal;
  foreach my $pack (@$l) {
    my $part = $pack->{'PART_OF'};
    my $name = $pack->{'PACKAGE_NAME'};
    my @deps ; 
    if($part){
      @deps = split(/,/,$part);
      foreach(@deps){
        if(/\s*$profName\s*/){
          push @retVal, ($name);
        }
      }
    }
  }
  return @retVal;
}





sub handleEnvironment{
  my $indepData = $instConfig->getIndepData();
  my $indepProg = $instConfig->getIndepProg();
  $callerDir =      $SAPDB::Install::Values::curDir;
  $installerDir =         $SAPDB::Install::Config{'RuntimeDir'};
  setPathInEnv("$indepProg/bin");
  setPathInEnv("$indepProg/pgm") if $^O =~ /mswin/i;
}                               #handleEnvironment
sub showEnvironment{
  my $sysinfo = GetSystemInfo();
  my $os;
  if (defined $sysinfo) {
    $os = $sysinfo->{'system'}.' '.$sysinfo->{'architecture'}.' '.$sysinfo->{'version'};
    $os .= ' '.$sysinfo->{'subversion'} if exists $sysinfo->{'subversion'}; 	
    $os .= ' '.$sysinfo->{'c_runtime'} if exists $sysinfo->{'c_runtime'}; 	
  } else {
    $os = $^O;
  }
  my $archiveDir = $instConfig->getArchiveDir();
  my $indepData = $instConfig->getIndepData();
  my $indepProg = $instConfig->getIndepProg();
  my $sysProgPath=$instConfig->getSystemProgPath();
  my $appDataPath=$instConfig->getApplicationDataPath();
  my $regLoc=$instRegistry->getLocation();
  status("[ENVIRONMENT]\n");
  status("Package Directory:               $archiveDir\n") if (defined $archiveDir);
  status("System Programs  Directory:      $sysProgPath\n") if (defined $sysProgPath);
  status("System Appl. Data  Directory:    $appDataPath\n") if (defined $appDataPath);
  status("Independend Data  Directory:     $indepData\n") if (defined $indepData);
  status("Independend Programs  Directory: $indepProg\n") if (defined $indepProg);
}
sub GetPackagesToInstall{
  my ($retVal,$allPackRef) = (0);
  ($retVal,$allPackRef) = getPackages();
  $retVal and return ($retVal,undef);
  my @list = initializePackages($allPackRef);
  my @packObjList;
  my $apoPackage;
  my $kernelPackage;
  foreach my $packagedata (@list) {
    my $newpackage = SAPDB::Install::Templates2::InstUnit::new($packagedata,$instConfig);
    if($KERNEL eq $newpackage->getName()){
      $kernelPackage = $newpackage;
    }
    elsif($APO eq $newpackage->getName()){
      $apoPackage = $newpackage;
    }
    push @packObjList,$newpackage;
  }
  if($apoPackage and $kernelPackage){
    $apoPackage->setKernelPackage($kernelPackage);
  }
  return ($retVal,\@packObjList);
}

sub GetInstalledPackages{
  my $newPackages  = shift;
  my %installedPackObjs;
  my $oldkernelpackage;
  foreach my $packobj (@$newPackages){
    my $path = $packobj->getMainPath();
    my $name = $packobj->getName();
    my $installedVersion = $instRegistry->GetVersion($name,$path);
    my $newVersion = $packobj->getVersion();
    if ($installedVersion) {
      my $oldPackage = $instRegistry->createPackageObject($name,$path,$instConfig);
      if($KERNEL eq $name){
        $oldkernelpackage = $oldPackage;
      }
      $installedPackObjs{$packobj}=$oldPackage;
    }
  }
  foreach my $packobj (@$newPackages){
    my $path = $packobj->getMainPath();
    my $name = $packobj->getName();
    if($APO eq $name ){
      $packobj->setOldKernelPackage($oldkernelpackage);
    }
  }
  return %installedPackObjs;
}

sub GetPackagesToUnInstall{
  my ($allPackages) = @_;
  my $l = $instConfig->getRemoveList();
  my @list;
  foreach(@$l){
    my $name = $_->[0] ;
    my $path = $_->[1] ;
    my $version = $instRegistry->GetVersion($name,$path);
    if($version){
      my $obj = $instRegistry->createPackageObject($name,$path,$instConfig);
      push @list, ($obj);
    }
    else{
      warn "WARNING: Ignored package $name\[$path\] (Not installed)\n";
    }

  }
  return @list;
}



sub doFrameStart{
  my ($packObjList,$olds) = @_;
  foreach my $packobj (@$packObjList) {
    my $d=$packobj->displayString;
    status($d);
    my $result = $packobj->framestart($olds->{$packobj}) ;
    if($result){
      status( "   [ERROR]\n");
      return $result;
    }
    else{
      status( "   [OK]\n");
    }
  }
  return 0;
}

sub initialize{
  my $retVal =0;
  SDB::Common::Getopt::Long::Configure('pass_through');
  SDB::Common::Getopt::Long::Configure('noauto_abbrev');
  SetDebug(\@ARGV);
  handleEnvironment();
  $instRegistry = createRegistry($instConfig);
  return $retVal;
}
sub createRegistry{
  my $aInstConfig =  shift;
  my $aInstRegistry;
  my $regFile  = $aInstConfig->getRegistryFile();
  if (-f $regFile) {
    $aInstRegistry = SAPDB::Install::Templates2::InstRegistry::new($regFile);
    unless($aInstRegistry){
      return "Registry Initialization failed\n"; 
    }
  } else {
    $aInstRegistry = SAPDB::Install::Templates2::InstRegistry::new();
    $aInstRegistry->setLocation($regFile);
  } 
  return $aInstRegistry;
}


sub install {
  my $forceReinstall = shift;
  status( "\n[MAXDB INSTALLATION AND UPGRADE]\n\n");
  my $retVal; 
  $retVal = initialize() and return $retVal;
  showEnvironment();
  my $instPackRef;
  ($retVal,$instPackRef) =  GetPackagesToInstall();
  $retVal and return $retVal;
  my @packObjList = @$instPackRef;
  my %installedPackObjs = GetInstalledPackages(\@packObjList);
  status( "\n[PREPARATION PHASE]\n");
  $retVal = doFrameStart(\@packObjList,\%installedPackObjs) and return $retVal;
  status( "\n[TEST PHASE]\n");
  foreach my $packobj (@packObjList) {
    # write headline
    my $dispString=$packobj->displayString;
    my $path = $packobj->getMainPath();
    my $newVersion = $packobj->getVersion();
    my $name = $packobj->getName();
    my $installedVersion = $instRegistry->GetVersion($name,$path);
    my $fromTo;
    if (defined $installedVersion and $installedVersion lt $newVersion) {
      $fromTo =  $installedVersion . " -> " . $newVersion;
    } elsif (defined $installedVersion and $installedVersion gt $newVersion) {
      return "Cannot downgrade $name " .  $installedVersion . " -> " . $newVersion . "\n";
    } elsif (defined $installedVersion and $installedVersion eq $newVersion) {
      $fromTo =  $installedVersion . " <-> " . $newVersion;
    } else {
      $fromTo = "$newVersion";
    }
    if (defined $installedVersion and $installedVersion eq $newVersion and not $forceReinstall) {
      status("$dispString ($fromTo): [SKIPPED]\n");
    } else {
      status("$dispString ($fromTo): ");
      my $prepResult = $packobj->prepare(); 
      my $res = $prepResult ? 'ERROR' : 'OK'; 
      status("[$res]\n");
      $prepResult and  return $prepResult;
    }
  }
  status( "\n[INSTALLATION PHASE]\n");
  foreach my $packobj (@packObjList) {
    my $name = $packobj->getName();
    my $path = $packobj->getMainPath();
    my $tmpstring=$packobj->displayString;
    my $installedVersion = $instRegistry->GetVersion($name,$path);
    my $newVersion = $packobj->getVersion();
    my $result=0;
    unless($forceReinstall){
      if (defined $installedVersion and $installedVersion eq $newVersion) {
        status( "$tmpstring: [SKIPPED]\n");
        next;
      }
    }
    status( "$tmpstring: ");
    if ($installedVersion) {
      my $oldPackage = $installedPackObjs{$packobj};
      $result = $packobj->doUpdate($oldPackage);
    } else {
      $result = $packobj->install();
    }
    if ($result ) {
      status ("[FAILED]\n");
      return $result;
    } else {
      $instRegistry->addPackage($packobj);
      my $location = $instRegistry->getLocation();
      my $regDir =  dirname($location);
      unless(-d $regDir){
        makedir($regDir,0555);
        unless(-d $regDir){
          return "Error creating directory $regDir\n";
        }
      }
      $retVal  =  $instRegistry->save() and 
        return "Cannot save Registry '$location':\n".$retVal;
      my $action;
      if (defined $installedVersion and $installedVersion lt $newVersion) {
        $action = 'UPDATED';
      } elsif (defined $installedVersion and $installedVersion eq $newVersion) {
        $action = 'REINSTALLED';
      } else {
        $action = 'INSTALLED';
      }
      status( " [$action]\n");
    }
  }
  #FRAMESTOP:
  status( "\n[FINALIZATION PHASE]\n");
  foreach my $packobj (@packObjList) {
    my $displayS=$packobj->displayString;
    status( $displayS .  ": ");
    my $result = $packobj->framestop($installedPackObjs{$packobj});
    my $fin = $result? "[ERROR]" : "[OK]";
    status( $fin .  "\n");
    $result and return  $result ; 
  }
  return 0;	
}#install




sub uninstall {
  status( "\n[MAXDB UNINSTALLATION]\n\n");
  my $res = initialize();
  $res  and return $res;
  showEnvironment();
  my @list = GetPackagesToUnInstall();
  @list or status( "[NO PACKAGES FOUND TO UNINSTALL]\n") and return 0;
  status( "[PREPARATION PHASE]\n");
  foreach my $packobj (@list) {
    my $dispName = $packobj->displayString;
    status( "$dispName: ");
    my $res = $packobj->preuninstall() ;
    my $msg = $res ? "ERROR" : "OK";
    status( "[$msg]\n");
    $res and return $res;
  }
  status( "\n[UNINSTALLATION PHASE]\n");
  foreach my $packobj (@list) {
    $instRegistry->SetPackageValidFlag($packobj,0);
    my $dispName = $packobj->displayString;
    my $res =0; 
    status( "$dispName: ");
    $res = doUninstallPackage($packobj);
    my $msg = $res ? "ERROR" : "OK";
    status( "[$msg]\n");
    $res and return $res;
  }
  return 0;
}


sub doUninstallPackage{
  my $packobj = shift;
  my $res =0; 
  $res = $packobj->unregister() and return $res;
  $res = $packobj->uninstall() and return $res;
  $res = $packobj->postuninstall() and return $res;
  $instRegistry->removePackage($packobj);
  $instRegistry->save();
  return $res;
}



sub status{
  my $msg = shift;
  foreach(@{$instConfig->GetListeners('status')}){
    &$_($msg);
  }
}



sub parseDeps{
  my $aString = shift;
  my $retVal = [];
  unless($aString){
    return $retVal;
  }
  my @dep = split(/,/ , $aString);
  foreach (@dep) {
    my @a = split(/(<=|>=|<|>|=)/);
    my $b = [];
    foreach (@a) {
      s/\s*$//;
      s/^\s*//;
      push @$b,($_);
    }
    push @$retVal,($b);
  }
  return $retVal;
}
sub parseDepPackNames{
  my $deps = parseDeps(shift);
  my @retVal;
  foreach (@$deps) {
    push @retVal,($_->[0]);
  }
  return @retVal;
}


sub initializePackages{
  my ($allPackages) = @_;
  my $l = $instConfig->getPackages();
  my %packHash = packageListToPackageHash($allPackages);
  my %names;
  my @retVal ; 
  foreach (@$l) {
    my $aPack = $packHash{$_};
    unless($aPack){
      warn "Package $_ not found in archive\n";
      next;
    }
    my @deps = GetDepPacksRecursive($aPack,$allPackages);
    foreach (@deps) {
      my $aName = $_->{'PACKAGE_NAME'};
      unless (exists $names{$aName}) {
        push @retVal, ($_);
        $names{$aName}=undef;
      }
    }
    unless (exists $names{$_}) {
      push @retVal, ($aPack);
      $names{$_}=undef;
    }
  }
  return @retVal;
}









sub GetDepPacksRecursive{
  my ($pack,$packages,$names) =@_;
  my @retVal ;
  $names = {} unless $names;
  my @deps = GetDepPacks($pack,$packages);
  foreach (@deps) {
    push @retVal, GetDepPacksRecursive($_,$packages,$names);
  }
  foreach (@deps) {
    my $packName = $_->{'PACKAGE_NAME'};
    unless (exists($names->{$packName})) {
      push @retVal,($_);
      $names->{$packName}=undef;
    }
  }
  return @retVal;
}

sub GetDepPacks{
  my ($pack,$packages) =@_;
  my %packHash = packageListToPackageHash($packages);
  my @retVal;
  my $require =  $pack->{'REQUIRE'};
  my @packNames = parseDepPackNames($require);
  foreach (@packNames) {
    push @retVal,$packHash{$_};
  }
  return @retVal;
}



sub packageListToPackageHash{
  my $listRef = shift;
  my %retVal;
  $listRef or return %retVal;
  foreach (@$listRef) {
    $retVal{$_->{"PACKAGE_NAME"}}=$_;
  }
  return  %retVal;
}
#Does package arg1 depend on package with name arg2 in packagelist arg3
sub dependsOnRecursive{
  my ($pack1,$aName,$packListHash) = @_;
  my $require =  $pack1->{'REQUIRE'};
  my $pack1Name = $pack1->{'PACKAGE_NAME'};
  my @packNames = parseDepPackNames($require);
  my $retVal = 0;
  foreach (@packNames) {
    my $aPackName = $_;
    if ( $aPackName eq $aName) {
      $retVal = 1;
      last;
    }
  }
  if (@packNames) {
    unless($retVal){
      foreach (@packNames) {
        my $newPack = $packListHash->{$_};
        if (dependsOnRecursive($newPack,$aName,$packListHash)) {
          $retVal=1;
          last;
        }
      }
    }
  }
  return $retVal;
}                               #dependsOnRecursive

#Does package arg1 depend on package arg2 in packagelist arg3
sub dependsOn{
  my ($pack1,$pack2,$packListHash) = @_;
  return dependsOnRecursive($pack1,$pack2->{'PACKAGE_NAME'},$packListHash);
}                               #dependsOn

sub comparePacks{
  my ($pack1, $pack2,$packListHash,$reverse) = @_;
  my $retVal = 0;
  my $pack1Name = $pack1->{'PACKAGE_NAME'};
  my $pack2Name = $pack2->{'PACKAGE_NAME'};
  if (dependsOn($pack1,$pack2,$packListHash)) {
    $retVal = $reverse ? -1 : 1;
  } elsif (dependsOn($pack2,$pack1,$packListHash)) {
    $retVal = $reverse ? 1 : -1;
  } else {
  }
  return $retVal;
}
sub sortPacks{
  my $p = shift;
  my $reverse = shift;
  my %packHash = packageListToPackageHash($p);
  my @retVal = sort {comparePacks($a,$b,\%packHash,$reverse);} @$p;
  return @retVal;
}




1;


