package  SAPDB::Install::Templates2::test::InstTemplateTest;

use SAPDB::Install::Templates2::InstTemplate;
use SAPDB::Install::Templates2::Debug;
use SAPDB::Install::Templates2::Util;
use strict;


my $desktopTemplate; 
my $customTemplate; 
my $instTemplate;



our @ISA;
if($SCRIPTING){
  require Exporter;
  push @ISA,qw(Exporter);
  require Data::Dumper;
  import Data::Dumper;
}
else{
  require SDB::Common::Exporter;
  push @ISA,qw(SDB::Common::Exporter);
}
our @EXPORT = qw (allTests);

my $templateFile;


sub setUp{
  my $retVal;
  $desktopTemplate = SAPDB::Install::Templates2::InstTemplate::new(undef,'desktop') or die ;
  $customTemplate = SAPDB::Install::Templates2::InstTemplate::new(undef,'custom') or die;
  $instTemplate = SAPDB::Install::Templates2::InstTemplate::new($templateFile)or die;
  $retVal and Logit($retVal);
}


sub tearDown{
  undef $desktopTemplate;
  undef $customTemplate;
}

sub testGetSystemDrive{
  my $s = $instTemplate->getSystemDrive();
#  print $s,"\n";
  return 0;
}


sub testSetAsyncCallback{
  my $test;
  $customTemplate->setAsyncCallback(sub {$test=1;});
  my $db = $customTemplate->getDBInstance();
  $db->async_status('sfsdfdfs');
  $test or return "Async callback not called!";
  return 0;
}

sub testSetStatusListCallback{
  my $test;
  $customTemplate->setStatusListCallback(sub{$test=1;});
  my $db = $customTemplate->getDBInstance();
  $db->status_callback('sfsdfdfs');
  $test or return "Status callback not called!";
  return 0;
}

sub testGetXmlData(){
  my @d = $desktopTemplate->GetXmlData();
  return 0;
}


sub testInstTemplate{
  my $indepData = $instTemplate->getIndepData();
  my $applicationDataPath = $instTemplate->getApplicationDataPath();
  my $applicationDataPath2 = $instTemplate->Get('applicationDataPath');
  $applicationDataPath eq $applicationDataPath2 or return "Check of application data path failed";
  my $dbVolumes = $instTemplate->getDBVolumes();
  return 0;
}
sub testGetTemplateId{
  my $id1= $desktopTemplate->getTemplateID();
  my $id2= $customTemplate->getTemplateID();
  'desktop' eq $id1 or return "Wrong template id '$id1'";
  'custom' eq $id2 or return "Wrong template id '$id2'";
  return 0;
}

sub testCreateTemplateOverView{
  my $ov = $desktopTemplate->createTemplateOverView();
  @$ov eq 3 or return "Wrong number of frames in desktop overview"  ;
  $ov->[0]->{'Data'}->{'string:MONA (Password RED)'} or return "MONA is missing";
  my $db = $desktopTemplate->getDBInstance();
  $db->setLoadSampleSchema(0);
  $ov = $desktopTemplate->createTemplateOverView();
  $ov->[0]->{'Data'}->{'string:MONA (Password RED)'} and return "MONA should not be there";

  $ov = $customTemplate->createTemplateOverView();
  @$ov eq 3 or return "Wrong number of frames in custom overview"  ;
  return 0;
}


sub testCreateFrame{
  my $f =   $desktopTemplate->createFrame('VolumesMinimalUx');
  TestAFrame($f,'VolumesMinimalUx',qw (db:indepProg db:volstructSimple));
}

sub testNextFrame(){
  my ($nextFrame,$frameId);
  my $retVal = 0;
  my @idsDesktop = ( 'DBNamePasswdDesktop', $^O =~ /mswin/i ? 'VolumesMinimal' : 'VolumesMinimalUx' );
  my @idsCustom = qw ( DBNamePasswdSimple
                       VolumesAdvanced
                       SizeMemoryAdvanced
                       GlobalDBSettings
               );
  while($nextFrame = $desktopTemplate->nextFrame($nextFrame)){
    $retVal = TestAFrame($nextFrame,shift @idsDesktop) and return $retVal;
  }
  while($nextFrame = $customTemplate->nextFrame($nextFrame)){
    $retVal = TestAFrame($nextFrame,shift @idsCustom) and return $retVal;
  }

  my $afr = $desktopTemplate->nextFrame();
  return 0;
}

sub testDefaultIndepProg{
  my $def  = $desktopTemplate->defaultIndepProg();
  return 0;
}

sub testGetOwnerName{
  my $r =$desktopTemplate->getOwnerName();
#  print "Owner is $r\n";
  $r or return "No owner";
  return 0;
}

my %frameData = (
                 'DBNamePasswdDesktop' =>
                  {
                   'assert_params' => [qw (db:dBName db:dBSize db:loadSampleSchema db:dBUser) ]
                  },

                 'VolumesMinimalUx' =>
                  {
                   'assert_params' => [qw (indepProg db:volStructSimple) ] 
                  },
                 
                 'VolumesMinimal' =>
                  {
                   'assert_params' => [qw (sWDrive db:volStructSimple) ]
                  },

                 'DBNamePasswdSimple' =>
                  {
                   'assert_params' => [qw (db:loadSampleSchema db:adminUserStruct db:dBName softwarePath ) ]
                  },

                 'VolumesAdvanced' =>
                  {
                   'assert_params' => [qw (db:volstructLog db:volstructData ) ]
                  },

                 'SizeMemoryAdvanced' =>
                  {
                   'assert_params' => [qw (db:dBSize db:dBMemory db:cPUCount db:parallelConnections ) ]
                  },

                 'GlobalDBSettings' =>
                  {
                   'assert_params' => [qw (db:autoRestart db:autoExtend db:loggingStruct db:autoUpdateStatistics ) ]
                  },
                 
                 'GeneralPropertiesCustom' =>
                  {
                   'assert_params' => [qw ( db:dBName db:dBM db:dBA ) ]
                  },
                );

sub TestAFrame{
  my ($frame,$assertId) = @_;
  my $frameId = $frame->{'FrameId'};
  $frame or return "Undefined frame";
  $frameId or return "No frame id in $frame";
  $frameId eq $assertId or return "Wrong frame id: '$frameId' (Expected '$assertId')";
  my $assertParams;
  my $params = $frame->{'Data'};
  if($assertParams = $frameData{$frameId}->{'assert_params'}){
    foreach my $key(@$assertParams){
      my $param = $params->{$key}; 
      $params->{$key} or return "Missing Parameter: '$key'";
      my $getter = $param->{'Getter'};
      $getter or return "Missing Getter: $key";
      my $val = &$getter();
      my $setter = $param->{'Setter'};
      &$setter($val);
    }
  }
  return 0;
}


sub testInitializeOwnerName{
  my $in = $desktopTemplate->initializeOwnerName();
  my $ownerName = $desktopTemplate->getOwnerName();
  $in eq $ownerName or return "Error in ownerName";
  return 0;
}

sub testInitializeOwnerGroup{
  my $in = $desktopTemplate->initializeOwnerGroup();
  my $in2 = $desktopTemplate->getOwnerGroup();
  $in eq $in2 or return "Error in ownerGroup";
  return 0;
}
sub testDescribe{
  my $txt = $desktopTemplate->Describe();
#  print $txt;
  return 0;
}


sub testGetConfigParameters{
  my ($txt,$retVal) ;
#  $retVal = $desktopTemplate->GetConfigParameters(\$txt) and return $retVal;
#  print $txt;
  return 0;
}


sub testCreateTemplate{
  my ($txt,$retVal) ;
  $retVal = $desktopTemplate->GetConfigParameters(\$txt) and return $retVal;
  my $at = createTemplateObject($txt);
  $at->setSWDrive('F');
  $at->getDBInstance()->setLoadSampleSchema('FALSE');
  $retVal = $at->GetConfigParameters(\$txt) and return $retVal;
  return 0;
}


sub testValidate{
  my $retVal;
  $desktopTemplate->getDBInstance()->setDBName('MAXDB1isfsdf+sdf#');
  $retVal = $desktopTemplate->Validate();
  $retVal or return "Validation failed in testValidate";
  return 0;
}



sub allTests{
  $templateFile = shift;
  unitTest(__PACKAGE__);
}
1;


# Local Variables: ***
# eval: (setq my-sdbrun (format "%s\\sys\\src\\install\\perl\\sdbrun" "c:\\SapDevelop\\aw")) ***
# eval: (setq my-script (format "%s\\sys\\wrk\\script" "c:\\SapDevelop\\aw")) ***
# eval: (setq my-src (format "%s\\sys\\src\\install\\perl" (sdb-get-current-own))) ***
# eval: (setq perl-dbg-flags (format "-w -I%s -I%s\\readkey -I%s -I%s" my-sdbrun my-sdbrun my-script my-src )) ***
# End: ***


