#!/usr/bin/perl
#
# $Header: //sapdb/V76/c_00/b_32/sys/src/install/perl/SAPDB/Install/User/SyncMan/Acc.pm#1 $
# $DateTime: 2006/07/06 16:50:32 $
# $Change: 130190 $
#
# Desc: 

package SAPDB::Install::User::SyncMan::Acc;

use SAPDB::Install::User::SyncMan::Config::Java;
use SAPDB::Install::Registry;

our $synmanacc_options = {
		'DB_HOST' => '-database_host',
		'DB_NAME' => '-database_name',
		'USER'=> '-user',
		'PASSWORD' => '-password',
		'ACCESS_KEY' => '-access_key',
		'DB_ENCRYPTION' => '-database_encryption',
		'MSG_ENCRYPTION' => '-msgserver_encryption',
		'MSG_SERVER' => '-msgserver_host',
		'JNDI_SERVER' => '-jndiserver_host',
		'MSG_SERVER_PORT' => '-msgserver_port',
        'JNDI_SERVER_PORT' => '-jndiserver_port',
		'MSG_SERVER_ADMIN_PORT' => '-msgserver_admin_port',
		'MSG_SERVER_PASSWORD' => '-msgserver_password',
		'COMMAND' => '-command',
		'SYNC_SERVICE_ADMIN_PORT' => '-sync_admin_port',
        'SECURITY_PROVIDER' => '-security_provider'
};

our $syncmanacc_list_mappings = {
		'Database Host' => 'DB_HOST',
		'Database Name' => 'DB_NAME',
		'Database User' => 'USER',
		'Message Server Host' => 'MSG_SERVER',
		'Message Server Port' => 'MSG_SERVER_PORT',
		'Jndi Server Port' => 'JNDI_SERVER_PORT',
		'Message Server Admin Port' => 'MSG_SERVER_ADMIN_PORT',
		'Sync Service Admin Port' => 'SYNC_SERVICE_ADMIN_PORT'
};


sub new{
	my $members = {
		'accesskey' => undef,
		'java_cmd' => undef,
	};
	my $self = bless($members,shift);
	
	my $java_config = SAPDB::Install::User::SyncMan::Config::Java->new();
	$java_config->load();
	my $java;
	if(exists $java_config->{'configuration'}->{'JAVAPATH'} and
		$java_config->{'configuration'}->{'JAVAPATH'} =~ /\S/){
		$java = $java_config->{'configuration'}->{'JAVAPATH'};
		$java .= ($^O =~ /mswin/i ? '\java.exe' : '/java');
	}
	else{
		return undef;
	}
	my ($indep_data,$indep_prog) = readIndepPath();
	
	my $syncinst = "$indep_prog/app/syncman";
	
	my $path_seperator = ':';
	my $quotes = ''; 
	 
	if ($^O =~ /mswin/i){
		$java = "\"$java\"";
		$path_seperator = ';';
		$quotes = '"';
		
	}
	$self->{'accesskey'} = shift;
	my $java_command = "-Djava.library.path=$quotes$syncinst/lib$quotes ".
					   "-classpath $quotes$syncinst/lib$path_seperator".
					   "$syncinst/lib/syncmanacc.jar$quotes ".	
					   "com.sap.sdb.syncMan.acc.SyncManACC";

	if($^O =~ /mswin/i){
		$java_command =~ s/\//\\/g;
	}
	
	$self->{'java_cmd'} = $java.' '.$java_command;

	
	return $self	
}

sub set{
	my ($self,$config) = @_;
	my $option_list = '-c set';
	$option_list .= ' -A '.$self->{'accesskey'};
	foreach my $param (keys(%$config)){
		my $value = $config->{$param};
		next unless $value =~ /\S/;
		$value = '"'.$value.'"' if $value =~ /\s/;
		if(exists $synmanacc_options->{$param}){
			$option_list .= ' '.$synmanacc_options->{$param}.' '.$value;
		}
		else{
			print STDERR ">>> Sorry, param $param has no defined command line switch\n";
		}
	}
	my $cmd = $self->{'java_cmd'}.' '.$option_list;
	my $out = `$cmd 2>&1`;
	if($? != 0){
		print STDERR "syncacc returned $?: $out\n";
		return -1;
	} 
	return 0;
}

sub list{
	my ($self,$config) = @_;
	my $option_list = '-c list';
	$option_list .= ' -A '.$self->{'accesskey'};
	my $cmd = $self->{'java_cmd'}.' '.$option_list;
	my $out = `$cmd 2>&1`;
	if($? != 0){
		$self->{'last_error'} = "syncmanacc list returned $?: $out\n";
		return -1;
	} 
	foreach my $line (split("\n",$out)){
		my ($key,$value) = ($line =~ /^(.*):\s+(.*)/);
		next if $value !~ /\S/;
		next if $key =~ /password/i and $value =~ /^\*+$/; 
		
		if(exists $syncmanacc_list_mappings->{$key}){
			$DEBUG && print ">>>> SET ".$syncmanacc_list_mappings->{$key}." to $value\n";
			$config->{$syncmanacc_list_mappings->{$key}} = $value; 
		}	
	}	
	return 0;
}

sub exists{
	my ($self) = @_;
	my $option_list = '-c exists';
	$option_list .= ' -A '.$self->{'accesskey'};
	my $cmd = $self->{'java_cmd'}.' '.$option_list;
	my $out = `$cmd 2>&1`;
	return $?;
}





1;