#!/usr/bin/perl
#
# $Header: //sapdb/V76/c_00/b_32/sys/src/install/perl/SAPDB/Install/Window/DBLogin.pm#1 $
# $DateTime: 2006/07/06 16:50:32 $
# $Change: 130190 $
#
# Desc: 
#

package SAPDB::Install::Window::DBLogin;
use Wx qw (	wxHORIZONTAL wxVERTICAL wxTE_PASSWORD
			wxALIGN_CENTER_VERTICAL wxLEFT
			wxALIGN_RIGHT wxTOP wxRIGHT wxBOTTOM
			wxEXPAND);

use base Wx::Panel;
use SAPDB::Install::Registry;
use SAPDB::Install::Tools;
use SAPDB::Install::Resources;
use SAPDB::Install::System;


our $img_keys = SAPDB::Install::Resources->new('RESOURCES.TGZ')->
						getImages('icon/keys.png')->{'icon/keys.png'};

our ($indep_data,$indep_prog) = readIndepPath();

#
#	$my_args = {
#				'dbname' => <database name>,
#				'log' => <a object of type SAPDB::Install::Log or with similar interface>,
#			};
#



sub new{
	my ($class,$my_args,@args) = @_;
	my $self = $class->SUPER::new(@args);
	$self->{'dbname'} = $my_args->{'dbname'};
	$self->{'log'} = $my_args->{'log'};
	my $bitmap = Wx::StaticBitmap->new($self,-1,Wx::Bitmap->new($img_keys));
	my $topsizer = Wx::BoxSizer->new(wxVERTICAL);
	my $static_box = Wx::StaticBox->new($self,-1,'Database System Administrator Login');
	my $boxsizer = Wx::StaticBoxSizer->new($static_box,wxHORIZONTAL);
	my $edit_sizer = Wx::GridSizer->new(2,2);
	$edit_sizer->SetVGap(10);
	$edit_sizer->SetHGap(10);
	my $label_dbm = Wx::StaticText->new($self,-1,'User Name',[0,0],[-1,-1]);
	my $label_dbm_passwd = Wx::StaticText->new($self,-1,'Password',[0,0],[-1,-1]);
	$self->{'edit_dbm'} = Wx::TextCtrl->new($self,-1,'',[0,0],[70,-1]);
	$self->{'edit_dbm_pass'} = Wx::TextCtrl->new($self,-1,'',[0,0],[70,-1],wxTE_PASSWORD);
	$edit_sizer->Add($label_dbm,0,wxALIGN_CENTER_VERTICAL | wxLEFT , 10);
	$edit_sizer->Add($self->{'edit_dbm'},0,wxALIGN_RIGHT);
	$edit_sizer->Add($label_dbm_passwd,0,wxALIGN_CENTER_VERTICAL | wxLEFT , 10);
	$edit_sizer->Add($self->{'edit_dbm_pass'},0,wxALIGN_RIGHT);
	$edit_sizer->CalcMin;
	$boxsizer->Add($bitmap,0);
	$boxsizer->Add($edit_sizer,0, wxTOP | wxRIGHT | wxBOTTOM , 20);
	$boxsizer->CalcMin;
	$topsizer->Add($boxsizer,1,wxEXPAND | wxALL, 10 );
	$self->Fit;
	$self->SetSizer($topsizer);
	$self->SetAutoLayout(1);
	$self->Layout();
	$self->Fit;
	return $self;
}

sub getstate{
	my ($self) = @_;
	my $prog  = "$indep_prog/bin/x_cons";
	if( $^O =~ /mswin/i){
		$prog .= '.exe';
		$prog =~ s/\//\\/g;
	}
	
	unless(-f $prog){
		return undef;
	}

	my $cmd  = '"'.$prog.'" '.$self->{'dbname'}.' state';
	 
	my $db_state;
	callsystem($cmd,1,sub{
		my ($text) = @_;
		foreach my $line (split("\n",$text)){
			if($line =~ /state/i){
				($db_state) = ($line =~ /\s(\S+)\s*$/);				
				last;
			}
		}
		return 1;	
	});
	unless($db_state =~ /\S/){
		return undef;
	}
	
	return $db_state;		
	
}


sub GetUserName{
	my ($self) = @_;
	return $self->{'edit_dbm'}->GetValue();
}


sub GetPassword{
	my ($self) = @_;
	return $self->{'edit_dbm_pass'}->GetValue();
}



sub connect{
	my ($self) = @_;
	
	if ($self->{'edit_dbm'}->GetValue() !~ /\S/){
		$self->{'last_err'} = 'Unkown user name';
		return undef;	
	}
	
	if ($self->{'edit_dbm_pass'}->GetValue()!~ /\S/){
		$self->{'last_err'} = 'Emtpy password';
		return undef;	
	
	}
	
	#
	# local communication connect
	#
	$self->{'dbm'} = SAPDB::Install::DBMCmd->new('',$self->{'dbname'});

	unless (defined $self->{'dbm'}){
			
			#
			# try tcp connect as fallback
			#
			
			require SAPDB::Install::Service::Commumication;
	
			my $xserver = SAPDB::Install::Service::Commumication->new ();
	
			unless (defined $xserver->start()){
				$self->{'last_err'} = 'TCP connect failed - cannot start x_server: '.
										$xserver->last_error ();
				return undef;
			}
			
			$self->{'dbm'} = SAPDB::Install::DBMCmd->new (	'localhost',
															$self->{'dbname'});
			unless (defined $self->{'dbm'}){
				$self->{'last_err'} = 'Connect failed - cannot connect to '.
										$db.'@localhost';
				return undef;
			}
	}
	
	
	
	my $value = $self->{'edit_dbm'}->GetValue().
				','.$self->{'edit_dbm_pass'}->GetValue();
	
	$self->{'user_passwd'} = $self->{'edit_dbm'}->GetValue().
				','.$self->{'edit_dbm_pass'}->GetValue();
	
	my $rc = $self->{'dbm'}->user_logon($self->{'user_passwd'});		
	
	unless(defined $rc){
		$self->{'last_err'} = 'user logon failed: '."\n".
							  $self->{'dbm'}->lastmsg;
		$self->{'dbm'}->release;
		return undef;
	}	
	$self->{'connected'} = 1;
	
	$self->{'log'}->SetMsg("MSG: connection to database ".$self->{'dbname'}.
							" successfully established\n")
							if defined $self->{'log'};
	
	return $self->{'dbm'}
}

sub release{
	my ($self) = @_;
	if($self->{'connected'}){
		delete $self->{'dbm'};
		delete $self->{'connected'}
	}
	return 1;
}

sub Destroy{
	my ($self) = @_;
	$self->release;	
	if(defined $self->GetSizer){
		while($self->GetSizer->Remove(0)){}
	}
	if(defined $self->{'edit_dbm'}){
		$self->{'edit_dbm'}->Destroy;
		delete  $self->{'edit_dbm'};
	}
	if(defined $self->{'edit_dbm_pass'}){
		$self->{'edit_dbm_pass'}->Destroy;
		delete  $self->{'edit_dbm_pass'};
	}
	$self->SUPER::Destroy;
}


sub DESTROY{
	my ($self) = @_;
	print ">>>>> destroying $self\n"
		if $SAPDB::Install::DEBUG::DESTRUCT;
}



1;
