package SAPDB::Install::Window::LogMngmnt;


use Wx::Event qw(EVT_RADIOBUTTON);
use Wx qw(wxVERTICAL wxALL wxEXPAND
	wxLEFT wxRIGHT wxLEFT wxTOP wxBOTTOM
	wxHORIZONTAL wxALIGN_CENTER_VERTICAL);
use SAPDB::Install::Control::PathCtrl;
use base Wx::Panel;

sub new{
	my $class = shift;
	my $self = $class->SUPER::new(@_);
	$self->SetSizer(Wx::BoxSizer->new(wxVERTICAL));
	$self->{'rb_autologbackup'} = Wx::RadioButton->new($self,-1,'&Automatic log backup');	
	EVT_RADIOBUTTON($self,$self->{'rb_autologbackup'}->GetId,\&OnRBClicked);
	$self->{'rb_manuallogbackup'} = Wx::RadioButton->new($self,-1,'&Manual log backup');
	EVT_RADIOBUTTON($self,$self->{'rb_manuallogbackup'}->GetId,\&OnRBClicked);
	$self->{'rb_nologbackup'} = Wx::RadioButton->new($self,-1,'N&o log backup');
	EVT_RADIOBUTTON($self,$self->{'rb_nologbackup'}->GetId,\&OnRBClicked);
	my $grid_sizer = Wx::FlexGridSizer->new(2,2,5,10);
	$grid_sizer->AddGrowableCol(1);
	$self->{'st_databackup'} = Wx::StaticText->new($self,-1,'Initial data backup',[-1,-1],[-1,-1]);
	$self->{'st_logbackup'} = Wx::StaticText->new($self,-1,'Log backup',[-1,-1],[-1,-1]);
	$self->{'pc_databackup'} = SAPDB::Install::Control::PathCtrl->new($self,-1,[0,0],[-1,-1]);
	$self->{'pc_logbackup'} = SAPDB::Install::Control::PathCtrl->new($self,-1,[0,0],[-1,-1]);	
	$grid_sizer->Add($self->{'st_databackup'},0,wxALIGN_CENTER_VERTICAL);
	$grid_sizer->Add($self->{'pc_databackup'},0,wxEXPAND);
	$grid_sizer->Add($self->{'st_logbackup'},0,wxALIGN_CENTER_VERTICAL);
	$grid_sizer->Add($self->{'pc_logbackup'},0,wxEXPAND);
	$self->GetSizer->Add($self->{'rb_autologbackup'},0,wxEXPAND | (wxALL) & (~wxTOP),10);
	$self->GetSizer->Add($grid_sizer,0,wxEXPAND | wxLEFT,50);
	$self->GetSizer->Add($self->{'rb_manuallogbackup'},0,wxEXPAND | (wxALL) & (~wxTOP),10);
	$self->GetSizer->Add($self->{'rb_nologbackup'},0,wxEXPAND | (wxALL) & (~wxTOP),10);
	$self->SetAutoLayout(1);
	$self->Layout;
	return $self;
}

our $struct_member = 
	{
		'autologbackup' => 'AutoSaveLog',
		'nologbackup' => 'AutoOverWriteLog',
		'databackup' => 'InitialBackupDir',
		'logbackup' => 'LogSaveDir'
	};


sub SetValue{
	my ($self,$struct) = @_;
	$self->{'pc_databackup'}->SetValue($struct->{$struct_member->{'databackup'}});
	$self->{'pc_logbackup'}->SetValue($struct->{$struct_member->{'logbackup'}});
	if($struct->{$struct_member->{'autologbackup'}} xor $struct->{$struct_member->{'nologbackup'}}){
		if($struct->{$struct_member->{'autologbackup'}}){
			$self->{'rb_autologbackup'}->SetValue(1);
			$self->{'pc_databackup'}->Enable(1);
			$self->{'pc_logbackup'}->Enable(1);
		}
		else{
			$self->{'rb_nologbackup'}->SetValue(1);
			$self->{'pc_databackup'}->Enable(0);
			$self->{'pc_logbackup'}->Enable(0);	
		}
	}
	else{
		$self->{'rb_manuallogbackup'}->SetValue(1);
		$self->{'pc_databackup'}->Enable(0);
		$self->{'pc_logbackup'}->Enable(0);		
	}
	$self->Layout;
}

sub OnRBClicked{
	my ($self,$event) = @_;
	if($self->{'rb_autologbackup'}->GetValue){	
		$self->{'pc_databackup'}->Enable(1);
		$self->{'pc_logbackup'}->Enable(1);	
	}
	else{
		$self->{'pc_databackup'}->Enable(0);
		$self->{'pc_logbackup'}->Enable(0);	
	}
	return 1;
}



sub GetValue{
	my ($self) = @_; 
	my %struct;

	$struct{$struct_member->{'autologbackup'}} = 0;
	$struct{$struct_member->{'nologbackup'}} = 0;
	$struct{$struct_member->{'logbackup'}} = $self->{'pc_logbackup'}->GetValue;
	$struct{$struct_member->{'databackup'}} = $self->{'pc_databackup'}->GetValue;
	if($self->{'rb_nologbackup'}->GetValue){
		$struct{$struct_member->{'nologbackup'}} = 1;
	
	}
	elsif($self->{'rb_autologbackup'}->GetValue){
		$struct{$struct_member->{'autologbackup'}} = 1;
	}
	return \%struct; 
}

sub Destroy{
	my ($self) = @_;
	if(defined $self->GetSizer){
		while($self->GetSizer->Remove(0)){}		
	}
	$self->{'rb_autologbackup'}->Destroy if defined $self->{'rb_autologbackup'};
	delete $self->{'rb_autologbackup'};
	$self->{'rb_nologbackup'}->Destroy if defined $self->{'rb_nologbackup'};
	delete $self->{'rb_nologbackup'};
	$self->{'rb_manuallogbackup'}->Destroy if defined $self->{'rb_manuallogbackup'};
	delete $self->{'rb_manuallogbackup'};	
	$self->{'pc_databackup'}->Destroy if defined $self->{'pc_databackup'};
	delete $self->{'pc_databackup'};	
	$self->{'pc_logbackup'}->Destroy if defined $self->{'pc_logbackup'};
	delete $self->{'pc_logbackup'};		
	$self->SUPER::Destroy;
}

sub DESTROY{
	my ($self) = @_;
	print ">>>>> destroying $self\n"
		if $SAPDB::Install::DEBUG::DESTRUCT;
}


1;
