#!/usr/bin/perl
#
# $Header: //sapdb/V76/c_00/b_32/sys/src/install/perl/SAPDB/Install/Wizard/Dialog/ArchiveDir.pm#1 $
# $DateTime: 2006/07/06 16:50:32 $
# $Change: 130190 $
#
# Desc: start SAP DB installation/update/patch/instanceupgrade,
# called by SDBINST/SDBUPD
#
package SAPDB::Install::Wizard::Dialog::ArchiveDir::myPackageList;

use base SAPDB::Install::Window::ItemList;
use Wx qw(wxALIGN_RIGHT);

our @cols = (	
		{
			'key' => 'name',
			'label' => 'Package Name'
		},
		{
			'key' => 'size',
			'label' => 'Size in MB',
			'alignment' => wxALIGN_RIGHT
		},
		{
			'key' => 'version',
			'label' => 'Version'
		},
		{
			'key' => 'archive',
			'label' => 'Archive'
		},
		{
			'key' => 'desc',
			'label' => 'Description'
		}
);

1;

package SAPDB::Install::Wizard::Dialog::ArchiveDir;

use Wx::Event qw (EVT_TEXT_ENTER);
use Wx qw (
	wxICON_ERROR wxCENTRE wxVERTICAL wxDefaultSize 
	wxBOTTOM wxHORIZONTAL wxEXPAND
	wxALL wxLEFT wxTOP wxALIGN_CENTER_VERTICAL
	wxALIGN_RIGHT wxRIGHT wxSYS_SYSTEM_FONT);

use SAPDB::Install::Wizard::Dialog::Base;
use SAPDB::Install::Wizard::Dialog::DBHandling;
use SAPDB::Install::PackageCollector;
use SAPDB::Install::Control::PathCtrl;
use SAPDB::Install::ObjMngr;
	
our @ISA = qw (SAPDB::Install::Wizard::Dialog::Base);



sub new{
	my( $self ) = shift->SUPER::new(@_);
	my $sizer = Wx::BoxSizer->new(wxVERTICAL);
	my $line_sizer = Wx::BoxSizer->new(wxHORIZONTAL);
	my $static_box = Wx::StaticBox->new($self,-1,'Found Software Packages');
	$self->{'sb_sizer'} = Wx::StaticBoxSizer->new($static_box,wxHORIZONTAL);
	$self->{'label'} = Wx::StaticText->new($self,-1,'Software Package Folder');
	$self->{'pathctrl'} = SAPDB::Install::Control::PathCtrl->new($self,-1,[0,0],[20,20]);
	$self->{'pathctrl'}->SetValue($SAPDB::Install::Config{'CallerDir'});
	$self->{'packagelistctrl'} = SAPDB::Install::Wizard::Dialog::ArchiveDir::myPackageList->new($self,-1);
	$line_sizer->Add($self->{'label'},0,wxRIGHT | wxCENTRE,10);
	$line_sizer->Add($self->{'pathctrl'},1);		
	$sizer->Add($line_sizer,0,wxEXPAND | wxALL, 20);
	$self->{'sb_sizer'}->Add($self->{'packagelistctrl'},1, wxEXPAND | wxALL, 5);
	$sizer->Add($self->{'sb_sizer'},1, wxEXPAND | ((wxALL) & (~wxTOP)), 20);
	$self->SetSizerAndFit($sizer);
	$self->SetAutoLayout(1);
	$self->Layout();
	return $self;
}

sub init{
	my ($self,$config) = @_;
	$self->{'config'} = $config;
	unless (defined $config->{'container'}){
		$config->{'container'} = SAPDB::Install::ObjMngr->new();
	}
	
	EVT_TEXT_ENTER($self,$self->{'pathctrl'}->GetId,sub{$self->RefreshList});
	
	$config->{'container'}->Log(SAPDB::Install::Log->new());
	$config->{'collector'} = SAPDB::Install::PackageCollector->new();
	$config->{'collector'}->log($config->{'container'}->Log);
	$self->RefreshList();
	$self->{'initialized'} = 1;
	$self->Layout;
	return {}; 
}

sub RefreshList{
	my ($self) = @_;
	$self->GetWizard->setBusy(1);
	$self->{'config'}->{'collector'}->package_index({}); # reset
	$self->{'config'}->{'collector'}->{'archive_dir'} = $self->{'pathctrl'}->GetValue();
	$self->{'config'}->{'collector'}->GenPackageIndex();
	my @list;
	$self->{'sb_sizer'}->Remove($self->{'packagelistctrl'});
	$self->{'packagelistctrl'}->Destroy;
	$self->{'packagelistctrl'} = SAPDB::Install::Wizard::Dialog::ArchiveDir::myPackageList->new($self,-1);
	$self->{'sb_sizer'}->Add($self->{'packagelistctrl'},1, wxEXPAND | wxALL, 5);
	
	if(defined $self->{'config'}->{'collector'}->package_index){
		foreach my $package (keys(%{$self->{'config'}->{'collector'}->package_index})){
			my $name = $self->{'config'}->{'collector'}->package_index->{$package}->{'DISPLAY_NAME'};
			unless(defined $name){
				$name = $package;
			}
			my $size = $self->{'config'}->{'collector'}->package_index->{$package}->{'SIZE'};
			if(defined $size){
				my $size_kb = $size / 1024;
				my $size_mb = int($size_kb / 1024);
				my $size_mb_rest = int(($size_kb % 1024) * 1000 / 1024);
				$size_mb_rest = '0' x  (3 - length($size_mb_rest)).$size_mb_rest;
				$size = $size_mb.'.'.$size_mb_rest.'  ';
			}
			else{
				$size = '<unknown>';
			}
			my $version = $self->{'config'}->{'collector'}->package_index->{$package}->{'BUILD_STRING'};
			unless(defined $version){
				$version = $self->{'config'}->{'collector'}->package_index->{$package}->{'SOFTWARE_VERSION'};
			}
			my $desc = $self->{'config'}->{'collector'}->package_index->{$package}->{'DESC'};
			unless(defined $desc){
				$desc = '<unknown>';
			}
			
			my $archive = $self->{'config'}->{'collector'}->package_index->{$package}->{'ARCHIVE_NAME'};
			$archive =~ s/.*[\\\/]//;
			
			push @list,{'name' => $name,
						'size' =>  $size,
						'version' => $version,
						'desc' => $desc,
						'archive' => $archive,
						};
		} 
		$self->GetWizard->{'nbutton'}->Enable(1);
	}
	
	@list = sort {$a->{'name'} cmp $b->{'name'}} @list;
	
	$self->{'packagelistctrl'}->create(\@list);
	$self->Layout();

	if($#list == -1){
		$self->GetWizard()->{'nbutton'}->Enable(0);	
		$self->GetWizard()->{'nbutton_disabled'} = 1;
	}
	else{
		$self->GetWizard()->{'nbutton_disabled'} = 0;
	}
	$self->GetWizard->setBusy(0);
	return 1;
}


sub Process{
	my ($self,$config) = @_;
	return {};
}

sub Destroy{
	my ($self) = @_;
	if(defined $self->GetSizer){
		while($self->GetSizer->Remove(0)){}
	}
	if(defined $self->{'packagelistctrl'}){
		$self->{'packagelistctrl'}->Destroy();
		delete $self->{'packagelistctrl'};
	}
	if(defined $self->{'pathctrl'}){
		$self->{'pathctrl'}->Destroy;
		delete $self->{'pathctrl'};
	}
	$self->SUPER::Destroy;
}

sub DESTROY{
	my ($self) = @_;
	print ">>>>> destroying $self\n"
		if $SAPDB::Install::DEBUG::DESTRUCT;
}


1;
