#!/usr/bin/perl
#
# $Header: //sapdb/V76/c_00/b_32/sys/src/install/perl/SAPDB/Install/Wizard/Dialog/Parameter.pm#1 $
# $DateTime: 2006/07/06 16:50:32 $
# $Change: 130190 $
#
# Desc: start SAP DB installation/update/patch/instanceupgrade, called by SDBINST/SDBUPD

package SAPDB::Install::Wizard::Dialog::Parameter;

use Wx::Event qw (EVT_TEXT EVT_CHECKBOX EVT_GRID_CELL_LEFT_CLICK);
use Wx qw (
	wxSP_ARROW_KEYS wxALIGN_CENTER wxEXPAND wxALL wxBOTTOM wxTOP
	wxVERTICAL wxALIGN_CENTER_VERTICAL wxSL_LABELS wxLEFT wxRIGHT
	wxICON_ERROR wxCENTER wxALIGN_LEFT wxTE_PASSWORD wxSUNKEN_BORDER
	wxDECORATIVE wxNORMAL wxBOLD wxALIGN_TOP wxCB_READONLY);

use base SAPDB::Install::Wizard::Dialog::Base;
use SAPDB::Install::StdIO;
use SAPDB::Install::Control::StaticText;
use SAPDB::Install::Wizard::Dialog::CreateInstance;

our $checkbox_distance = 0;


sub new{
	my($this ) = shift->SUPER::new(@_);
	if($^O !~ /mswin/i){
		$this->{'title'} = SAPDB::Install::Control::StaticText->new($this,-1,[0,0],[-1,-1]);
		$this->{'title'}->{'auto_resize'} = 1;
	}
	else{	
		$this->{'title'} = Wx::StaticText->new($this,-1,'',[0,0],[-1,-1]); 
	}
	#$this->{'title'}->{'font'} = Wx::Font->new(10,wxDECORATIVE, wxNORMAL, wxBOLD, 0);
	return $this;
}



sub init{
	my ($self,$config) = @_;
	if(defined $self->{'frame'}){
		$self->cleanFrame;
	}
	$self->{'frame'} = $config->{'container'}->Template->nextFrame();
	$self->{'topsizer'} = Wx::BoxSizer->new(wxVERTICAL);
	$self->{'topboxsizer'} = Wx::BoxSizer->new(wxVERTICAL);
	#$self->{'paramsizer'} = Wx::GridSizer->new(2,2);
	$self->{'topsizer'}->Add(Wx::BoxSizer->new(wxVERTICAL),0,wxALL,2); # dummy
	$self->{'topsizer'}->Add($self->{'title'},0,wxALIGN_CENTER_VERTICAL | wxLEFT,20);
	#$self->{'topsizer'}->Add(Wx::BoxSizer->new(wxVERTICAL),0,wxALL,2);# dummy
	#$self->{'topboxsizer'}->Add($self->{'paramsizer'},0,wxEXPAND | wxALL, 10) ;
	$self->{'topsizer'}->Add($self->{'topboxsizer'},1,wxEXPAND | (wxALL) & (~wxTOP),10);	
	$self->SetSizer($self->{'topsizer'});
	$self->SetAutoLayout(1);
	$self->initFrame($config);
	$self->Layout();
	return {'test' => 'test'};
}


sub updateFrameValues{
	my ($self) = @_;
	foreach my $param (@{$self->{'frame'}->{'params'}}){
		my $value = &{$self->{'frame'}->{'Data'}->{$param}->{'Getter'}}();
		my $type = $self->{'frame'}->{'Data'}->{$param}->{'ControlType'}->{'id'};
		if($self->{'frame'}->{'Data'}->{$param}->{'ControlType'}->{'unit'} eq 'pages' ||
		   $type eq 'pagesType'){
			$value = int($value * 1024 / 8);
		}
		my $cur_value = $self->{$self->{'frame'}->{'FrameId'}.$param.'_ctrl'}->GetValue;
		unless($value eq $cur_value){
			$self->{$self->{'frame'}->{'FrameId'}.$param.'_ctrl'}->SetValue($value);
		}
	}
}


sub UpdateParam{
	my ($self,$param,$param_desc) = @_;
	my $type = $param_desc->{'ControlType'}->{'id'};
	my $max = $param_desc->{'Maximum'};
	my $min = $param_desc->{'Minimum'};
	my $default = $param_desc->{'Value'};
	if($self->{'frame'}->{'Data'}->{$param}->{'ControlType'}->{'unit'} eq 'pages' ||
		   $type eq 'pagesType'){
			$max = int(($max * 8) / 1024);
			$min = int(($min * 8) / 1024);
			$default = int(($default * 8) / 1024);
	}
	if($type eq 'booleanType'){
			$self->{$self->{'frame'}->{'FrameId'}.$param.'_ctrl'}->SetValue($default);
		}
		elsif($type eq 'stringType'){
			$self->{$self->{'frame'}->{'FrameId'}.$param.'_ctrl'}->SetValue($default);
		}
		elsif($type eq 'pagesType'){
			$self->{$self->{'frame'}->{'FrameId'}.$param.'_ctrl'}->SetValue($default);
		}
		elsif($type eq 'passwordType'){
			$self->{$self->{'frame'}->{'FrameId'}.$param.'_ctrl'}->SetValue($default);
		}
		elsif($type eq 'sliderType'){
				$self->{$self->{'frame'}->{'FrameId'}.$param.'_ctrl'}->SetValue($default);
				$self->{$self->{'frame'}->{'FrameId'}.$param.'_ctrl'}->SetRange($min,$max);
		}
		elsif($type eq 'spinnerType'){
				$self->{$self->{'frame'}->{'FrameId'}.$param.'_ctrl'}->SetValue($default);
				$self->{$self->{'frame'}->{'FrameId'}.$param.'_ctrl'}->SetRange($min,$max);
		}
		elsif($type eq 'driveType'){
			$self->{$self->{'frame'}->{'FrameId'}.$param.'_ctrl'}->SetValue($default);
		}
		else{
			$self->{$self->{'frame'}->{'FrameId'}.$param.'_ctrl'}->SetValue($default);
		}
		$self->Layout;
		#$self->GetWizard->LayoutDlg;
}





sub ParamChanged{
	my ($self,$param,$param_desc,$config,$value) = @_;
	if($param_desc->{'unit'} eq 'pages' ||
			   $param_desc->{'ControlType'}->{'id'} eq 'pagesType'){
		$value = int($value * 1024 / 8);
	}
	&{$param_desc->{'Setter'}}($value);								
	
	my $dependencies = $self->{'frame'}->{'Data'}->{$param}->
			{'ControlType'}->{'dependencies'};
	foreach my $dependency (@$dependencies){
          #Get the depending parameter from the frame:
          my $depParam =  $self->{'frame'}->{'Data'}->{$dependency};
          #If the dependending parameter is NOT in the frame, do nothing:
          $depParam or next;
          my $aDesc = $depParam->{'ControlType'};
          my $paramDesc = $config->{'container'}->Template->createTemplateParameter($dependency,$aDesc);
          $self->UpdateParam($dependency,$paramDesc);
	}
}






sub initFrame{
	my ($self,$config) = @_;
	$self->{'title'}->SetLabel($self->{'frame'}->{'Title'});
	$self->{'topsizer'}->SetItemMinSize($self->{'title'},$self->{'title'}->GetSizeWH);
	$self->{'frame'}->{'params'} = [ sort { $self->{'frame'}->{'Data'}->{$a}->
									{'ControlType'}->{'priority'} <=> $self->
									{'frame'}->{'Data'}->{$b}->{'ControlType'}->{'priority'}
									 } keys(%{$self->{'frame'}->{'Data'}})];
	
	my $flat_params = 0;
	foreach my $param (@{$self->{'frame'}->{'params'}}){
		$flat_params++ if($self->{'frame'}->{'Data'}->
						{$param}->{'Type'} !~ /COMPOUND$/);
	}
	
	if($flat_params > 0){
		$self->{'paramsizer'} = Wx::FlexGridSizer->new($#{@{$self->{'frame'}->{'params'}}} + 1,2,15,15);
		$self->{'paramsizer'}->AddGrowableCol(1);
	}
	else{
		if(defined $self->{'paramsizer'}){
			$self->{'topboxsizer'}->Remove($self->{'paramsizer'});
			delete $self->{'paramsizer'};
		}
	}
	foreach my $param (@{$self->{'frame'}->{'params'}}){
		my $type = $self->{'frame'}->{'Data'}->{$param}->{'ControlType'}->{'id'};
		my $max = $self->{'frame'}->{'Data'}->{$param}->{'Maximum'};
		my $min = $self->{'frame'}->{'Data'}->{$param}->{'Minimum'};
		my $default = $self->{'frame'}->{'Data'}->{$param}->{'Value'};
		my $unit = '';
		if($self->{'frame'}->{'Data'}->{$param}->{'Type'} !~ /COMPOUND$/){			
			if($self->{'frame'}->{'Data'}->{$param}->{'ControlType'}->{'unit'} eq 'pages' ||
			 $type eq 'pagesType'){
				$unit = '(MB)';
				$max = int(($max * 8) / 1024); # pages -> MegaBytes
				$min = int(($min * 8) / 1024); # pages -> MegaBytes
				$default = int(($default * 8) / 1024); # pages -> MegaBytes
			}
			$self->{$self->{'frame'}->{'FrameId'}.$param.'_text'} = Wx::StaticText->new($self,-1,
										$type eq 'booleanType' ? '' : 
										($self->{'frame'}->{'Data'}->{$param}->{'Label'}.' '.$unit),
										[0,0],[-1,-1]);
			$self->{'paramsizer'}->Add($self->{$self->{'frame'}->{'FrameId'}.$param.'_text'},0,wxALIGN_CENTER_VERTICAL) unless($type eq 'booleanType');
			if($type eq 'booleanType'){
				$self->{$self->{'frame'}->{'FrameId'}.$param.'_ctrl'} = Wx::CheckBox->new($self,-1,$self->{'frame'}->{'Data'}->{$param}->{'Label'},[0,0],[-1,-1]);
				$self->{$self->{'frame'}->{'FrameId'}.$param.'_ctrl'}->SetValue($default);
				EVT_CHECKBOX($self,$self->{$self->{'frame'}->{'FrameId'}.$param.'_ctrl'}->GetId,
							sub {
								my ($self,$evt) = @_;
								my $value = $self->{$self->{'frame'}->{'FrameId'}.$param.'_ctrl'}->GetValue;
								$self->ParamChanged($param,$self->{'frame'}->
													{'Data'}->{$param},$config,$value);
							}
				);	
			}
			elsif($type eq 'stringType'){
				$self->{$self->{'frame'}->{'FrameId'}.$param.'_ctrl'} = Wx::TextCtrl->new($self,-1,$default,[0,0],[-1,-1]);
				EVT_TEXT($self,$self->{$self->{'frame'}->{'FrameId'}.$param.'_ctrl'}->GetId,
							sub {
								my ($self,$evt) = @_;
								my $value = $self->{$self->{'frame'}->{'FrameId'}.$param.'_ctrl'}->GetValue;
								$self->ParamChanged($param,$self->{'frame'}->{'Data'}->{$param},$config,$value);
							}
				);	
			}
			elsif($type eq 'pagesType'){
				$self->{$self->{'frame'}->{'FrameId'}.$param.'_ctrl'} = Wx::TextCtrl->new($self,-1,$default,[0,0],[-1,-1]);
				EVT_TEXT($self,$self->{$self->{'frame'}->{'FrameId'}.$param.'_ctrl'}->GetId,
							sub {
								my ($self,$evt) = @_;
								if(exists $self->{'self_changed'}){
									delete $self->{'self_changed'};
									return;
								}
								my $insert_point = $self->{$self->{'frame'}->{'FrameId'}.$param.'_ctrl'}->GetInsertionPoint; 
								my $value = $self->{$self->{'frame'}->{'FrameId'}.$param.'_ctrl'}->GetValue;
								my $count = ($value =~ s/\D//g);
								if($count > 0){
									$self->{'self_changed'} = undef;
									$self->{$self->{'frame'}->{'FrameId'}.$param.'_ctrl'}->SetValue($value);
									$self->{$self->{'frame'}->{'FrameId'}.$param.'_ctrl'}->SetInsertionPoint($insert_point - $count);
								}
								else{
									$self->ParamChanged($param,$self->{'frame'}->{'Data'}->{$param},$config,$value);
								}
							}
				);
			}
			elsif($type eq 'passwordType'){
				$self->{$self->{'frame'}->{'FrameId'}.$param.'_ctrl'} = Wx::TextCtrl->new($self,-1,$default,[0,0],[-1,-1],wxTE_PASSWORD);
			}
			elsif($type eq 'sliderType'){
					$self->{$self->{'frame'}->{'FrameId'}.$param.'_ctrl'} = Wx::Slider->new( $self, -1, $default, $min, $max,
			                               [0,0], [-1, -1],wxSL_LABELS);
			}
			elsif($type eq 'spinnerType'){
				$self->{$self->{'frame'}->{'FrameId'}.$param.'_ctrl'} = Wx::SpinCtrl->new( $self, -1, $default,[0,0],[-1,-1],
												wxSP_ARROW_KEYS, $min,$max);
			}
			elsif($type eq 'driveType'){
				require SAPDB::Install::Control::DriveSelector;
				$self->{$self->{'frame'}->{'FrameId'}.$param.'_ctrl'} = SAPDB::Install::Control::DriveSelector->new($self,-1);
                                $default =~ /\s*(\w)\s*/ and $default = $1 . ':';
				$self->{$self->{'frame'}->{'FrameId'}.$param.'_ctrl'}->SetValue($default);
				EVT_TEXT($self,$self->{$self->{'frame'}->{'FrameId'}.$param.'_ctrl'}->GetId,
							sub {
								my ($self,$evt) = @_;
								my $value = $self->{$self->{'frame'}->{'FrameId'}.$param.'_ctrl'}->GetValue;
								$self->ParamChanged($param,$self->{'frame'}->{'Data'}->{$param},$config,$value);
							}
				);	
			}
			elsif($type eq 'comboBoxType'){
				$self->{$self->{'frame'}->{'FrameId'}.$param.'_ctrl'} = Wx::ComboBox->new($self,-1,$default,[0,0],[-1,-1],
										$self->{'frame'}->{'Data'}->{$param}->{'ControlType'}->{'list'},wxCB_READONLY);
				EVT_TEXT($self,$self->{$self->{'frame'}->{'FrameId'}.$param.'_ctrl'}->GetId,
							sub {
								my ($self,$evt) = @_;
								my $value = $self->{$self->{'frame'}->{'FrameId'}.$param.'_ctrl'}->GetValue;
								$self->ParamChanged($param,$self->{'frame'}->{'Data'}->{$param},$config,$value);
							}
				);	
			}
			elsif($type eq 'pathType'){
				require SAPDB::Install::Control::PathCtrl;
				$self->{$self->{'frame'}->{'FrameId'}.$param.'_ctrl'} = SAPDB::Install::Control::PathCtrl->new(
										$self,-1,[0,0],[-1,-1]);
				$self->{$self->{'frame'}->{'FrameId'}.$param.'_ctrl'}->SetValue($default);
			}
			else{
				$self->{$self->{'frame'}->{'FrameId'}.$param.'_ctrl'} = Wx::TextCtrl->new(
										$self,-1,$default,[0,0],[-1,-1]);
			}
			if($self->{'frame'}->{'focus'} eq $param){
				$self->{$self->{'frame'}->{'FrameId'}.$param.'_ctrl'}->SetFocus;
				if(defined $self->{'frame'}->{'insertion_point'}){
					$self->{$self->{'frame'}->{'FrameId'}.$param.'_ctrl'}->SetInsertionPoint(
							$self->{'frame'}->{'insertion_point'});
				}
				delete $self->{'frame'}->{'focus'};
				delete $self->{'frame'}->{'insertion_point'};
			}		
			if($self->{'frame'}->{'Data'}->{$param}->{'ControlType'}->{'Disabled'}){
					$self->{$self->{'frame'}->{'FrameId'}.$param.'_ctrl'}->Enable(0);	
			}
			
			unless(Wx::wxX11){
				$self->{$self->{'frame'}->{'FrameId'}.$param.'_ctrl'}->SetToolTip($self->{'frame'}->
						{'Data'}->{$param}->{'Description'});
			}
			if($type eq 'booleanType'){
				# add a dummy 
				$self->{'paramsizer'}->Add($self->{$self->{'frame'}->{'FrameId'}.$param.'_ctrl'},0, wxEXPAND | 
										wxALIGN_CENTER_VERTICAL | wxLEFT,$checkbox_distance);
				$self->{'paramsizer'}->Add(Wx::BoxSizer->new(wxVERTICAL));
			}
			else{
				$self->{'paramsizer'}->Add($self->{$self->{'frame'}->{'FrameId'}.$param.'_ctrl'},0, wxEXPAND | 
										wxALIGN_CENTER_VERTICAL);
			}
		}
		else{
			$self->{$param.'_box'} = Wx::StaticBox->new($self,-1,
				$self->{'frame'}->{'Data'}->{$param}->{'Label'});		
			
			$self->{$param.'_sizer'} =  Wx::StaticBoxSizer->new($self->{$param.'_box'},wxVERTICAL); 
			
			if($type eq 'datavolumesType'){
				require SAPDB::Install::Window::VolumesOneMode;
				$self->{$self->{'frame'}->{'FrameId'}.$param.'_ctrl'} = SAPDB::Install::Window::VolumesOneMode->new($self,-1);
			}
			elsif($type eq 'logvolumesType'){
				require SAPDB::Install::Window::VolumesOneMode;
				$self->{$self->{'frame'}->{'FrameId'}.$param.'_ctrl'} = SAPDB::Install::Window::VolumesOneMode->new($self,-1);
			}
			elsif($type eq 'simplevolumesType'){
				require SAPDB::Install::Window::VolumesSimple;
				$self->{$self->{'frame'}->{'FrameId'}.$param.'_ctrl'} = SAPDB::Install::Window::VolumesSimple->new($self,-1);
			}
			elsif($type eq 'mediumvolumesType'){
				require SAPDB::Install::Window::VolumesMedium;
				$self->{$self->{'frame'}->{'FrameId'}.$param.'_ctrl'} = SAPDB::Install::Window::VolumesMedium->new($self,-1);
			}
			elsif($type eq 'dbuserType'){
				require SAPDB::Install::Window::DBUserSimple;
				$self->{$self->{'frame'}->{'FrameId'}.$param.'_ctrl'} = SAPDB::Install::Window::DBUserSimple->new($self,-1);
			}
			elsif($type eq 'adminusersType'){
				require SAPDB::Install::Window::DBUser;
				$self->{$self->{'frame'}->{'FrameId'}.$param.'_ctrl'} = SAPDB::Install::Window::DBUser->new($self,-1);
			}
			elsif($type eq 'loggingType'){
				require SAPDB::Install::Window::LogMngmnt;
				$self->{$self->{'frame'}->{'FrameId'}.$param.'_ctrl'} = SAPDB::Install::Window::LogMngmnt->new($self,-1);
			}
                        else{
                          #This is a fatal ERROR:
                          #print ">>Unknown control type: '$type'\n";
                        }
			$self->{$self->{'frame'}->{'FrameId'}.$param.'_ctrl'}->SetValue(&{$self->{'frame'}->{'Data'}->{$param}->{'Getter'}}());
			$self->{$param.'_sizer'}->Add($self->{$self->{'frame'}->{'FrameId'}.$param.'_ctrl'},1,wxEXPAND | wxALL,10);
			$self->{$param.'_sizer'}->CalcMin;
			$self->{'topboxsizer'}->Add($self->{$param.'_sizer'},1,wxALIGN_TOP | wxEXPAND | wxLEFT | wxRIGHT | wxBOTTOM,10);
		}
	}
	if($flat_params > 0){
		$self->{'paramsizer'}->CalcMin;
		$self->{'topboxsizer'}->Prepend($self->{'paramsizer'},0,wxEXPAND | wxALL | wxALIGN_CENTER_VERTICAL, 10) ;
	}
	$self->{'topboxsizer'}->CalcMin;
	$self->GetWizard->LayoutDlg;
}

sub cleanFrame{
	my ($self) = @_;
	foreach my $param (@{$self->{'frame'}->{'params'}}){
		if($self->{'frame'}->{'Data'}->{$param}->{'Type'} !~ /COMPOUND$/){
			if(defined $self->{$self->{'frame'}->{'FrameId'}.$param.'_text'}){
				$self->{'paramsizer'}->Remove($self->{$self->{'frame'}->{'FrameId'}.$param.'_text'})
					 if defined $self->{'paramsizer'};
				$self->{$self->{'frame'}->{'FrameId'}.$param.'_text'}->Destroy;
				delete $self->{$self->{'frame'}->{'FrameId'}.$param.'_text'};
			}
			if(defined $self->{$self->{'frame'}->{'FrameId'}.$param.'_ctrl'}){
				$self->{'paramsizer'}->Remove($self->{$self->{'frame'}->{'FrameId'}.$param.'_ctrl'})
						 if defined $self->{'paramsizer'};
				$self->{$self->{'frame'}->{'FrameId'}.$param.'_ctrl'}->Destroy;
				delete $self->{$self->{'frame'}->{'FrameId'}.$param.'_ctrl'};
			}
		}
		else{
			$self->{'topboxsizer'}->Remove($self->{$param.'_sizer'});
			$self->{$param.'_box'}->Destroy;
			delete $self->{$param.'_box'};
			$self->{$self->{'frame'}->{'FrameId'}.$param.'_ctrl'}->Destroy;
			delete $self->{$self->{'frame'}->{'FrameId'}.$param.'_ctrl'};
		}
		
	}
	if(defined $self->{'paramsizer'}){
			$self->{'topboxsizer'}->Remove($self->{'paramsizer'});
			delete $self->{'paramsizer'};
	}
}


sub frameVisible{
	my ($self,$visible) = @_;
	if($visible){
		$self->{'title'}->SetLabel($self->{'frame'}->{'Title'});
	        $self->{'topsizer'}->SetItemMinSize($self->{'title'},$self->{'title'}->GetSizeWH);
		my $flat_params = 0;
		foreach my $param (@{$self->{'frame'}->{'params'}}){
			$flat_params++ if($self->{'frame'}->{'Data'}->
					{$param}->{'Type'} !~ /COMPOUND$/);
		}
	
		if($flat_params > 0){
			$self->{'paramsizer'} = Wx::FlexGridSizer->new($#{@{$self->{'frame'}->{'params'}}} + 1,2,15,15);
			$self->{'paramsizer'}->AddGrowableCol(1);
		}
		else{
			if(defined $self->{'paramsizer'}){
				$self->{'topboxsizer'}->Remove($self->{'paramsizer'});
				delete $self->{'paramsizer'};
			}
		}
		foreach my $param (@{$self->{'frame'}->{'params'}}){
			if($self->{'frame'}->{'Data'}->{$param}->{'Type'} !~ /COMPOUND$/){
				if($self->{'frame'}->{'Data'}->{$param}->{'ControlType'}->{'id'}  eq 'booleanType'){
					$self->{'paramsizer'}->Add($self->{$self->{'frame'}->{'FrameId'}.$param.'_text'},0,wxALIGN_CENTER_VERTICAL | wxLEFT,$checkbox_distance);
					$self->{'paramsizer'}->Add(Wx::BoxSizer->new(wxVERTICAL));
				}
				else{
					$self->{'paramsizer'}->Add($self->{$self->{'frame'}->{'FrameId'}.$param.'_text'},0,wxALIGN_CENTER_VERTICAL);
					$self->{$self->{'frame'}->{'FrameId'}.$param.'_ctrl'}->SetSize([50,-1]);
					$self->{'paramsizer'}->Add($self->{$self->{'frame'}->{'FrameId'}.$param.'_ctrl'},0,
										wxEXPAND | wxALIGN_CENTER_VERTICAL);
				}
				$self->{$self->{'frame'}->{'FrameId'}.$param.'_text'}->Show(1);
				$self->{$self->{'frame'}->{'FrameId'}.$param.'_ctrl'}->Show(1);
			}
			else{	
				$self->{$self->{'frame'}->{'FrameId'}.$param.'_ctrl'}->Fit();
				$self->{$param.'_box'}->Show(1);
				$self->{$self->{'frame'}->{'FrameId'}.$param.'_ctrl'}->Show(1);
				$self->{$param.'_sizer'} = Wx::StaticBoxSizer->new($self->{$param.'_box'},wxVERTICAL);
				$self->{$param.'_sizer'}->Add($self->{$self->{'frame'}->{'FrameId'}.$param.'_ctrl'},1,wxEXPAND | wxALL,10);
				$self->{'topboxsizer'}->Add($self->{$param.'_sizer'},0,wxEXPAND | wxALL,10);
			}
		}
		if($flat_params > 0){
                        $self->{'paramsizer'}->CalcMin;
                        $self->{'topboxsizer'}->Prepend($self->{'paramsizer'},0,wxEXPAND | wxALL | wxALIGN_CENTER_VERTICAL, 10) ;
                }
		$self->{'topboxsizer'}->CalcMin;
		$self->GetWizard()->LayoutDlg;
	}
	else{
		foreach my $param (@{$self->{'frame'}->{'params'}}){
			if($self->{'frame'}->{'Data'}->{$param}->{'Type'} !~ /COMPOUND$/){
				$self->{'paramsizer'}->Remove($self->{$self->{'frame'}->{'FrameId'}.$param.'_text'});
				$self->{'paramsizer'}->Remove($self->{$self->{'frame'}->{'FrameId'}.$param.'_ctrl'});
				$self->{$self->{'frame'}->{'FrameId'}.$param.'_text'}->Show(0);
				$self->{$self->{'frame'}->{'FrameId'}.$param.'_ctrl'}->Show(0);
			}
			else{
				$self->{$param.'_box'}->Show(0);
				$self->{$self->{'frame'}->{'FrameId'}.$param.'_ctrl'}->Show(0);
				$self->{'topboxsizer'}->Remove($self->{$param.'_sizer'});
			}
		}
		if(defined $self->{'paramsizer'}){
			$self->{'topboxsizer'}->Remove($self->{'paramsizer'});
			delete $self->{'paramsizer'};
		}
		$self->Layout;
	}
	$self->Show(1);
}



sub Process{
	my ($self,$config) = @_;
	foreach my $param (@{$self->{'frame'}->{'params'}}){
		my $value = $self->{$self->{'frame'}->{'FrameId'}.$param.'_ctrl'}->GetValue;
		my $type = $self->{'frame'}->{'Data'}->{$param}->{'ControlType'}->{'id'};
		if($self->{'frame'}->{'Data'}->{$param}->{'ControlType'}->{'unit'} eq 'pages' ||
		   $type eq 'pagesType'){
			$value = int($value * 1024 / 8);
		}
		my ($rc,$short_text,$long_text) = (1,'','');
		if(defined $self->{'frame'}->{'Data'}->{$param}->{'Validator'}){
			 eval{
				($rc,$short_text,$long_text) = &{$self->{'frame'}->
					{'Data'}->{$param}->{'Validator'}}($value);
			 };
			 if($@){
				$self->SetError("validation error","$param: eval of validator failed: $@");
				return undef;
			 }
		}
		unless($rc){
			$self->SetError("Error: invalid parameter ".$self->{'frame'}->{'Data'}->
								{$param}->{'Label'},$short_text.":\n".$long_text);
			return undef;
		}
		&{$self->{'frame'}->{'Data'}->{$param}->{'Setter'}}($value);
	}
	my $new_frame = $config->{'container'}->Template->nextFrame($self->{'frame'});
	unless(defined $new_frame){
		return {
			'AddNewSteps' => [
								[{	'name' => 'Creating database instance, please wait',
								'dlg' => SAPDB::Install::Wizard::Dialog::CreateInstance->
											new($self->GetWizard,-1,[0,0],[10,10])},
												'Software installation']
							]
		};	
	}
	$self->frameVisible(0);
	$new_frame->{'prev_frame'} = $self->{'frame'};
	$self->{'frame'} = $new_frame;   
	$self->initFrame($config);
	return undef;
}


sub Back{
	my ($self,$config) = @_;
	if(defined $self->{'frame'}->{'prev_frame'}){
		$self->cleanFrame;
		$self->{'frame'} = $self->{'frame'}->{'prev_frame'};
		$self->frameVisible(1);
		$self->Layout;
		return undef;
	}
	return {'state' => 'edit'};	
	
}

sub Destroy{
	my ($self) = @_;
	if(defined $self->{'frame'}){
		while(1){
			$self->cleanFrame;
			$self->{'frame'} = $self->{'frame'}->{'prev_frame'};
			last unless defined $self->{'frame'};
		}
	}
	$self->SUPER::Destroy;
}


sub DESTROY{
	my ($self) = @_;
	print ">>>>>> destroying $self\n"
		if $SAPDB::Install::DEBUG::DESTRUCT;
}


1;
