#!/usr/bin/perl
#
# $Header: //sapdb/V76/c_00/b_32/sys/src/install/perl/SAPDB/Install/Wizard/Dialog/Template.pm#1 $
# $DateTime: 2006/07/06 16:50:32 $
# $Change: 130190 $
#
# Desc: start SAP DB installation/update/patch/instanceupgrade
# called by SDBINST/SDBUPD
#

package SAPDB::Install::Wizard::Dialog::Template;

use Wx::Event qw (EVT_RADIOBUTTON);
use Wx qw (
	wxTOP wxEXPAND wxALL wxALIGN_CENTER 
	wxALIGN_CENTER_HORIZONTAL wxVERTICAL);

use base SAPDB::Install::Wizard::Dialog::Base;
use SAPDB::Install::Templates2::InstTemplate;
use SAPDB::Install::ObjMngr;
use SAPDB::Install::Log;
use SAPDB::Install::Wizard::Dialog::Parameter;

sub new{
	my($this ) = shift->SUPER::new(@_);
	return $this;
}

sub init{
	my ($self,$config) = @_;
	unless(defined $config->{'container'}){
		$config->{'container'} = SAPDB::Install::ObjMngr->new();
	}
	unless(defined $config->{'container'}->Log){
		$config->{'container'}->Log(SAPDB::Install::Log->new());
	}	
	$config->{'container'}->Log->LogName("MaxDBInstanceCreation");
	my $first_long;
	
	my $rb_sizer = Wx::BoxSizer->new(wxVERTICAL);
	
	my $templateStruct =  SAPDB::Install::Templates2::TemplateList::createTemplateList();

	
	$self->{'sb'} = Wx::StaticBox->new($self,-1,'Description');
	
	$self->{'sb_sizer'} = Wx::StaticBoxSizer->new($self->{'sb'},wxVERTICAL);
	
	if($^O =~ /mswin/i){
		$self->{'text'} = Wx::StaticText->new($self,-1,'');
	}
	else{
		$self->{'text'} = SAPDB::Install::Control::StaticText->new($self,-1,[0,0],[-1,-1]);
		$self->{'text'}->{'auto_split'} = 1;
		$self->{'text'}->{'auto_resize'} = 0;
	}
	
	$self->{'sb_sizer'}->SetMinSize(150,250);
		
	$self->{'sb_sizer'}->Add($self->{'text'},1,wxEXPAND | wxALL,20);
	#EVT_SIZE($self->{'sb'},\&OnStaticBoxSize);
	my $index = 0;	
	
	foreach  my $section (@$templateStruct){
		my $label = $section->{'label'};
		my $section_sizer = Wx::BoxSizer->new(wxVERTICAL);
		my $templateDescriptions = $section->{'data'};
		foreach  my $templateDescription (@$templateDescriptions){
		   my $templateCode = $templateDescription->{"TemplateCode"};
		   $first_long = $templateDescription->{"LongDescription"} unless defined $first_long;
		   my $rb = Wx::RadioButton->new($self,-1,$templateDescription->{'ShortDescription'});
		   if($index++  == 0){
				$self->{'text'}->SetLabel($templateDescription->{"LongDescription"});
				$rb->SetValue(1);
				$self->{'selected'} = $templateDescription->{'ShortDescription'};
		   }
		   EVT_RADIOBUTTON($self,$rb->GetId,\&OnTemplateSelect);		   
		   $section_sizer->Add($rb,0,wxEXPAND | wxALL  | wxALIGN_CENTER,5) 	
		}
		$rb_sizer->Add($section_sizer,1,wxEXPAND | (wxALL) & (~wxTOP), 20);

	}
	my $sizer = Wx::GridSizer->new(1,2);
	$sizer->Add($rb_sizer,1,wxALIGN_CENTER_HORIZONTAL | wxALL ,20);
	$sizer->Add($self->{'sb_sizer'},1,wxEXPAND | wxALIGN_CENTER | wxALL ,20);
	$self->SetSizer($sizer);
	$self->SetAutoLayout(1);
	$self->Layout;
	return $config;
}


sub OnTemplateSelect{
	my ($self,$event) = @_;
	my $templateDescriptions =  SAPDB::Install::Templates2::TemplateList::createTemplateList();
	my @templates;
	my $templateStruct =  SAPDB::Install::Templates2::TemplateList::createTemplateList();
	my $rb_label = $self->FindWindow($event->GetId())->GetLabel; 
	foreach  my $section (@$templateStruct){
		my $label = $section->{"label"};
	    my $templateDescriptions = $section->{"data"};
		foreach  my $templateDescription (@$templateDescriptions){
			my $templateCode = $templateDescription->{"TemplateCode"};
			my $shortDesc = $templateDescription->{"ShortDescription"};
			if($shortDesc eq $rb_label){
				$self->{'text'}->SetLabel($templateDescription->{"LongDescription"});
				$self->Layout;
				$self->{'selected'} = $shortDesc; 
				last;
			}
		}
	}
}

sub OnStaticBoxSize{
	my ($sb,$event) = @_;
	$sb->Layout;
}


sub Process{
	my ($self,$config) = @_;
	my $templateStruct =  SAPDB::Install::Templates2::TemplateList::createTemplateList();
	foreach  my $section (@$templateStruct){
		my $label = $section->{"label"};
	    my $templateDescriptions = $section->{"data"};
		foreach  my $templateDescription (@$templateDescriptions){
		   my $templateCode = $templateDescription->{"TemplateCode"};
		   my $shortDesc = $templateDescription->{"ShortDescription"};
		   if($shortDesc eq $self->{'selected'}){
				$config->{'template_desc'} = $templateDescription;
				$config->{'container'}->Template(SAPDB::Install::Templates2::InstTemplate::new(undef,$templateCode));
 				my $log = $config->{'container'}->Log;
				$config->{'container'}->SetRefLog(\$log);
				$config->{'container'}->Template->setLogCallback(
					sub{
						my ($text) = @_;
						$log->SetMsg($text);
					}
				);
				last;
	 
			}	
		}
	}
		
	$config->{'instmode'} = 'Custom';
	
	return {
				'AddNewSteps' => [
									[{
									#'name' => 'Customize Database',
									'name' => 'Database configuration ['.$config->{'template_desc'}->{'ShortDescription'}.']',
									'dlg' => SAPDB::Install::Wizard::Dialog::Parameter->new($self->GetWizard,-1)
									},undef]
								]
		   };
}


sub Back{
	my ($self,$config) = @_;
	$config->{'container'}->Template(undef);
	return 1;
}


sub Destroy{
	my ($self) = @_;
	if(defined $self->GetSizer){
		while($self->GetSizer->Remove(0)){}
		$self->{'text'}->Destroy;
		delete $self->{'text'};
		$self->{'sb'}->Destroy;
		delete $self->{'sb'};
	}
	$self->SUPER::Destroy;
}

sub DESTROY{
	my ($self) = @_;
	print ">>>> destroying $self\n"
		if $SAPDB::Install::DEBUG::DESTRUCT;
}	


1;
