#!/usr/bin/perl
#
# $Header: //sapdb/V76/c_00/b_32/sys/src/install/perl/SAPDB/Install/Wizard/Dialog/UninstProgress.pm#1 $
# $DateTime: 2006/07/06 16:50:32 $
# $Change: 130190 $
#
# Desc: 
#

package SAPDB::Install::Wizard::Dialog::UninstProgress;

use Wx qw (
	wxVERTICAL wxEXPAND wxALIGN_CENTER
	wxTOP wxLEFT wxICON_ERROR wxCENTRE);

use SAPDB::Install::Control::StaticText;
use SAPDB::Install::Window::Progress;
use SAPDB::Install::Uninstall::Start;
use SAPDB::Install::System;
use SAPDB::Install::Tools;
use SAPDB::Install::Trace;
use SAPDB::Install::Registry;
use SAPDB::Install::Uninstall::RemoveFilesInUse;


use base SAPDB::Install::Wizard::Dialog::Base;

sub new{
	my($this ) = shift->SUPER::new(@_);
	$this->{'progresswin'} = SAPDB::Install::Window::Progress->new($this,-1);
	$this->{'caption'} = Wx::StaticText->new($this,-1,'Uninstalling package',[0,0],[-1,-1]);
	my $top_sizer = Wx::BoxSizer->new(wxVERTICAL);
	$top_sizer->Add($this->{'caption'},0,wxTOP | wxLEFT ,60);
	$this->{'progresswin'}->SetSize([100,80]);
	$top_sizer->Add($this->{'progresswin'},0,wxEXPAND | wxALIGN_CENTER | wxTOP | wxLEFT, 60);
	$this->SetSizer($top_sizer);
	$this->SetAutoLayout(1);
	$this->Layout;
	return $this;
}



sub init{
	my ($self,$config) = @_;
	$self->GetWizard()->KillPrevSteps();
	return {'auto_next' => 1};
}

sub Process{
	my ($self,$config) = @_;

	if($self->{'retry'}){
		$self->GetWizard->SetRetryMode(0);
	}

	
	my $file_operations = 0;
	my $funcs = 0;
	
	my $eval_rc = eval{
		my @packages = @{$config->{'collector'}->GenRegPackages($config->{'container'}->InstReg)};
	
		foreach my $regpackobj (@packages){
			$config->{'container'}->SetRefInstReg(\$regpackobj);
			$regpackobj->evalScript;
			my @files = defined $regpackobj->FileList ? keys(%{$regpackobj->FileList}) : ();
			$file_operations += $#files + 1;
			if(defined $regpackobj->unregister){
				$funcs++;
			}
			if(defined $regpackobj->postuninstall){
				$funcs++;
			}
			if(defined $regpackobj->preuninstall){
					$funcs++;
			}
	
		}
		
		#$file_operations <<= 1;
		$file_operations += $funcs;
		$file_operations ++;
		
		$self->{'progresswin'}->init(my $dummy,1,$file_operations,"Software Uninstallation ...");
	
		foreach my $regpackobj (@packages){
			my $path = $regpackobj->Path;
			if($^O =~ /mswin/i){
				$path = getRealPathName($path);
			}
			$self->{'caption'}->SetLabel('Preparing package '.$regpackobj->DispName.' ['.$path.']');
			$self->{'progresswin'}->SetProgress($dummy,-1,'Calling preuninstall function');	
			if(defined $regpackobj->preuninstall){
				&{$regpackobj->preuninstall};
			}
			
		}
				
		
		
		$config->{'container'}->InstReg->DenyWrite(0); # scharf schalten
		my @unremoved_dirs;
		my @removed_dirs;
		my ($data,$prog) = readIndepPath();
	
		my ($sdb_uid, $sdba_gid, $sdb_user, $sdba_group);
 	
 		unless ($^O =~ /mswin/i){
 			my $globals = readGlobals();
 			if (exists $globals->{'SdbOwner'}){
 				$sdb_user = $globals->{'SdbOwner'};
 				$sdb_uid = getpwnam ($sdb_user);
 			}
 			if (exists $globals->{'SdbGroup'}){		
 				$sdba_group = $globals->{'SdbGroup'};
 				$sdba_gid = getgrnam ($sdba_group);
 			}
 		}

		foreach my $regpackobj (@packages){	
			$DEBUG && print "DEBUG: uninstallation of package ".$regpackobj->DispName.' ('.$regpackobj->Path.")\n";
			my $path = $regpackobj->Path;
			if($^O =~ /mswin/i){
				$path = getRealPathName($path);
			}
			$self->{'caption'}->SetLabel('Uninstalling package '.$regpackobj->DispName.' ['.$path.']');
			$regpackobj->setValid(0);
			if(defined $regpackobj->unregister){
				eval{
					$self->{'progresswin'}->SetProgress($dummy,-1,'Calling unregister function');
					&{$regpackobj->unregister};
				}; $@ and $self->SetError('Unregister of package '.$regpackobj->DispName.' failed!',"error in unregister():\n$@");
			}
		
			my ($busy,$delete_later) = SAPDB::Install::Uninstall::Start::uninstall($regpackobj,
					{
						'callback_progress' => sub {$self->{'progresswin'}->SetProgress(@_)},
						'callback_initprogress' => sub {$self->{'progresswin'}->init(@_)}
					}
			);
			my  @files_left = @$busy;
			my @undeleted_files;		
			if(defined $regpackobj->postuninstall){
					$self->{'progresswin'}->SetProgress($dummy,-1,'Calling postuninstall function');
					eval{
					@undeleted_files = &{$regpackobj->postuninstall};
				}; $@ and $self->SetError('Postuninstall script of package '.$regpackobj->DispName.' failed!',"error in postuninstall():\n$@");
			}
			
			if($#undeleted_files != -1){
				foreach my $file (@undeleted_files){
					push @files_left,$file if -f $file;
				}
			}
			
			if($^O =~ /mswin|hpux/i and $#files_left > -1){
				killUsedFiles($prog,@files_left);
			}	
			
			foreach my $file (@$delete_later){
				if( unlink($file)){
					$SAPDB::Install::Values::log->SetMsg("MSG: $file deleted\n")
						if defined $SAPDB::Install::Values::log;
				}
				else{
					$SAPDB::Install::Values::log->SetMsg("WRN: cannot delete $file: $!\n")
						if defined $SAPDB::Install::Values::log;
				}
			}
			
			if(defined $regpackobj->ParentPackage){
				my $parent_id = $regpackobj->ParentPackage;
				if(defined $parent_id){
					my $parent = $config->{'container'}->InstReg->getPackage(@$parent_id);
					if(defined $parent){
						my @subpackages = @{$parent->SubPackages};
						my @new_subpackages;
						foreach my $subpackage (@subpackages){
							unless($subpackage->[0] eq $regpackobj->Name and
							   $subpackage->[1] eq $regpackobj->Path){
								push @new_subpackages,$subpackage;							
							}	
						}
						$parent->SubPackages(\@new_subpackages);
						$parent->setValues();
						$SAPDB::Install::InstallRegistry::Package::SELFOBJ[$parent->ID] = undef; 	
					}
				}
			}
						
			# remove package from install registy
			$config->{'container'}->InstReg->removePackage($regpackobj->Name,$regpackobj->Path);
			# try to delete directories
			my $main_path_inside = 0; 
			my $drive = '';
		
			if(defined $regpackobj->Pathes){
				foreach my $dirref (@{$regpackobj->Pathes}){	
					if(normalizePath($dirref->{'value'}) eq normalizePath($regpackobj->Path)){
  						$main_path_inside = 1;
					}
					if($^O =~ /mswin/i){
						($drive) = ($dirref->{'value'} =~ /^([a-z]:)/i); 
					}
					chdir($drive.'/'); # change into root directory
					my ($rc,$ref_removed,$ref_unremoved) = removeEmptyDirs( $dirref->{'value'});
					@unremoved_dirs = union (\@unremoved_dirs,$ref_unremoved);  
					@removed_dirs = union (\@removed_dirs,$ref_removed);  

					my $subpath = $dirref->{'value'};

					while(1){
						$subpath =~ s/\/[^\/]+$//;
						if ($^O =~ /mswin/i){
							if ($subpath !~ /[\\\/]/){
								last;
							}
							opendir(SDH,$subpath);
							my @entries = grep {not /^\.{1,2}$/} readdir (SDH);
							closedir(SDH);
							last if $#entries > 3;
						}
						if(-d $subpath){
							my ($lrc,$lref_removed,$lref_unremoved) = 
							removeEmptyDirs($subpath,{'uid' => $sdb_uid, 'gid' => $sdba_gid});
 							@removed_dirs = union (\@removed_dirs,$lref_removed);
						}
						else{
							last;
						}
					}
				}
			}
			else{
				TraceMsg("member pathes not defined\n",4,\$DEBUG);
			}
			unless($main_path_inside){
				TraceMsg("main path not part of pathes\n",4,\$DEBUG);
				if($^O =~ /mswin/i){
						($drive) = ($regpackobj->Path =~ /^([a-z]:)/i); 
				}
				chdir($drive.'/'); # change into root directory
				my ($rc,$ref_removed,$ref_unremoved) = removeEmptyDirs($regpackobj->Path);
				
				my $subpath = $regpackobj->Path;
				
				while(1){
					$subpath =~ s/\/[^\/]+$//;
					if ($^O =~ /mswin/i){
						if ($subpath !~ /[\\\/]/){
							last;
						}
						opendir(SDH,$subpath);
						my @entries = grep {not /^\.{1,2}$/} readdir (SDH);
						closedir(SDH);
						last if $#entries > 3;
					}
					if(-d $subpath){
						my ($lrc,$lref_removed,$lref_unremoved) = 
						removeEmptyDirs($subpath,{'uid' => $sdb_uid, 'gid' => $sdba_gid});
 						@removed_dirs = union (\@removed_dirs,$lref_removed);
					}
					else{
						last;
					}
				}

				@unremoved_dirs = union (\@unremoved_dirs,$ref_unremoved);  
				@removed_dirs = union (\@removed_dirs,$ref_removed);  
			}
	
			
		}
		$self->{'progresswin'}->SetProgress($dummy,-1,'Finished');
		1;
	};

	if($@ =~ /fatal\serror/){
		return {'errstate' => 1};	
	}
	elsif($@){
		$self->SetError("Runtime Error: ",$@);
		return {'errstate' => 1};
	}
	
	unless($eval_rc == 1){
		return $eval_rc;
	}

	return 1;
}


sub Destroy{
	my ($self) = @_;
	if(defined $self->GetSizer){
		while($self->GetSizer->Remove(0)){}
	}
	if(defined $self->{'progresswin'}){
		$self->{'progresswin'}->Destroy;
		delete $self->{'progresswin'};
	}
	if(defined $self->{'caption'}){
		$self->{'caption'}->Destroy;
		delete $self->{'caption'};
	}
	$self->SUPER::Destroy;
}



sub DESTROY{
	my ($self) = @_;
	print ">>>> destroying $self\n"
		if $SAPDB::Install::DEBUG::DESTRUCT;
}


1;
